Package: phenoTest
Type: Package
Title: Tools to test association between gene expression and phenotype
        in a way that is efficient, structured, fast and scalable. We
        also provide tools to do GSEA (Gene set enrichment analysis)
        and copy number variation.
Version: 1.60.0
Date: 2013-05-29
Author: Evarist Planet
Maintainer: Evarist Planet <evarist.planet@epfl.ch>
Description: Tools to test correlation between gene expression and
        phenotype in a way that is efficient, structured, fast and
        scalable. GSEA is also provided.
License: GPL (>=2)
Depends: R (>= 3.6.0), Biobase, methods, annotate, Heatplus, BMA,
        ggplot2, Hmisc
Imports: survival, limma, gplots, Category, AnnotationDbi, hopach,
        biomaRt, GSEABase, genefilter, xtable, annotate, mgcv,
        hgu133a.db, ellipse
Suggests: GSEABase, GO.db
Enhances: parallel, org.Ce.eg.db, org.Mm.eg.db, org.Rn.eg.db,
        org.Hs.eg.db, org.Dm.eg.db
LazyLoad: yes
biocViews: Microarray, DifferentialExpression, MultipleComparison,
        Clustering, Classification
Config/pak/sysreqs: cmake make libicu-dev libpng-dev libuv1-dev
        libxml2-dev libssl-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:35:04 UTC
RemoteUrl: https://github.com/bioc/phenoTest
RemoteRef: RELEASE_3_23
RemoteSha: 540df6461fb5aca9093c0479501caf9153dded67
NeedsCompilation: no
Packaged: 2026-04-29 23:50:03 UTC; root
Built: R 4.6.0; ; 2026-04-30 01:25:34 UTC; unix

importClassesFrom(Biobase, ExpressionSet, eSet, VersionedBiobase, Versioned)

export(summary.gseaSignificanceSign, summary.gseaSignificanceVar, plot.gseaSignaturesSign,
       plot.gseaSignaturesVar, ExpressionPhenoTest, epheno2html,heatmapPhenoTest,
       ClusterPhenoTest, smoothCoxph, getEsPositions, findCopyNumber, genesInArea,
       gsea, summary.gseaData, plot.gseaData, gsea2html, gsea.selVars, gsea.selGsets, 
       eset2genelevel, write.html, pca)

exportClasses("epheno","gseaSignatures","gseaSignaturesSign","gseaSignaturesVar",
              "gseaSignificanceSign","gseaSignificanceVar")

exportMethods(gseaSignatures, gseaSignificance, getEs, getEsSim, getFcHr, dim, phenoNames,
              p.adjust.method, approach, phenoClass, show, getMeans, getPvals, getPostProbs,
              getSignif, getSummaryDif, getFc, getHr, logFcHr, export2CSV, "[",
              barplotSignatures, barplotSignifSignatures, pAdjust, getVars2test)

importClassesFrom(GSEABase, GeneSet, GeneSetCollection)

importMethodsFrom(AnnotationDbi, head)

importMethodsFrom(GSEABase, geneIds)

importMethodsFrom(genefilter, nsFilter, plot)

importFrom(Category, ttperm)

importFrom(Hmisc, cut2)

importFrom(biomaRt, getLDS, useMart)

importFrom(gplots, barplot2, greenred)

importFrom(hopach, distancematrix)

importFrom(limma, contrasts.fit, eBayes, lmFit)

importFrom(survival, Surv, coxph, survfit, pspline)

importFrom(xtable, xtable)

importFrom(mgcv, gam, s, predict.gam)

importFrom(hgu133a.db, hgu133aCHR, hgu133aCHRLOC, hgu133aCHRLOCEND)

importFrom(ellipse, ellipse)

pca.2d <- function(x, pc, pc.lab, group, group2, pair, names, ellipse=FALSE, main='', components= c(1, 2), legend=TRUE) {
  #control errors
  stopifnot(is(x, 'ExpressionSet'))
  if (!missing(group)) {
    stopifnot(group %in% colnames(pData(x)))
  }
  if (!missing(group2)) {
    stopifnot(group2 %in% colnames(pData(x)))
  }
  if (!missing(pair)) {
    stopifnot(pair %in% colnames(pData(x)))
  }
  if (!missing(names)) {
    stopifnot(names %in% colnames(pData(x)))
  }
  stopifnot(is(components, 'numeric') | is(components, 'integer'))
  stopifnot(all(!is.na(components)))
#
  colnames(pc) <- c('pc1', 'pc2')
  #get plot limits
  lim <- range(pc)
  d <- dist(lim) *.05
  lim <- c(lim[1]-d, lim[2]+d)  
#
  #parameters for plot
  add <- c()
  if (!missing(group)) {
    colour <- x[[group]]
    if (legend) legend.position <- 'top' else legend.position <- 'none'
  } else {
    colour <- 'black'
    legend.position <- 'none'
  }
  if (!missing(group2)) {
    shape <- x[[group2]]
  } else {
    shape <- ''
    add <- c(add, 'guides(shape=FALSE)')
  }
  if (!missing(pair)) {
    line <- x[[pair]]
    add <- c(add, "geom_line(aes(group=line), colour='grey')")
  } else {
    line <- ''
  }
  if (!missing(names)) {
    label <- x[[names]]
    add <- c(add, 'geom_text(aes(label=label), size=3)')
  } else {
    label <- ''
  }
  dat <- data.frame(pc, colour, shape, line, label)
#
  #ellipse with 95% CI of the mean of each group
  if (ellipse) {
    df.ell <- data.frame()
    for (i in 1:length(levels(colour))) {
      sel <- colour %in% levels(colour)[i]
      df.ell <- rbind(df.ell, cbind(as.data.frame(ellipse(x=cov(data.frame(pc[sel, ])/sqrt(nrow(pc))),
                                                             centre=colMeans(pc[sel, ]))),
                      colour=levels(colour)[i], shape=''))
    }
    add <- c(add, 'geom_path(data=df.ell, aes(x=pc1, y=pc2, group=colour))')
  }
#
  #plot
  tmp <- qplot(x=pc1, y=pc2, data=dat, xlab=pc.lab[1], ylab=pc.lab[2], colour=colour, shape=shape, main=main) + 
    geom_point() + coord_cartesian(xlim=lim, ylim=lim) + theme(legend.position=legend.position) +
      ggtitle(main)
  if (length(add)>0) for (i in 1:length(add)) tmp <- tmp + eval(parse(text=add[i]))
  tmp
}

## pca.multipleDim <- function(x, pc, pc.lab, group, group2, pair, names, ellipse, main, components) {
##   idx <- combn(1:length(components), 2)
##   grid.newpage()
##   pushViewport(viewport(layout = grid.layout(length(components), length(components)-1,
##                           heights = unit(c(0.5, rep(10/(length(components)-1), length(components)-1)), "null"))))
##   for (i in 1:ncol(idx)) {
##     pc.sel <- pc[, idx[, i]]
##     tmp <- pca.2d(x, pc.sel, pc.lab[idx[, i]], group, group2, pair, names, ellipse, main='', components[idx[, i]],
##                   legend=F)
##     print(tmp, vp=viewport(layout.pos.row = idx[1, i]+1, layout.pos.col=idx[2, i]-1))
##   }
##   #add title (we plot a blank plot)
##   vplayout <- function(x, y) viewport(layout.pos.row=x, layout.pos.col=y)
##   dummy <- ggplot(data.frame(main=main), aes(x=1,y=1,label=main)) + geom_text(size=5) +
##     theme(panel.background=element_blank(), panel.grid.minor=element_blank(), panel.grid.major=element_blank(),
##           axis.text.x=element_blank(), axis.text.y=element_blank(), axis.ticks=element_blank(),
##           axis.title.x=element_blank(), axis.title.y=element_blank())
##   print(dummy,vp=vplayout(1,1:(length(components)-1)))
##   #add legend
##   add <- c()
##   if (!missing(group2)) {
##     shape <- x[[group2]]
##   } else {
##     shape <- ''
##     add <- c(add, 'guides(shape=FALSE)')
##   }
##   dat <- data.frame(colour=x[[group]], shape=shape, dummy1=rep(.5, ncol(x)), dummy2=rep(.5, ncol(x)))
##   tmp <- ggplot(aes(x=dummy1, y=dummy2, colour=colour, shape=shape), data=dat) + geom_point()
##   tmp <- tmp + theme(panel.background=element_blank(), panel.grid.minor=element_blank(),
##                      panel.grid.major=element_blank(), axis.text.x=element_blank(), axis.text.y=element_blank(),
##                      axis.ticks=element_blank(), axis.title.x=element_blank(), axis.title.y=element_blank(),
##                      legend.position=c(.5, .5))
##   if (length(add)>0) for (i in 1:length(add)) tmp <- tmp + eval(parse(text=add[i]))
##   print(tmp, vp=viewport(layout.pos.row = length(components), layout.pos.col=1))
## }

pca <- function(x, group, group2, pair, names, ellipse=FALSE, main='', components= c(1, 2)) {
  #libs
  require(ggplot2)
#  require(gridExtra)
  #error control
  stopifnot(length(components)<=4)
  #calculate principal components
  pcdat <- prcomp(t(exprs(x)))
  pc <- data.frame(pcdat$x[, components])
  #pc.lab <- round(pcdat$sdev/sum(pcdat$sdev)*100,1)[components]
  pc.lab <- round(pcdat$sdev^2/sum(pcdat$sdev^2)*100,1)[components]
  pc.lab <- paste('PC', components, ' (', pc.lab, '%)', sep='')
  #choose 2d or 3d
  if (length(components)==2) {
    pca.2d(x, pc, pc.lab, group, group2, pair, names, ellipse, main, components)
  } else {
#    pca.multipleDim(x, pc, pc.lab, group, group2, pair, names, ellipse, main, components)
    stop('multiple dimension pca will not work until problems in gridExtra package get solved')
  }
}

setClass('epheno',contains='ExpressionSet',representation=representation(p.adjust.method="character",approach="character"))
setValidity("epheno", function(object){
  msg <- NULL
  pd <- pData(object)
  if (ncol(pData(object))==0) {
    msg <- 'You can not create an epheno object without pData'
  } else {
    if (sum(c('phenoName','phenoClass','phenoType','meanLabel','survTime') %in% colnames(pd))<5) {
      msg <- 'pData colnames have to be phenoName,phenoClass,phenoType,meanLabel and survTime'
    } else {
      if (any(pd[,'phenoClass']=='survival' & is.na(pd[,'survTime']))) {
        msg <- 'Survival variables must have time variables'
      } else {
        if (any(pd[,'phenoType']=='mean' & is.na(pd[,'meanLabel']))) msg <- 'Survival variables must have time variables'
      }
    }
  }
  if (is.null(msg)) TRUE else msg
})

setClass("gseaSignatures",contains="list",representation(es="numeric",es.sim="numeric",signature="numeric"))
setClass("gseaSignaturesSign",contains="list",representation(gseaSignatures="gseaSignatures",fc.hr="character",s="logical",es.sim.gam="matrix",test='character'))
setClass("gseaSignaturesVar",contains="list",representation(gseaSignatures="gseaSignaturesSign"))
setClass("gseaSignificanceSign",contains="list",representation(gseaSignificance="matrix",p.adjust.method="character"))
setClass("gseaSignificanceVar",contains="list",representation(gseaSignificance="gseaSignificanceSign"))
#setClass("gseaDataSign",contains="list",representation(gseaSignatures="gseaSignatures",fc.hr="character",s="logical",test='character',gseaSignificance="matrix",p.adjust.method="character"))
setClass("gseaData",contains="list",representation(gseaSignatures="gseaSignaturesSign",gseaSignificance="gseaSignificanceSign",gsetOrigin='character'))

ClusterPhenoTest <- function(x,cluster,vars2test,B=10^4,p.adjust.method='none') {
  if (length(cluster) != ncol(x)) stop('Dimensions of cluster and x do not match')
  if (is(cluster, 'character')) cluster <- factor(cluster)
  if (!is.null(vars2test$continuous)) {
    if (sum(vars2test$continuous %in% names(pData(x))) != length(vars2test$continuous)) stop('Invalid variable names in vars2test$continuous')
  }
  if (!is.null(vars2test$categorical)) {
    if (sum(vars2test$categorical %in% names(pData(x))) != length(vars2test$categorical)) stop('Invalid variable names in vars2test$categorical')
  }
  if (!is.null(vars2test$ordinal)) {
    if (sum(vars2test$ordinal %in% names(pData(x))) != length(vars2test$ordinal)) stop('Invalid variable names in vars2test$ordinal')
  }  
  if (!is.null(vars2test$survival)) {
    if (any(!(as.character(vars2test$survival) %in% names(pData(x))))) stop('Invalid variable names in vars2test$survival')
  }

  summaryDif <- pval <- vector("list",4); names(summaryDif) <- names(pval) <- c('cont','categ','ordinal','surv')
  if (!is.null(vars2test$continuous)) {
    mysummary <- sapply(vars2test$continuous,function(y) by(pData(x)[,y,drop=FALSE],cluster,summary),simplify=FALSE)
    pval$cont <- sapply(vars2test$continuous,function(y) kruskal.test(pData(x)[,y],g=cluster)$p.value)
  }

  if (!is.null(vars2test$categorical)) {
    summaryDif$categ <- sapply(vars2test$categorical,function(y) table(factor(pData(x)[,y]),cluster),simplify=FALSE)
    pval$categ <- sapply(summaryDif$categ,function(y) chisq.test(y,simulate.p.value=TRUE,B=B)$p.value)
  }

  if (!is.null(vars2test$ordinal)) {
    summaryDif$ordinal <- sapply(vars2test$ordinal,function(y) table(factor(pData(x)[,y]),cluster),simplify=FALSE)
    pval$ordinal <- sapply(vars2test$ordinal,function(y) kruskal.test(pData(x)[,y],g=cluster)$p.value)
  }

  if (!is.null(vars2test$survival)) {
    summaryDif$surv <- apply(vars2test$survival,1,function(y) coxph(Surv(pData(x)[,y['time']],pData(x)[,y['event']]) ~ cluster))
    pval$surv <- sapply(summaryDif$surv,function(y) summary(y)$logtest['pvalue'])
  }

  allpvals <- p.adjust(c(pval$cont,pval$categ,pval$ordinal,pval$surv),method=p.adjust.method)
  names(allpvals) <- c(vars2test$continuous,vars2test$categorical,vars2test$ordinal,as.character(vars2test$survival[,'time']))
  return(list(summaryDif=summaryDif,p.value=allpvals))
}

setGeneric("heatmapPhenoTest",function(x, signatures, vars2test, probes2genes=FALSE, filterVar, filteralpha=.05, distCol='pearson', nClust=2, distRow='cor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='average',equalize=FALSE,center=TRUE,col,survCol,heat.kaplan='both',...) standardGeneric("heatmapPhenoTest"))

setMethod("heatmapPhenoTest",signature(x="ExpressionSet",signatures="character"), function(x, signatures, vars2test, probes2genes=FALSE, filterVar, filteralpha=.05, distCol='pearson', nClust=2, distRow='cor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='average',equalize=FALSE,center=TRUE,col,survCol,heat.kaplan='both',...) {
  if (!missing(filterVar)) {  if (is.null(pData(x)[,filterVar])) stop(paste(filterVar,'not found in pData(x)')) }
  idnotfound <- sum(signatures %in% featureNames(x))
  if ((idnotfound>0) & (idnotfound<length(signatures))) {
     warning('Some of the identifiers in signatures are not present in featureNames(x)')
     signatures <- signatures[signatures %in% featureNames(x)]
   } else if (idnotfound==0) {
     stop('None of the specified identifiers could be found in featureNames(x)')
   }
  if (!is.null(vars2test$continuous)) {
    if (sum(vars2test$continuous %in% names(pData(x))) != length(vars2test$continuous)) stop('Invalid variable names in vars2test$continuous')
  }
  if (!is.null(vars2test$categorical)) {
    if (sum(vars2test$categorical %in% names(pData(x))) != length(vars2test$categorical)) stop('Invalid variable names in vars2test$categorical')
  }
  if (!is.null(vars2test$survival)) {
    if (any(!(as.character(vars2test$survival) %in% names(pData(x))))) stop('Invalid variable names in vars2test$survival')    
  }

#Select single probe per gene
x <- x[signatures,]
if (probes2genes==TRUE) {
  if (length(annotation(x))==0) {
    warning('annotation(x) is empty. Cannot select single probe per gene. All rows in exprs(x) were used')
  } else {
    x <- nsFilter(x,require.entrez=FALSE,remove.dupEntrez=TRUE,var.filter=FALSE)$eset
  }
}
  
#Clustering
if (center) { exprs(x) <- exprs(x) - rowMeans(exprs(x)) }
if (distCol=='pearson') {
  hc1 <- hclust(as.dist((1 - cor(exprs(x)))/2), method=linkage)
} else if (distCol=='spearman') {
  hc1 <- hclust(as.dist((1 - cor(exprs(x),method='spearman'))/2), method=linkage)
} else {
  hc1 <- hclust(dist(t(exprs(x)), method=distCol), method=linkage)
}
clus1 <- cutree(hc1,k=nClust)

#Test cluster vs phenotype association
pvals <- ClusterPhenoTest(x,cluster=clus1,vars2test=vars2test,p.adjust.method=p.adjust.method)$p.value
pvals <- ifelse(pvals<.0001,'(P<0.0001)',paste('(P=',round(pvals,4),')',sep=''))
  
#Heatmap
varlabels <- c(vars2test$continuous, vars2test$categorical, vars2test$ordinal, vars2test$survival[,'event'])
addvar <- pData(x)[,varlabels,drop=FALSE]
addvar[is.na(addvar)] <- 0
if (!is.data.frame(addvar)) { addvar <- matrix(addvar,ncol=1); colnames(addvar) <- varlabels }
colnames(addvar) <- paste(colnames(addvar),pvals)
sel <- sapply(addvar,function(x) any(x==TRUE)) #select only logic, or 0/1 variables
if (any(!sel)) {
  warning(paste('The following variables in vars2test will not be ploted because they are not of type logic: ',paste(varlabels[!sel],collapse=', '),sep=''))
  addvar <- addvar[,sel & !is.na(sel),drop=FALSE]
}
eset4plot <- exprs(x); colnames(eset4plot) <- NULL
if (nrow(eset4plot)>100) rownames(eset4plot) <- NULL
  
if (!missing(filterVar)) {
  sel <- !is.na(pData(x)[,filterVar])
  lm1 <- lmFit(x[signatures,sel], model.matrix(~ pData(x)[,filterVar]))
  lm2 <- contrasts.fit(lm1,coefficients=colnames(coef(lm1))[grep('filterVar',colnames(coef(lm1)))])
  eb <- eBayes(lm2)
  if (sum(eb$F.p.value<filteralpha)<2) stop('Could not perform heatmap. When filtering by significant genes less than two were kept.')
  eset4plot <- eset4plot[eb$F.p.value<filteralpha,]
}

cluscol <- gray(1-0.5*((1:max(clus1)) %% 2))
if (distRow=='pearson') distRow <- 'cor'
dx <- as.dist(hopach::as.matrix(distancematrix(eset4plot,d=distRow)))
rowden <- as.dendrogram(hclust(dx,method=linkage))

op <- par(no.readonly=TRUE)

if (heat.kaplan %in% c('heat','both')) {
  if (missing(col)) {
    mymax <- 2^max(abs(eset4plot))
    breaks <- log2(c(1,1.15,1.25,seq(1.5,4,length=20),5))
    if (mymax>4) breaks <- c(breaks,log2(mymax))
    breaks <- c(-breaks,breaks); breaks <- breaks[order(breaks)]; breaks <- unique(breaks)
    col <- greenred(length(breaks)-1)
    heatmap_plus(eset4plot,Rowv=rowden,Colv=as.dendrogram(hc1),clus=clus1,addvar=addvar,col=col,breaks=breaks,cluscol=cluscol,equalize=equalize)
  } else {
    heatmap_plus(eset4plot,Rowv=rowden,Colv=as.dendrogram(hc1),clus=clus1,addvar=addvar,col=col,cluscol=cluscol,equalize=equalize)
  }
  par(op)
}
  
if (heat.kaplan %in% c('kaplan','both')) {  
  if (!is.null(vars2test$survival)) {
    for (i in 1:nrow(vars2test$survival)) {
      tname <- vars2test$survival[i,'time']
      ename <- vars2test$survival[i,'event']
      s <- Surv(pData(x)[,tname], pData(x)[,ename]) ~ factor(clus1)
      coxph1 <- coxph(s)
      surv <- survfit(s)
      if (missing(survCol)) survCol <- 1:length(unique(clus1))
      plot(surv, col=survCol, ylab=paste('Survival (',ename,')',sep=''), xlab='Time', ...)
      pval <- 1-pchisq(2*diff(summary(coxph1)$loglik),df=1)
      hr <- round(exp(abs(coef(coxph1)))*sign(coef(coxph1)),3)
      hr <- ifelse(hr>10^3,Inf,hr); hr <- ifelse(hr< -10^3,-Inf,hr)
      text(.1*par('usr')[2],par('usr')[3]+.1*(par('usr')[4]-par('usr')[3]),paste('HR=',paste(hr,collapse=',')),pos=4)
      text(.1*par('usr')[2],par('usr')[3]+.05*(par('usr')[4]-par('usr')[3]),paste('P=',round(pval,4)),pos=4)
    }
  }
  par(op)
}

return(pvals)  
}
)

setMethod("heatmapPhenoTest",signature(x="ExpressionSet",signatures="missing"), function(x, signatures, vars2test, probes2genes=FALSE, filterVar, filteralpha=.05, distCol='pearson', nClust=2, distRow='cor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='average',equalize=FALSE,center=FALSE,col,survCol,heat.kaplan='both',...) {
  signatures <- featureNames(x)
  heatmapPhenoTest(x=x, signatures=signatures, vars2test=vars2test, probes2genes=probes2genes, filterVar=filterVar, filteralpha=filteralpha, distCol=distCol, nClust=nClust, distRow=distRow, p.adjust.method=p.adjust.method, simulate.p.value=simulate.p.value, B=B, linkage=linkage,equalize=equalize,center=center,col=col,survCol=survCol,toPDF=TRUE,heat.kaplan=heat.kaplan,...)
}
)

setMethod("heatmapPhenoTest",signature(x="ExpressionSet",signatures="list"), function(x, signatures, vars2test, probes2genes, filterVar, filteralpha=.05, distCol='manhattan', nClust=2, distRow='abscor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='complete',equalize=FALSE,center=FALSE,col,survCol,heat.kaplan='both',...) {
  if(!all(sapply(signatures,function(x) is.character(x)))) stop('All elements in argument signatures should be character vectors')
  for (i in 1:length(signatures)) {
    heatmapPhenoTest(x=x, signatures=signatures[[i]], vars2test=vars2test, probes2genes=probes2genes, filterVar=filterVar, distCol=distCol, nClust=nClust, distRow=distRow, p.adjust.method=p.adjust.method, simulate.p.value=simulate.p.value, B=B, linkage=linkage,equalize=equalize,center=center,col=col,survCol=survCol,heat.kaplan=heat.kaplan,...)
  }
}
)

setMethod("heatmapPhenoTest",signature(x="ExpressionSet",signatures="GeneSetCollection"), function(x, signatures, vars2test, probes2genes, filterVar, filteralpha=.05, distCol='manhattan', nClust=2, distRow='abscor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='complete',equalize=FALSE,center=FALSE,col,survCol,heat.kaplan='both',...) {
  signatures <- geneIds(signatures)
  heatmapPhenoTest(x=x, signatures=signatures, vars2test=vars2test, probes2genes=probes2genes, filterVar=filterVar, distCol=distCol, nClust=nClust, distRow=distRow, p.adjust.method=p.adjust.method, simulate.p.value=simulate.p.value, B=B, linkage=linkage,equalize=equalize,center=center,col=col,survCol=survCol,heat.kaplan=heat.kaplan,...)
}
)

setMethod("heatmapPhenoTest",signature(x="ExpressionSet",signatures="GeneSet"), function(x, signatures, vars2test, probes2genes, filterVar, filteralpha=.05, distCol='manhattan', nClust=2, distRow='abscor', p.adjust.method='none', simulate.p.value=FALSE, B=10^5, linkage='complete',equalize=FALSE,center=FALSE,col,survCol,heat.kaplan='both',...) {
  signatures <- geneIds(signatures)
  heatmapPhenoTest(x=x, signatures=signatures, vars2test=vars2test, probes2genes=probes2genes, filterVar=filterVar, distCol=distCol, nClust=nClust, distRow=distRow, p.adjust.method=p.adjust.method, simulate.p.value=simulate.p.value, B=B, linkage=linkage,equalize=equalize,center=center,col=col,survCol=survCol,heat.kaplan=heat.kaplan,...)
}
)

#####################
## 1 GET ES ES.SIM ##
#####################

tellNumPerm <- function(B,gsets) {
  if (is(gsets,  'character') | is(gsets, 'list') | is(gsets ,'GeneSetCollection')) {
    gsets.len <- ifelse(is(gsets, 'character'),1,length(gsets))
    numPerm <- ceiling(B/gsets.len)
    if (numPerm<50) {
      warning('You are using less than 50 permutations per gene set. Results will be very inaccurate. You should increse the number of permutations!\n')
    } else {
      cat(paste(gsets.len,'gene set(s) were provided and',B,'permutations were assigned,\n therefore ',numPerm,'permutations will be computed on each gene set.\n'))
    }
  }
}

preProcessX <- function(x,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,center=FALSE) {
  if (absVals) x <- abs(x)
  if (averageRepeats) {
    x <- tapply(x,names(x),mean)
  }
  x <- x[complete.cases(x) & sign(x)!=0]
  x <- x[!is.na(as.character(names(x)))]
  x <- x[order(x,decreasing=TRUE)]  
  if (logScale) x <- ifelse(x==0,0,log(abs(x)) * sign(x)) #else x <- ifelse(x==0,0,x)
  if (center) x <- x-median(x)
  return(x)
}

selSignatures <- function(x,signatures) {
  if (is(signatures, 'list')) {
    sel <- unlist(lapply(signatures,function(y) any(y %in% names(x))))
  } else {
    sel <- any(signatures %in% names(x))
  }      
  if (any(sel)) {
    signatures <- signatures[sel]
  } else {
    stop('No genes of the provided signatures are in names(x).')
  }
  signNotInX <- any(unlist(lapply(signatures, function(y) any(!(y %in% names(x))))))
  if (signNotInX) {
    warning('There is at least one signature that contains elements that are not in x!')
  }
  return(signatures)
}

checkGsetLen <- function(gsets,minGenes,maxGenes) {
  gsets.len <- unlist(lapply(gsets,length))
  if (any(gsets.len<minGenes)) warning(paste(sum(gsets.len<minGenes),' gene sets were removed due to having less than minGenes (',minGenes,') genes.',sep=''))
  if (any(gsets.len>maxGenes)) warning(paste(sum(gsets.len>maxGenes),' gene sets were removed due to having more than maxGenes (',maxGenes,') genes.',sep=''))
  gsets <- gsets[gsets.len>=minGenes & gsets.len<=maxGenes]
  if (length(gsets)==0) stop('After removing genes sets with less than minGenes and with more than maxGenes no gene sets left over.\nChange the size of your gene sets or change the values of parameters minGenes and maxGenes.')
  gsets
}

checkGsetSimmetry <- function(x,gsets,test,B) {
  #case1: fc/hr is strongly not centered
  prop.dif <- abs(diff(prop.table(table(x>0))))
  if (prop.dif > 0.1) {
    warning("The provided score is strongly not simmetric or is not centered on 0. You should set paremeter 'center' to TRUE.\n")
  } else {
    #case2: gene set is to large for a fc/hr that is softly not centered
    gsets.len <- unlist(lapply(gsets,length))
    probNotCentered <- (1-pnorm(0,mean(x),sd(x) * sqrt((1/gsets.len) * (1-gsets.len/length(x)) * (length(x)/(length(x)-1)))))
    if (any(probNotCentered>0.8)) { #check if x is centered
      maxSigLen <- sum(1-pnorm(0,mean(x),sd(x) * sqrt((1/(1:length(x))) * (1-(1:length(x))/length(x)) * (length(x)/(length(x)-1)))) < 0.8)
      warning("The provided score is not simmetric or is not centered on 0.\nYou can either:\n-Set parameter 'center' to TRUE\n-Use method 'wilcox'\n-Remove gene sets with more than ",maxSigLen," genes and run again.\n")
    }
  }
}

enrichmentScore <- function(fchr,sign) {
  nfchr <- length(fchr)
  p <- numeric(nfchr)
  p[] <- -1 / (nfchr - length(sign))
  absfchr <- abs(fchr[sign])
  p[sign] <- absfchr / sum(absfchr)
  cumsum(p)
}

getEsSimulations <- function(x,s,B,mc.cores) {
  myFun1 <- function(dummy) { return(which(sample(s,replace=FALSE))) } 
  myFun2 <- function(s.rand) {
    ans <- enrichmentScore(x,as.integer(s.rand))
    ans <- ans[abs(ans)==max(abs(ans))]
    return(ans)
  }
  s.rand <- lapply(vector("list", B),myFun1)
  if (mc.cores==1) {
    es.sim <- unlist(lapply(s.rand,myFun2))      
  } else {
    if ('parallel' %in% loadedNamespaces()) {
      es.sim <- unlist(parallel::mclapply(s.rand,myFun2,mc.set.seed=FALSE,mc.preschedule=TRUE,mc.cores=mc.cores))
    } else stop('parallel library has not been loaded!')
  }
  return(es.sim)
}

getEsSimGam <- function(x,gsets,B,mc.cores) {
  gset.n <- unique(unlist(lapply(gsets,length)))
  if (length(gset.n)>100) numpoints <- 100 else numpoints <- length(gset.n)
  #grid n
  gset.n.rg <- range(unique(gset.n))
  n.grid <- unique(round(seq(gset.n.rg[1],gset.n.rg[2],length.out=numpoints),0))
  n.grid.s <- lapply(as.list(n.grid),function(len) c(rep(TRUE,len),rep(FALSE,length(x)-len)))
  #es sim
  es.sim <- lapply(n.grid.s,function(y) getEsSimulations(x,s=y,B=ceiling(B/numpoints),mc.cores))
  names(es.sim) <- n.grid
  ans <- do.call(cbind,es.sim)
  return(ans)
}
    
getSourceData <- function(x,s,B=1000,mc.cores=1,test='perm',compute.es.sim=T) {
  s <- names(x) %in% unique(as.character(s))
  getEs <- function(x,s) {
    es <- enrichmentScore(x,as.integer(which(s))) 
    return(es)
  }
  es <- getEs(x,s)
  if (test=='perm' & compute.es.sim) es.sim <- getEsSimulations(x,s,B=B,mc.cores=mc.cores) else es.sim <- NA
  ans <- list(es=es,es.sim=es.sim,signature=which(s))
  return(ans)
}



###############
## 2 SUMMARY ##
###############

getNesGam <- function(escore,gsets.len,es.sim) {
  getPred <- function(i,escore,mygam.pos,mygam.neg,gsets.len) {
    if (escore[i]>0) {
      ans <- predict.gam(object=mygam.pos,newdata=data.frame(x.sel=gsets.len[i]))
    } else {
      ans <- predict.gam(object=mygam.neg,newdata=data.frame(x.nosel=gsets.len[i]))
    }
    return(ans)
  }
  y <- as.numeric(es.sim)
  x <- log(rep(as.numeric(colnames(es.sim)),each=nrow(es.sim)))
  gsets.len <- log(gsets.len)
  sel.pos <- y>0
  y.sel <- y[sel.pos]
  x.sel <- x[sel.pos]
  y.nosel <- y[!sel.pos]
  x.nosel <- x[!sel.pos]
  k <- min(10,length(unique(gsets.len))-1)
  mygam.pos <- gam(y.sel ~ s(x.sel,k=k,bs='cr')) #bs=cs is to force cubic splines
  mygam.neg <- gam(y.nosel ~ s(x.nosel,k=k,bs='cr'))
  escore.pred <- sapply(1:length(escore),function(i) getPred(i,escore,mygam.pos,mygam.neg,gsets.len))
##   #qc plot
##   plot(x,y)
##   lines(gsets.len[escore>0],escore.pred[escore>0],col=2); points(gsets.len[escore>0],escore.pred[escore>0],col=2)
##   lines(gsets.len[escore<0],escore.pred[escore<0],col=3); points(gsets.len[escore<0],escore.pred[escore<0],col=3)
  ans <- escore / abs(escore.pred)
  return(ans)
}

getNesSimGam <- function(es.sim) {
  es.pos.mean <- apply(es.sim,2,function(x) mean(x[x>0],na.rm=T))
  es.neg.mean <- apply(es.sim,2,function(x) mean(x[x<0],na.rm=T))
  es.pos.mean[is.nan(es.pos.mean)] <- 0
  es.neg.mean[is.nan(es.neg.mean)] <- 0
  sel.pos <- ifelse(es.sim>0,1,0)
  sel.neg <- ifelse(es.sim<0,1,0)
  es.sim.mean.pos <- (t(apply(sel.pos,1,function(x) as.numeric(x) * as.numeric(es.pos.mean))))
  es.sim.mean.neg <- (t(apply(sel.neg,1,function(x) as.numeric(x) * as.numeric(es.neg.mean))))
  es.sim.mean <- es.sim.mean.pos - es.sim.mean.neg
  nes.sim <- as.numeric(es.sim / es.sim.mean)
  return(nes.sim)
}

getNesSim <- function(es.sim) {
  nes.sim <- es.sim / ifelse(es.sim>0,mean(es.sim[es.sim>0]),-mean(es.sim[es.sim<0]))
  return(nes.sim)
}

getNesObs <- function(escore,es.sim) {
#  if (any(sign(es.sim)==sign(escore))) {
    es.sim.pos.mean <- mean(es.sim[sign(es.sim)==1])
    es.sim.neg.mean <- -mean(es.sim[sign(es.sim)==(-1)])
#    ms <- abs(mean(es.sim[sign(es.sim)==sign(escore)],na.rm=TRUE))
    ms <- ifelse(escore>0,es.sim.pos.mean,es.sim.neg.mean)
    nes <- escore / ms
#  } else {
#    nes <- NA
#  }
  return(nes)
}

getSummary <- function(es,es.sim,fchr,p.adjust.method='none',pval.comp.method='original',pval.smooth.tail=TRUE,signatures,test,fewGsets=TRUE) {
  getNesPval <- function(nes,pval.comp.method,pval.smooth.tail) { # nes.sim has been computed in getNesSim function
    if (!is.na(nes)) {
      if (pval.comp.method=='original') {
        if (sum(sign(nes.sim)==sign(nes),na.rm=TRUE)>1) {
          nes.sim.sign <- abs(nes.sim[sign(nes.sim)==sign(nes)])
          if (pval.smooth.tail) {
            myDens <- density(nes.sim.sign,from=0,to=max(abs(nes),nes.sim.sign)*2,n=2^13)
            myFun <- approxfun(myDens$x,myDens$y)
            mypval <- (1 - (integrate(myFun,0,abs(nes))$value / integrate(myFun,0,max(abs(nes),nes.sim.sign)*2)$value))
          } else {
            mypval <- sum(abs(nes.sim.sign) > abs(nes)) / length(nes.sim.sign)
          }
        } else {
          mypval <- NA
        }
      } else if (pval.comp.method=='signed') {
        if (pval.smooth.tail) {
          myDens <- density(nes.sim,from=min(nes,nes.sim)*2,to=max(nes,nes.sim)*2,n=2^13)
          myFun <- approxfun(myDens$x,myDens$y)
          if (sign(escore)==1) {
            tmp <- integrate(myFun,min(nes,nes.sim)*2,nes)$value
            mypval <- (1 - (tmp / (tmp+integrate(myFun,nes,max(nes,nes.sim)*2)$value)))
          } else {
            tmp <- integrate(myFun,nes,max(nes,nes.sim)*2)$value
            mypval <- (1 - (tmp / (tmp+integrate(myFun,min(nes,nes.sim)*2,nes)$value)))
          }
        } else {
          if (sign(escore)==1) mypval <- sum(nes.sim > nes) / length(nes.sim) else mypval <- sum(nes.sim < nes) / length(nes.sim)
        }
      } else {
        stop("pval.comp.method has to be 'original' or 'signed'")
      }
    } else {
      mypval <- NA
    }
    mypval <- ifelse(mypval<0,0,ifelse(mypval>1,1,mypval))
    return(mypval)
  }
  getFdr <- function(nes,pval.nes) {
    fdr <- vector('numeric',length=length(nes)); names(fdr) <- names(nes)
    nes.numeric <- as.numeric(nes)
    nes.numeric <- nes.numeric[!is.na(nes.numeric)]
    myFun <- function(x,y) { x / (sum(abs(nes.numeric[sign(nes.numeric)==sign(y)])>=abs(y)) / length(nes.numeric[sign(nes.numeric)==sign(y)])) }
    fdr <- mapply(function(x,y) myFun(x,y),x=pval.nes,y=nes)
    fdr <- ifelse(fdr<0,0,ifelse(fdr>1,1,fdr))
    return(fdr)
  }
  #
  getEsScore <- function(es) {
    escore <- range(es,na.rm=TRUE)
    escore <- escore[abs(escore)==max(abs(escore))][1]
  }
  gseaSignificance <- function(es,es.sim,pval.comp.method,pval.smooth.tail) {
    getEsPval <- function(escore,es.sim,pval.comp.method,pval.smooth.tail) {
      if (pval.comp.method=='original') {
        if (sum(sign(es.sim)==sign(escore),na.rm=TRUE)>1) {
          es.sim.sign <- abs(es.sim[sign(es.sim)==sign(escore)])
          if (pval.smooth.tail) {
            myDens <- density(es.sim.sign,from=0,to=max(abs(escore),es.sim.sign)*2,n=2056)
            myFun <- approxfun(myDens$x,myDens$y)
            mypval <- try((1 - (integrate(myFun,0,abs(escore))$value / integrate(myFun,0,max(abs(escore),es.sim.sign)*2)$value)),silent=TRUE)
            if (inherits(mypval, "try-error")) {
              mypval <- NA
            }
          } else {
            mypval <- sum(abs(es.sim.sign) > abs(escore)) / length(es.sim.sign)
          }
        } else {
          mypval <- NA
        }
      } else if (pval.comp.method=='signed') {
          if (pval.smooth.tail) {
            myDens <- density(es,from=min(escore,es.sim)*2,to=max(escore,es.sim)*2,n=2056)
            myFun <- approxfun(myDens$x,myDens$y)
            if (sign(escore)==1) {
              tmp <- integrate(myFun,min(escore,es.sim)*2,escore)$value
              mypval <- (1 - (tmp / (tmp+integrate(myFun,escore,max(escore,es.sim)*2)$value)))
            } else {
              tmp <- integrate(myFun,escore,max(escore,es.sim)*2)$value
              mypval <- (1 - (tmp / (tmp+integrate(myFun,min(escore,es.sim)*2,escore)$value)))
            }
          } else {
            if (sign(escore)==1) mypval <- sum(es.sim > escore) / length(es.sim) else mypval <- sum(es.sim < escore) / length(es.sim)
          }
      } else {
        stop("pval.comp.method has to be 'original' or 'signed'")
      }
      mypval <- ifelse(mypval<0,0,ifelse(mypval>1,1,mypval))
      return(mypval)
    }
    escore <- getEsScore(es)
    nes <- getNesObs(escore,es.sim)
    pval.nes <- getNesPval(nes,pval.comp.method,pval.smooth.tail)
    pval.es <- getEsPval(escore,es.sim,pval.comp.method,pval.smooth.tail)
    return(data.frame(es=escore,nes=nes,pval.es=pval.es,pval.nes=pval.nes))
  }
  #
  if (test!='perm') {
    if (test=='wilcox') {
      escore <- unlist(lapply(signatures,function(x) log2(mean(2^fchr[x]))))
      fchr.avg <- mean(fchr)
      pval <- unlist(lapply(signatures,function(x) max(c(wilcox.test(fchr[x],mu=0)$p.value,wilcox.test(fchr[x],mu=fchr.avg)$p.value))))
    } else if (test=='ttperm') {    
      myttperm <- function(x,y) {
        perm <- ttperm(matrix(x,nrow=1),factor(1:length(x) %in% y),B=1000)
        t.mine <- perm[[1]][,'statistic']
        t.perm <- unlist(lapply(perm[[2]], function(x) x[,'statistic']))
        ans <- sum((-abs(t.mine))<t.perm | abs(t.mine)>t.perm) / length(t.perm)
        return(ans)
      }
      escore <- unlist(lapply(es, getEsScore))
      pval <- mapply(function(x,y) myttperm(x,y),x=es,y=signatures)
    } else {
      stop('test has to be perm, wilcox or ttperm')
    }
    ans <- cbind(es=escore,pval)
    if (p.adjust.method!='none') ans[,'pval'] <- p.adjust(as.numeric(ans[,'pval']),p.adjust.method)
  } else {
    if (fewGsets) {
      nes.sim <- unlist(lapply(es.sim,getNesSim))
      ans <- mapply(function(x,y,z) gseaSignificance(x,y,pval.comp.method,pval.smooth.tail),x=es,y=es.sim)
      if (is(ans, 'list')) ans <- do.call(cbind,ans) #this line solves an error that appears on windows Windows Server 2003 R2 (32-bit) / x64 
      ans.tmp <- matrix(as.numeric(ans),ncol=ncol(ans)); colnames(ans.tmp) <- colnames(ans); rownames(ans.tmp) <- rownames(ans); ans <- ans.tmp
      fdr <- getFdr(ans['nes',],ans['pval.nes',])
      ans <- t(rbind(ans,fdr=fdr))
    } else {
      escore <- unlist(lapply(es,getEsScore))
      gsets.len <- unlist(lapply(signatures,length))
      nes <- getNesGam(escore,gsets.len,es.sim)
      nes.sim <- getNesSimGam(es.sim)
      pval.nes <- sapply(nes,function(x) getNesPval(x,pval.comp.method='original',pval.smooth.tail=pval.smooth.tail))
      fdr=getFdr(nes,pval.nes)
      ans <- data.frame(es=escore,nes,pval.nes,fdr)
    }
    if (p.adjust.method!='none') {
      if ('pval.es' %in% colnames(ans)) ans[,'pval.es'] <- p.adjust(as.numeric(ans[,'pval.es']),p.adjust.method)
      ans[,'pval.nes'] <- p.adjust(as.numeric(ans[,'pval.nes']),p.adjust.method)
    }
    ans <- ans
  }
  return(ans)
}



##################
## 3 MAKE PLOTS ##
##################

plotGSEA <- function(es.nes,fc.hr,s,mainTitle='',variable='',pvalfdr,p.adjust.method='',EsOrNes='ES',es.nes.ylim,test) {
  #functions
  plot1.perm <- function(x,myYlim,mainTitle,EsOrNes) {
    myDat <- approx(1:length(x),x,n=500)    
    plot(myDat$x,myDat$y,type='l',axes=FALSE,ylab=EsOrNes,xlab='',oma=c(0,0,0,0),main=mainTitle,ylim=myYlim,col='darkgreen',lwd=2)
    abline(h=0)
    axis(2)
  }
  plot1.wilcox <- function(fc.hr,s,mainTitle) {
    if (sum(s)>1) {
      xlim <- range(density(fc.hr[s])$x)[c(2,1)] #we invert the plot to show negatives first (like broad's gsea)
      ylim <- range(density(fc.hr[s])$y)
      plot(density(fc.hr[s]),xlim=xlim,ylim=ylim,col=3,main=mainTitle) 
    } else {
      plot(fc.hr[s],1,col=0)
    }
    abline(v=mean(fc.hr[s]),lty=2,col=3); abline(v=0,col=1,lty=2)
  }
  plot2 <- function(x,varLabel) {
    par(mar=c(1,4,1,1))
    x <- sort(x,decreasing=TRUE)
    myDat <- approx(1:length(x),x,n=300)
    plot(myDat$x,myDat$y,type='l',axes=FALSE,ylim=c(-1*max(abs(x)),max(abs(x))),ylab=paste(varLabel,'(log)'),xlab=''); abline(h=0); axis(2)
    if (any(myDat$y<0)) {
      abline(v=mean(myDat$x[myDat$y==min(myDat$y[myDat$y>0])],myDat$x[myDat$y==max(myDat$y[myDat$y<0])]),lty=2)
    }
    polygon(c(myDat$x,max(myDat$x),0),c(myDat$y,0,0), col=4, border = 4)
  }
  plot3 <- function(x) {
    par(mar=c(1,5.6,1,2.6))
    x.abs <- abs(x)
    x.abs.qua <- quantile(x.abs,(0:11)/11)
    x.abs.qua[1] <- 0
    x.qua <- c(-x.abs.qua[-1][(length(x.abs.qua)-1):1], x.abs.qua)
    x.idx <- ceiling((1:length(x))/50)
    x.small <- tapply(x,x.idx,mean)
    image(as.matrix(x.small),col=c(rgb(0,0.25,0.75,seq(0.1,1,length.out=11)[11:1]),rgb(1,0,0,seq(0.1,1,length.out=11))),breaks=x.qua,axes=FALSE)
  }
  plot4 <- function(s,subTitle) {
    par(mar=c(5,4,1,1))
    plot(NA,NA,xlim=c(1,length(s)),ylim=c(0,1),axes=FALSE,ylab='',xlab='Gene list rank',sub=subTitle,col='green',lwd=2)
    myTicks <- axTicks(1); myTicks[myTicks==0] <- 1; axis(1,lty=0,at=myTicks)
    if (sum(s)<50) myCols <- 1 else myCols <- densCols(which(s[1:length(s)]))
    abline(v=which(s[1:length(s)]),col=myCols)
  }
  #vars
  if (!missing(es.nes.ylim)) { myYlim <- es.nes.ylim } else { myYlim <- max(abs(range(es.nes))); myYlim <- c(-myYlim,myYlim) }
  if (EsOrNes=='ES') {
    mainTitle.es <- paste(EsOrNes,' plot / ',mainTitle,ifelse(is.na(pvalfdr),'(pval=NA)',ifelse(pvalfdr<0.001,'(pval<0.001)',paste('(pval=',round(pvalfdr,3),')',sep=''))),'*',sep=' ')
  } else {
    mainTitle.es <- paste(EsOrNes,' plot / ',mainTitle,ifelse(is.na(pvalfdr),'(fdr=NA)',ifelse(pvalfdr<0.001,'(fdr<0.001)',paste('(fdr=',round(pvalfdr,3),')',sep=''))),sep=' ')
  }
  if (EsOrNes=='ES') {
    subTitle <- ifelse(p.adjust.method=='none','(*) No pvalue adjustment method was used',paste('(*) pvalue adjustment method:',p.adjust.method))
  } else {
    subTitle <- ''
  }
  #plot
  def.par <- par(no.readonly = TRUE)
  par(mar=c(1,4,2,1))
  layout(c(1,2,3,4),heights=c(4,2,1,2))
  if (test=='perm') {
    plot1.perm(es.nes,myYlim,mainTitle=mainTitle.es,EsOrNes=EsOrNes)
  } else {
    plot1.wilcox(fc.hr,s,mainTitle.es)
  }
  plot2(fc.hr,variable)
  plot3(fc.hr)
  plot4(s,subTitle)
  par(def.par)
}

plotGseaPreprocess <- function(x,y,z,variable='',es.ylim,nes.ylim,test,es.nes,gsets.len) {
  if (test=='wilcox') es.nes <- 'es'
  if (is(z, 'list')) {
    gsl <- x[[1]]
    for (i in 1:length(z)) {
      fewGsets <- !is(x$es.sim.gam, 'matrix')
      es <- gsl[[i]][['es']]
      if (fewGsets) es.sim <- gsl[[i]][['es.sim']] else es.sim <- x$es.sim.gam
      fc.hr <- x[[2]]
      s <- names(fc.hr) %in% z[[i]]      
      myTitle <- paste(ifelse(missing(variable),'',paste('variable:',variable,' / ',sep='')),'signature:', names(z)[i],sep='')
      if (es.nes %in% c('es','both')) {
        #es plot
        if (test!='perm') pvalfdr <- y$summary[i,'pval'] else pvalfdr <- y$summary[i,][['pval.nes']]
        plotGSEA(es.nes=es,fc.hr=fc.hr,s=s,mainTitle=myTitle,variable=ifelse(missing(variable),'',variable),pvalfdr=pvalfdr,p.adjust.method=y[[2]],EsOrNes='ES',es.nes.ylim=es.ylim,test=test)
      }
      if (es.nes %in% c('nes','both')) {
        #nes plot
        if (test=='perm') {
          es.range <- range(es)
          escore <- es.range[abs(es.range)==max(abs(es.range))]
          if (fewGsets) {
            if (sum(sign(es.sim)==sign(escore))>2) {
              #nes <- es / abs(mean(es.sim[sign(es.sim)==sign(escore)],na.rm=TRUE))
              nes <- getNesObs(es,es.sim)
            } else {
              warning('Too few simulations with the same sign as ES were obtained in some signature(s). NAs will be assigned to nes.')
            }
          } else {
            gsets.len <- as.numeric(colnames(es.sim))
            nes <- es / abs(getNesGam(escore,gsets.len,es.sim=es.sim))
          }
          plotGSEA(es.nes=nes,fc.hr=fc.hr,s=s,mainTitle=myTitle,variable=ifelse(missing(variable),'',variable),pvalfdr=y[[1]][i,][['fdr']],p.adjust.method=y[[2]],EsOrNes='NES',es.nes.ylim=nes.ylim,test=test)
        }
      }
    }
  } else if (is(z, 'character')) {
    if (es.nes %in% c('es','both')) {
      #es plot
      es <- x[[1]][[1]][['es']]
      es.sim <- x[[1]][[1]][['es.sim']]
      fc.hr <- x[['fc.hr']]
      s <- names(fc.hr) %in% z
      myTitle <- paste(ifelse(missing(variable),'',paste('variable:',variable,sep='')))
      if (test!='perm') pvalfdr <- summary(y)[i,'pval'] else pvalfdr <- y[[1]][,'pval.es'][[1]]
      plotGSEA(es.nes=es,fc.hr=fc.hr,s=s,mainTitle=myTitle,variable=ifelse(missing(variable),'',variable),pvalfdr=y[[1]][,'pval.es'][[1]],p.adjust.method=y[[2]],EsOrNes='ES',es.nes.ylim=es.ylim,test=test)
    }
    if (es.nes %in% c('nes','both')) {      
      #nes plot
      if (test=='perm') {
        es.range <- range(es)
        escore <- es.range[abs(es.range)==max(abs(es.range))]
        if (fewGsets) {
          if (sum(sign(es.sim)==sign(escore))>2) {
            #nes <- es / abs(mean(es.sim[sign(es.sim)==sign(escore)],na.rm=TRUE))
            nes <- getNesObs(es,es.sim)
          } else {
            warning('No simulations with the same sign as ES were obtained. NES could not be computed!')
          }
        } else {
          nes <- getNesGam(es,es.sim)
        }
        plotGSEA(es.nes=nes,fc.hr=fc.hr,s=s,mainTitle=myTitle,variable=ifelse(missing(variable),'',variable),pvalfdr=y[[1]][,'fdr'][[1]],p.adjust.method=y[[2]],EsOrNes='NES',es.nes.ylim=nes.ylim,test=test)            
      }
    }
  } else {
    stop('signatures has to be of class list or character')
  }
}



###############
## 4 METHODS ##
###############

# 4.1 methods for gseaSignatures #
##################################

setGeneric("gseaSignatures",function (x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) standardGeneric("gseaSignatures"))
setGeneric("getEs",function (x) standardGeneric("getEs"))
setGeneric("getEsSim",function (x) standardGeneric("getEsSim"))
setGeneric("getNes",function (x) standardGeneric("getNes"))
setGeneric("getFcHr",function (x) standardGeneric("getFcHr"))

setMethod("gseaSignatures",signature(x="numeric",gsets="list"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
  x <- preProcessX(x,logScale=logScale,absVals=absVals,averageRepeats=averageRepeats,center=center)
  gsets <- selSignatures(x,gsets)
  gsets <- checkGsetLen(gsets,minGenes,maxGenes)
  if (test=='perm' & any(x>0) & any(x<0)) checkGsetSimmetry(x,gsets,test,B)
  numDifLen <- length(unique(unlist(lapply(gsets,length))))
  if (numDifLen>10 & test=='perm') {
    cat('The provided gene sets have more than 10 distinct lengths, therefore gam approximation will be used.\n')
    ans <- lapply(gsets,function(y) getSourceData(x=x,s=y,B=ceiling(B/length(gsets)),mc.cores=mc.cores,test=test,compute.es.sim=FALSE))
    es.sim.gam <- getEsSimGam(x,gsets,B,mc.cores)
  } else {
    tellNumPerm(B,gsets)
    ans <- lapply(gsets,function(y) getSourceData(x=x,s=y,B=ceiling(B/length(gsets)),mc.cores=mc.cores,test=test,compute.es.sim=TRUE))    
    es.sim.gam <- NA
  }
  ans <- new("gseaSignaturesSign",list(es.esSim=new("gseaSignatures",ans),fc.hr=x,signatures=gsets,test=test,es.sim.gam=es.sim.gam))
  return(ans)
 }
)

setMethod("gseaSignatures",signature(x="epheno",gsets="list"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   phenoTest2list <- function(x) {
      ans <- getSummaryDif(x)
      ans.list <- vector('list',ncol(ans)); names(ans.list) <- colnames(ans)
      for (i in 1:length(ans.list)) ans.list[[i]] <- ans[,i]
      return(ans.list)
   }
   x <- phenoTest2list(x)
   ans <- lapply(x,function(x) gseaSignatures(x,gsets,logScale,absVals=absVals,averageRepeats=averageRepeats,B,mc.cores,test,minGenes,maxGenes,center=center)) # call method(numeric,list)
   return(new("gseaSignaturesVar",ans))
 }
)

setMethod("gseaSignatures",signature(x="matrix",gsets="list"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   matrix2list <- function(x) {
      ans <- x[,grep('\\.HR$|\\.fc$',colnames(x),perl=TRUE),drop=FALSE]
      ans.list <- vector('list',ncol(ans)); names(ans.list) <- colnames(ans)
      for (i in 1:length(ans.list)) ans.list[[i]] <- ans[,i]
      return(ans.list)
   }
   x <- matrix2list(x)
   B <- ceiling(B/length(x))
   ans <- lapply(x,function(x) gseaSignatures(x,gsets,logScale,absVals=absVals,averageRepeats=averageRepeats,B,mc.cores,test,minGenes,maxGenes,center)) # call method(numeric,list)
   return(new("gseaSignaturesVar",ans))
 }
)

setMethod("gseaSignatures",signature(gsets="character"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   gsets <- list(signature=gsets)
   ans <- gseaSignatures(x,gsets,absVals=absVals,averageRepeats=averageRepeats,logScale,B,mc.cores,test,minGenes,maxGenes,center) #call method( ,list)
   return(ans)
 }
)

#method for signature=geneset has to be defined for every possible class of x (there seems to be an error with R identifying GeneSetCollection objects)
setMethod("gseaSignatures",signature(x="numeric",gsets="GeneSetCollection"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   gsets <- geneIds(gsets)
   ans <- gseaSignatures(x,gsets,absVals=absVals,averageRepeats=averageRepeats,logScale,B,mc.cores,test,minGenes,maxGenes,center) #call method( ,list)
   return(ans)
 }
)
setMethod("gseaSignatures",signature(x="epheno",gsets="GeneSetCollection"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   tmpgsets <- geneIds(gsets); names(tmpgsets) <- names(gsets); gsets <- tmpgsets
   ans <- gseaSignatures(x,gsets,absVals=absVals,averageRepeats=averageRepeats,logScale,B,mc.cores,test,minGenes,maxGenes,center) #call method( ,list)
   return(ans)
 }
)
setMethod("gseaSignatures",signature(x="matrix",gsets="GeneSetCollection"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   tmpgsets <- geneIds(gsets); names(tmpgsets) <- names(gsets); gsets <- tmpgsets   
   ans <- gseaSignatures(x,gsets,absVals=absVals,averageRepeats=averageRepeats,logScale,B,mc.cores,test,minGenes,maxGenes,center) #call method( ,list)
   return(ans)
 }
)

setMethod("gseaSignatures",signature(gsets="GeneSet"),
 function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test='perm',minGenes=10,maxGenes=500,center=FALSE) {
   gsets <- geneIds(gsets)
   ans <- gseaSignatures(x,gsets,absVals=absVals,averageRepeats=averageRepeats,logScale,B,mc.cores,test,minGenes,maxGenes,center) #call method( ,character)
   return(ans)
 }
)

setMethod("getEs",signature(x="gseaSignaturesSign"),
  function (x) {
    ans <- lapply(x[[1]],function(x) x[['es']])
    return(ans)
  }
)

setMethod("getEs",signature(x="gseaSignaturesVar"),
  function (x) {
    ans <- lapply(x,getEs)
    return(ans)
  }
)

setMethod("getEsSim",signature(x="gseaSignaturesSign"),
  function (x) {
    ans <- lapply(x[[1]],function(x) x[['es.sim']])
    return(ans)
  }
)

setMethod("getEsSim",signature(x="gseaSignaturesVar"),
  function (x) {
    ans <- lapply(x,getEsSim)
    return(ans)
  }
)

setMethod("getNes",signature(x="gseaSignaturesSign"),
  function (x) {
    myFun <- function(es,es.sim) {
      es.range <- range(es)
      escore <- es.range[abs(es.range)==max(abs(es.range))]
      ans <- es / abs(mean(es.sim[sign(es.sim)==sign(escore)],na.rm=TRUE))
      ans
    }
    es <- getEs(x)
    es.sim <- getEsSim(x)
    ans <- mapply(myFun, es=es,es.sim=es.sim)
    mynames <- colnames(ans)
    ans <- split(ans, 1:ncol(ans))
    names(ans) <- mynames
    return(ans)
  }
)

setMethod("getNes",signature(x="gseaSignaturesVar"),
  function (x) {
    ans <- lapply(x,getNes)
    return(ans)
  }
)
setMethod("getFcHr",signature(x="gseaSignaturesSign"),
  function (x) {
    return(x[['fc.hr']])
  }
)

setMethod("getFcHr",signature(x="gseaSignaturesVar"),
  function (x) {
    ans <- lapply(x,getFcHr)
    return(ans)
  }
)

setMethod("show",signature(object="gseaSignaturesSign"),
  function (object) {
    cat("Object of class 'gseaSignaturesSign'\n")
    cat("You can use the getEs, getNes, getEsSim and getFcHr methods to easily acces its data\n")
    cat("-The tested gene sets are:\n ",names(object[[1]]),'\n')
  }
)

setMethod("show",signature(object="gseaSignaturesVar"),
  function (object) {
    cat("Object of class 'gseaSignaturesVar'\n")
    cat("You can use the getEs, getNes, getEsSim and getFcHr methods to easily acces its data\n")
    cat("-The tested variables are:\n ",names(object),'\n')
    cat("-The tested gene sets (for each variable) are\n :",names(object[[1]][[1]]),'\n')
  }
)


# 4.2 methods for gseaSignificanceSign #
########################################

setGeneric("gseaSignificance",function (x,p.adjust.method='none',pval.comp.method='original',pval.smooth.tail=TRUE) standardGeneric("gseaSignificance"))

setMethod("gseaSignificance",signature(x="gseaSignaturesSign"),
 function(x,p.adjust.method='none',pval.comp.method='original',pval.smooth.tail=TRUE) {
  y <- x[[1]]
  fewGsets <- !is(x$es.sim.gam, 'matrix')
  es <- lapply(y,function(x) x[['es']])
  if (fewGsets) es.sim <- lapply(y,function(x) x[['es.sim']]) else es.sim <- x[['es.sim.gam']]
  signatures <- lapply(y,function(x) x[['signature']])
  test <- x[['test']]
  fchr <- x[['fc.hr']]
  ans <- cbind(n=unlist(lapply(x$s,length)),getSummary(es,es.sim,fchr,p.adjust.method=p.adjust.method,pval.comp.method,pval.smooth.tail,signatures,test,fewGsets))
  if (any(is.na(ans))) warning('Some NAs were produced due to not having enough simulations with the same sign as the observed ES.')
  return(new("gseaSignificanceSign",list(summary=as.matrix(ans),p.adjust.method=p.adjust.method,es.sim.gam=NA)))
 }
)

setMethod("gseaSignificance",signature(x="gseaSignaturesVar"),
 function(x,p.adjust.method='none',pval.comp.method='original',pval.smooth.tail=TRUE) {
  ans <- lapply(x,function(x) gseaSignificance(x,p.adjust.method,pval.comp.method,pval.smooth.tail))
  return(new("gseaSignificanceVar",ans))
 }
)

summary.gseaSignificanceSign <- function (object,...) {
  return(object[[1]])
}

summary.gseaSignificanceVar <- function (object,...) {
  ans <- lapply(object,function(object) object[['summary']])
  idx <- rep(seq(1,length(ans),1),nrow(ans[[1]]))
  idx <- idx[order(idx)]
  ans <- data.frame(variable=names(ans)[idx],geneSet=rownames(ans[[1]]),do.call('rbind',ans),row.names=NULL)
  return(ans)
}

setMethod("show",signature(object="gseaSignificanceSign"),
  function (object) {
    cat("Object of class 'gseaSignificanceSign'\n")
    cat("You can use the summary method to easily acces its data\n")
    cat("-The tested gene sets are:\n",rownames(object[[1]]),'\n')
    cat("-P adjust method:",object[[2]],'\n')
  }
)

setMethod("show",signature(object="gseaSignificanceVar"),
  function (object) {
    cat("Object of class 'gseaSignificanceVar'\n")
    cat("You can use the summary method to easily acces its data\n")
    cat("-The tested variables are:\n",names(object),'\n')
    cat("-The tested gene sets (for each variable) are:\n",rownames(object[[1]][[1]]),'\n')
    cat("-P adjust method:",object[[1]][[2]],'\n')
  }
)


# 4.3 plot methods #
####################

plot.gseaSignaturesSign <- function(x,gseaSignificance,es.ylim,nes.ylim,es.nes='both',...) {
  if (!is(gseaSignificance, "gseaSignificanceSign")) stop("if x is off class gseaSignaturesSign gseaSignificance has to be of class gseaSignificanceSign")
  if (es.nes!='nes' & missing(es.ylim)) {
    es.abs.max <- max(abs(summary(gseaSignificance)[,'es']),na.rm=TRUE)
    es.ylim <- c(-es.abs.max,es.abs.max)
  }
  test <- x['test']
  if (test!='perm') es.nes <- 'es'
  if (missing(nes.ylim) & missing(test)) {
    nes.abs.max <- max(abs(summary(gseaSignificance)[,'nes']),na.rm=TRUE)
    nes.ylim <- c(-nes.abs.max,nes.abs.max)
  }
  mysign <- x$signatures
  plotGseaPreprocess(x=x,y=gseaSignificance,z=mysign,es.ylim=es.ylim,nes.ylim=nes.ylim,test=test,es.nes=es.nes)
}

plot.gseaSignaturesVar <- function(x,gseaSignificance,es.ylim,nes.ylim,es.nes='both',...) {
  if (!is(gseaSignificance, "gseaSignificanceVar")) stop("if x is off class gseaSignaturesVar gseaSignificance has to be of class gseaSignificanceVar")
  if (es.nes!='nes' & missing(es.ylim)) {
    es.abs.max <- max(abs(summary(gseaSignificance)[,'es']),na.rm=TRUE)
    es.ylim <- c(-es.abs.max,es.abs.max)
  }
  test <- unique(unlist(lapply(x,function(x) x['test'])))
  if (missing(nes.ylim) & missing(test)) {
    nes.abs.max <-max(abs(summary(gseaSignificance)[,'nes']),na.rm=TRUE)
    nes.ylim <- c(-nes.abs.max,nes.abs.max)
  }
  mysign <- x[[1]]$signatures
  for (i in 1:length(x)) plotGseaPreprocess(x=x[[i]],y=gseaSignificance[[i]],z=mysign,variable=names(x)[i],es.ylim=es.ylim,nes.ylim=nes.ylim,test=test,es.nes=es.nes)
}


# 4.4 Methods for gseaData #
############################

gsea <- function(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,B=1000,mc.cores=1,test="perm",p.adjust.method="none",
                 pval.comp.method="original",pval.smooth.tail=TRUE,minGenes=10,maxGenes=500,center=FALSE) {
  sim <- gseaSignatures(x=x,gsets=gsets,logScale=logScale,absVals=absVals,averageRepeats=averageRepeats,B=B,mc.cores=mc.cores,test=test,minGenes=minGenes,maxGenes=maxGenes,center=center)
  pva <- gseaSignificance(sim,p.adjust.method=p.adjust.method,pval.comp.method=pval.comp.method,pval.smooth.tail=pval.smooth.tail)
  ans <- list(simulations=sim,significance=pva,gsetOrigin='User')
  ans <- new('gseaData',ans)
  return(ans)
}

setMethod("show",signature(object="gseaData"),
  function (object) {
    cat("Object of class 'gseaData'\n")
    cat("You can use the summary method to produce result summaries.\n")
    cat("You can use the getEs, getNes, getEsSim and getFcHr methods to easily acces its data.\n")
    cat("Gam approximation was",ifelse(usedGam(object),"used.","not used."),"\n")    
    cat("The tested variables are:\n ",paste(names(object[[1]]),collapse=', '),'\n')
    cat("The tested gene sets (for each variable) are:\n",paste(names(object[[1]][[1]][[1]]),collapse=', '),'\n')
  }
)

setMethod("getEs",signature(x="gseaData"),
  function (x) {
    getEs(x[[1]])
  }
)

setMethod("getEsSim",signature(x="gseaData"),
  function (x) {
    getEsSim(x[[1]])
  }
)

setMethod("getNes",signature(x="gseaData"),
  function (x) {
    getNes(x[[1]])
  }
)

setMethod("getFcHr",signature(x="gseaData"),
  function (x) {
    getFcHr(x[[1]])
  }
)

setGeneric("usedGam",function (x) standardGeneric("usedGam"))

setMethod("usedGam",signature(x="gseaData"),
 function(x) {
   if (is(x[[1]], 'gseaSignaturesSign')) {
     es.sim.gam <- x[[1]]$es.sim.gam
   } else {
     es.sim.gam <- x[[1]][[1]]$es.sim.gam
   }
   ans <- is(es.sim.gam, 'matrix')
   ans
 }
)

summary.gseaData <- function (object,...) {
  summary(object[[2]])
}

gsea.selGsets <- function(x,selGsets) {
  sim <- x[['simulations']]
  pva <- x[['significance']]
  if (is(sim, 'gseaSignaturesVar')) {
    sim <- lapply(sim,function(x) list(es.esSim=x$es.esSim[selGsets], fc.hr=x$fc.hr, signatures=x$signatures[selGsets], test=x$test))
    sim <- new("gseaSignaturesVar",sim)
  } else {
    sim <- list(es.esSim=sim$es.esSim[selGsets], fc.hr=sim$fc.hr, signatures=sim$signatures[selGsets], test=sim$test, es.sim.gam=sim$es.sim.gam)
    sim <- new("gseaSignaturesSign",sim)
  }
  if (is(pva, 'gseaSignificanceVar')) {
    pva <- lapply(pva,function(x) list(summary=x$summary[selGsets,,drop=F], p.adjust.method=x$p.adjust.method))
    pva <- new("gseaSignificanceVar",pva)
  } else {
    pva <- list(summary=pva$summary[selGsets,,drop=F], p.adjust.method=pva$p.adjust.method)
    pva <- new("gseaSignificanceSign",pva)
  }
  ans <- new('gseaData',list(simulations=sim, significance=pva))
  return(ans)
}

gsea.selVars <- function(x,selVars) {
  sim <- x[['simulations']]
  pva <- x[['significance']]
  sim <- sim[selVars]
  sim <- new("gseaSignaturesVar",sim)
  pva <- pva[selVars]
  pva <- new("gseaSignificanceVar",pva)
  ans <- new('gseaData',list(simulations=sim, significance=pva))
  return(ans)
}

plot.gseaData <- function(x,selGsets,selVars,...) {
  if (!missing(selGsets)) {
    if (is(x[[1]], 'gseaSignaturesVar')) signatureNames <- names(x[[1]][[1]][['es.esSim']]) else signatureNames <- names(x[[1]][['es.esSim']])
    stopifnot(!any(!selGsets %in% signatureNames))
    x <- gsea.selGsets(x,selGsets)
  }
  if (!missing(selVars) & is(x[[1]], 'gseaSignaturesVar')) {
    stopifnot(!any(!selVars %in% names(x[[1]])))
    x <- gsea.selVars(x,selVars)
  }
  if (is(x[[1]][[1]]$es.sim.gam, 'matrix')) plot(x[[1]],x[[2]],es.nes='nes',...) else plot(x[[1]],x[[2]],...)
}

write.html <- function(x, links, tiny.pic, tiny.pic.size=100, title='', file, digits=3) {
  stopifnot(is(x, 'data.frame'))
  if (missing(links)) links <- vector('list',ncol(x))
  if (missing(tiny.pic)) tiny.pic <- vector('list',ncol(x))  
  stopifnot(is(links, 'list'))
  stopifnot(is(tiny.pic, 'list'))
  stopifnot(length(links)==ncol(x))
  stopifnot(length(tiny.pic)==ncol(x))  
  stopifnot(!missing(file))

  #set factors as characters
  column.class <- unlist(lapply(x,class))
  for (j in 1:ncol(x)) {
    if (column.class[j]=='factor') x[,j] <- as.character(x[,j])
    if (column.class[j]=='numeric') x[,j] <- round(x[,j],digits=digits)
  }

  #header
  cat('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">\n',sep='',file=file,append=F)
  cat('<html>\n',file=file,append=T)
  cat('<body>\n',file=file,append=T)  
  cat(paste('<CAPTION ALIGN="top"><center><B>',title,'</B></center></CAPTION><BR>\n'),sep='',file=file,append=T)
  cat('<TABLE border=1>\n',file=file,append=T)

  #first row with column names
  cat('<TR>\n',file=file,append=T)
  for (j in 1:ncol(x)) {
    cat('<TH>',file=file,append=T)
    cat(colnames(x)[j],file=file,append=T)
    cat('</TH>\n',file=file,append=T)
  }
  cat('</TR>\n',file=file,append=T)     

  #rest of the data
  for (i in 1:nrow(x)) {
    cat('<TR>\n',file=file,append=T)
    for (j in 1:ncol(x)) {
      cat('<TD>',file=file,append=T)
      if (is.null(links[[j]]) & is.null(tiny.pic[[j]])) {
        cat(x[i,j],file=file,append=T)
      } else if (is.null(links[[j]]) & !is.null(tiny.pic[[j]])) { #
        cat(paste('<A HREF="',links[[j]][[i]],'"><img src="',tiny.pic[[j]][[i]],'" height="',tiny.pic.size,'" width="',tiny.pic.size,'" /></A>',sep=''),file=file,append=T)
      } else if (!is.null(links[[j]]) & is.null(tiny.pic[[j]])) {
        cat(paste('<A HREF="',links[[j]][[i]],'">',x[i,j],'</A>',sep=''),file=file,append=T)
      } else if (!is.null(links[[j]]) & !is.null(tiny.pic[[j]])) { #
        cat(paste('<A HREF="',links[[j]][[i]],'"><img src="',tiny.pic[[j]][[i]],'" height="',tiny.pic.size,'" width="',tiny.pic.size,'" /></A>',sep=''),file=file,append=T)
      }
      cat('</TD>\n',file=file,append=T)
    }
    cat('</TR>\n',file=file,append=T)
  }

  #close table
  cat('</TABLE>\n',file=file,append=T)
  cat('</body>\n',file=file,append=T)
  cat('</html>\n',file=file,append=T)  

  #add javascript to order & drag columns
  phenoTest:::sortDragHtmlTable(filename=file)
}

rowVar <- function(x,...) { return((rowMeans(x^2,...)-rowMeans(x,...)^2)*ncol(x)/(ncol(x)-1)) }
rowSD <- function(x,...) { return(sqrt(rowVar(x,...))) }

#
# FUNCTION THAT CREATES THE EPHENO OBJECT
#
ExpressionPhenoTest <- function (x,vars2test,adjustVars,p.adjust.method='BH',continuousCategories=3,mc.cores=1,approach='frequentist') {

mycoxph <- function(x,eset,coxSurvEvent,coxSurvTime,adjustVarsTxt,approach) {
  val <- try({
    if (adjustVarsTxt=='') myFormula <- "Surv(coxSurvTime, coxSurvEvent) ~ I(as.numeric(x))" else myFormula <- paste("Surv(coxSurvTime, coxSurvEvent) ~ ",adjustVarsTxt,"I(as.numeric(x))",sep=' + ')
    coxph1 <- coxph(eval(parse(text=myFormula)))
    sel <- length(coefficients(coxph1))
    eCoef <- exp((abs(coef(coxph1)[sel])))*sign(coef(coxph1)[sel])
    summaryDif <- eCoef
    if (approach=='frequentist') {
      tmp <- anova(coxph1)
      signif <- tmp[nrow(tmp),'Pr(>|Chi|)']
    } else if (approach=='bayesian') {
      if (adjustVarsTxt=='') {
        dat <- cbind(myVar=as.numeric(x))
      } else {
        dat <- cbind(myVar=as.numeric(x),pData(eset)[,adjustVars])
      }
      my.bic.surv <- bic.surv(x=dat, surv.t=coxSurvTime, cens=coxSurvEvent, OR=Inf)
      signif <- my.bic.surv$postprob[my.bic.surv$label=='myVar']
    }
  },silent=TRUE)
  if (inherits(val, "try-error")) {
    summaryDif <- NA; signif <- NA
#    warning('Some errors happened analyzing survival vars. NAs were produced.')
  }
  return(list(summaryDif,signif))
}

postprobBic <- function(exprs,formula) {
  bic1 <- BIC(lm(eval(parse(text=gsub('\\+ as.numeric\\(tmpVar\\)','',gsub('\\+ tmpVar','',formula))))))
  bic2 <- BIC(lm(eval(parse(text=formula))))
  bic <- c(bic1,bic2)
  ans <- (exp(-0.5 * (bic2 - min(bic))))/sum(exp(-0.5 * (bic - min(bic))))
##   bic <- exp(-0.5 * c(bic1,bic2))
##   ans <- bic[2] / sum(bic)
  return(ans)
}

stopifnot(is(x, 'ExpressionSet'))
stopifnot(is.numeric(exprs(x)))
stopifnot(is.numeric(mc.cores))
stopifnot(approach %in% c('frequentist','bayesian'))

if (mc.cores!=1 & approach=='bayesian') exprs.list <- by(exprs(x),as.numeric(cut(1:nrow(x),b=mc.cores)),function(x) x,simplify=FALSE)

if (!is.null(vars2test$continuous)) {
  if (sum(vars2test$continuous %in% names(pData(x))) != length(vars2test$continuous)) stop("Invalid variable names in vars2test$continuous")
  if (!any(!is.na(pData(x)[,vars2test$continuous]))) stop('Some variable in vars2test$continuous contains only missing values.')
}
if (!is.null(vars2test$ordinal)) {
  if (sum(vars2test$ordinal %in% names(pData(x))) != length(vars2test$ordinal)) stop("Invalid variable names in vars2test$ordinal")
  if (!any(!is.na(pData(x)[,vars2test$ordinal]))) stop('Some variable in vars2test$ordinal contains only missing values.')
}
if (!is.null(vars2test$categorical)) {
  if (sum(vars2test$categorical %in% names(pData(x))) != length(vars2test$categorical)) stop("Invalid variable names in vars2test$categorical")
  if (!any(!is.na(pData(x)[,vars2test$categorical]))) stop('Some variable in vars2test$categorical contains only missing values.')
}
if (!is.null(vars2test$survival)) {
  if (any(!(as.character(vars2test$survival) %in% names(pData(x))))) stop('Invalid variable names in vars2test$survival')  
  if (ncol(vars2test$survival)!=2) stop("vars2test$survival must have 2 columns")
  if (sum(colnames(vars2test$survival) %in% c('event','time'))<2) stop("colnames(vars2test$survival) must be 'event' and 'time' (in any order)")
}

if (continuousCategories<2) stop('continuousCategories can not be less than 2')
if (continuousCategories>4) stop('continuousCategories can not be bigger than 4')
if (!missing(adjustVars)) {
  if (!is(adjustVars,'character')) stop('adjustVars has to be of class character')
  if (!all(adjustVars %in% colnames(pData(x)))) stop('adjustVars is not in colnames(pData(x))')
}

phenoName <- NULL; phenoClass <- NULL; phenoClass.signif <- NULL; phenoType <- NULL; meanLabel <- NULL; survTime <- NULL

summaryDif <- signif <- vector("list", 4)
names(summaryDif) <- names(signif) <- c("cont", "categ", "surv", "ordi")
if (!is.null(vars2test$continuous)) {
  numContVars <- (continuousCategories*2)-1
  summaryDif$cont <- matrix(NA,nrow=nrow(x),ncol=(length(vars2test$continuous)*numContVars))
  signif$cont <- matrix(NA,nrow=nrow(x),ncol=length(vars2test$continuous))

  #epheno names
  phenoName <- c(phenoName,as.character(sapply(vars2test$continuous, function(x) rep(x,numContVars))))
  phenoClass <- c(phenoClass,rep('continuous',numContVars*length(vars2test$continuous)))
  phenoType <- c(phenoType,as.character(unlist(sapply(vars2test$continuous, function(x) c(rep('mean',continuousCategories),rep('summaryDif',continuousCategories-1))))))
  survTime <- c(survTime,rep(NA,numContVars*length(vars2test$continuous)))

  colnames(signif$cont) <- vars2test$continuous
  mynames <- character(0)
  for (i in 1:length(vars2test$continuous)) {
    cat(paste('\nPerforming analysis for continuous variable ',vars2test$continuous[i],' ',sep=''))
    colsel <- !is.na(pData(x)[,vars2test$continuous[i]])
    tmpVar <- pData(x)[colsel,vars2test$continuous[i]]
    
    if (missing(adjustVars)) {
      myFormula <- "~ tmpVar"
    } else {
      adjustVarsTxt <- paste(paste("pData(x)[colsel,'",adjustVars[!(adjustVars %in% vars2test$continuous[i])],"']",sep='',collapse=' + '))
      myFormula <- paste("~ tmpVar",adjustVarsTxt,sep=' + ')
    }
    design <- model.matrix(eval(parse(text=myFormula)))
    colnames(design)[1:2] <- c('Intercept',vars2test$continuous[i])
    lm1 <- lmFit(x[,colsel],design=design) #select arrays in the design matrix (e.g. NAs excluded)
    ncateg <- length(unique(pData(x)[,vars2test$continuous[i]]))
    if (any(is.na(pData(x)[,vars2test$continuous[i]]))) ncateg <- ncateg-1
    if (ncateg>continuousCategories) {
      categories <- cut2(pData(x)[,vars2test$continuous[i]],g=continuousCategories)
    } else {
      categories <- factor(pData(x)[,vars2test$continuous[i]])
    }

    #epheno names
    meanLabel <- c(meanLabel,levels(categories),rep(NA,length(levels(categories))-1))
    phenoClass.signif <- c(phenoClass.signif,'continuous')
    
    mynames <- c(mynames,paste(vars2test$continuous[i],levels(categories),sep='.'))
    mynames <- c(mynames,paste(vars2test$continuous[i],'fc',levels(categories)[2:length(levels(categories))],sep='.'))
    sel.mean <- ((i-1)*numContVars) + 1:continuousCategories
    summaryDif$cont[,sel.mean] <- t(apply(exprs(x),1,function(x) sapply(levels(categories),function(y) mean(x[categories %in% y]))))
    numitems <- (2*continuousCategories)-1
    sel.fc <- ((i-1)*numContVars+continuousCategories) + 1:(continuousCategories-1)
    summaryDif$cont[,sel.fc] <- t(apply(summaryDif$cont[,sel.mean],1,function(x) 2^(abs(x[-1]-x[1]))*ifelse(x[-1]-x[1]<0,-1,1)))

    if (approach=='frequentist') {
      eb <- eBayes(contrasts.fit(lm1,contrasts=c(0,1,rep(0,ncol(design)-2))))
      signif$cont[,i] <- as.numeric(eb$p.value)
    } else if (approach=='bayesian') {
      myFormula <- gsub('~','~ 1 +',paste('exprs',myFormula))
      if (mc.cores==1) {
        signif$cont[,i] <- apply(exprs(x[,colsel]),1,function(x) postprobBic(x,formula=myFormula))
      } else {
        if ('parallel' %in% loadedNamespaces()) {
          signif$cont[,i] <- unlist(parallel::mclapply(exprs.list,function(y) apply(y,1,function(z) postprobBic(z,formula=myFormula)),mc.cores=mc.cores))
        } else stop('parallel library has not been loaded!')
      }
    }
  }
  colnames(summaryDif$cont) <- mynames
}

if (!is.null(vars2test$ordinal)) {
  nc <- unlist(apply(data.frame(pData(x)[,vars2test$ordinal]),2,function(x) length(unique(x[!is.na(x)]))))
  names(nc) <- vars2test$ordinal
  fc <- nc-1
  ncCoef <- c(1,1+cumsum(nc))
  fcCoef <- c(0,cumsum(fc))

  #epheno names
  phenoName <- c(phenoName,as.character(unlist(sapply(vars2test$ordinal,function(y) rep(y,2* length(unique(pData(x)[,y][!is.na(pData(x)[,y])])) -1)))))
  phenoType <- c(phenoType,as.character(unlist(sapply(nc, function(x) c(rep('mean',x),rep('summaryDif',x-1))))))
  phenoClass <- c(phenoClass,rep('ordinal',sum(2*nc+-1)))
  survTime <- c(survTime,rep(NA,sum(2*nc+-1)))
  
  summaryDif$ordi <- matrix(NA,nrow=nrow(x),ncol=(sum(nc)+sum(fc)))
  signif$ordi <- matrix(NA,nrow=nrow(x),ncol=length(vars2test$ordinal))
  colnames(signif$ordi) <- vars2test$ordinal
  mynames <- character(0)
  for (i in 1:length(vars2test$ordinal)) {
    cat(paste('\nPerforming analysis for ordinal variable ',vars2test$ordinal[i],' ',sep=''))

    #epheno names
    myLevels <- levels(as.factor(as.character(pData(x)[,vars2test$ordinal[i]])))
    meanLabel <- c(meanLabel,myLevels,rep(NA,length(myLevels)-1))
    phenoClass.signif <- c(phenoClass.signif,'ordinal')

    colsel <- !is.na(pData(x)[,vars2test$ordinal[i]])
    tmpVar <- as.factor(pData(x)[colsel,vars2test$ordinal[i]])
    
    if (missing(adjustVars)) {
      myFormula <- "~ -1 + tmpVar"
      myFormula4p <- "~ 1 + as.numeric(tmpVar)"
    } else {
      adjustVarsTxt <- paste(paste("pData(x)[colsel,'",adjustVars[!(adjustVars %in% vars2test$ordinal[i])],"']",sep='',collapse=' + '))
      myFormula <- paste("~ -1 + tmpVar",adjustVarsTxt,sep=' + ')
      myFormula4p <- paste("~ 1 + as.numeric(tmpVar)",adjustVarsTxt,sep=' + ')
    }
    design <- model.matrix(eval(parse(text=myFormula)))
    design4p <- model.matrix(eval(parse(text=myFormula4p)))
    lm1 <- lmFit(x[,colsel],design=design) #to get means
    mycontrasts <- rbind(rep(-1,nlevels(tmpVar)-1),diag(nlevels(tmpVar)-1),matrix(0,nrow=ncol(design)-nlevels(tmpVar),ncol=nlevels(tmpVar)-1))
    lm2 <- contrasts.fit(lm1,contrasts=mycontrasts) #to get fc
    categories <- levels(as.factor(pData(x)[,vars2test$ordinal[i]][colsel]))
    mynames <- c(mynames,paste(vars2test$ordinal[i],categories,sep='.'))
    mynames <- c(mynames,paste(vars2test$ordinal[i],categories[2:length(categories)],'fc',sep='.'))
    summaryDif$ordi[,(ncCoef[i]+fcCoef[i]):(ncCoef[i]+fcCoef[i]+fc[i])] <- coef(lm1)[,1:length(levels(tmpVar))] #means
    summaryDif$ordi[,(ncCoef[i]+fcCoef[i]+nc[i]):(ncCoef[i]+fcCoef[i]+nc[i]+(fc[i]-1))] <- (2^abs(coef(lm2)))*ifelse(coef(lm2)<0,-1,1) #fc
    if (approach=='frequentist') {
      lm4p <- lmFit(x[,colsel],design=design4p)
      mycontrasts <- rep(0,ncol(design4p))
      mycontrasts[2] <- 1
      eb <- eBayes(contrasts.fit(lm4p,contrasts=mycontrasts)) #to get signifs
      signif$ordi[,i] <- eb$F.p.value
    } else if (approach=='bayesian') {
      myFormula <- gsub('-1','+1',paste('exprs',myFormula))
      if (mc.cores==1) {
        signif$ordi[,i] <- apply(exprs(x[,colsel]),1,function(x) postprobBic(x,formula=myFormula4p))
      } else {
        if ('parallel' %in% loadedNamespaces()) {
          signif$ordi[,i] <- unlist(parallel::mclapply(exprs.list,function(y) apply(y[,colsel],1,function(z) postprobBic(z,formula=myFormula4p)),mc.cores=mc.cores))
        } else stop('parallel library has not been loaded!')
      }
    }
  }
  colnames(summaryDif$ordi) <- mynames
}

if (!is.null(vars2test$categorical)) {
  nc <- unlist(apply(data.frame(pData(x)[,vars2test$categorical]),2,function(x) return(length(unique(x[!is.na(x) & x!=""])))))
  names(nc) <- vars2test$categorical
  fc <- nc-1
  ncCoef <- c(1,1+cumsum(nc))
  fcCoef <- c(0,cumsum(fc))

  #epheno names
  phenoName <- c(phenoName,as.character(unlist(sapply(vars2test$categorical,function(y) rep(y,2* length(unique(pData(x)[,y][!is.na(pData(x)[,y]) & pData(x)[,y]!=""])) -1)))))
  phenoType <- c(phenoType,as.character(unlist(sapply(nc, function(x) c(rep('mean',x),rep('summaryDif',x-1))))))
  phenoClass <- c(phenoClass,rep('categorical',sum(2*nc-1)))
  survTime <- c(survTime,rep(NA,sum(2*nc+-1)))
    
  summaryDif$categ <- matrix(NA,nrow=nrow(x),ncol=sum(nc)+sum(fc))
  signif$categ <- matrix(NA,nrow=nrow(x),ncol=sum(fc)); colnames(signif$categ) <- paste('tmp', 1:ncol(signif$categ))
  mynames <- character(0)
  for (i in 1:length(vars2test$categorical)) {
    cat(paste('\nPerforming analysis for categorical variable ',vars2test$categorical[i],' ',sep=''))

    #epheno names
    if (is.factor(pData(x)[,vars2test$categorical[i]])) {
      tmpVar <- pData(x)[,vars2test$categorical[i]]
    } else {
      tmpVar <- as.factor(as.character(pData(x)[,vars2test$categorical[i]]))
    }
    myLevels <- levels(tmpVar)
    myLevels <- myLevels[myLevels!='']
    meanLabel <- c(meanLabel,myLevels,rep(NA,length(myLevels)-1))
    phenoClass.signif <- c(phenoClass.signif,rep('categorical', fc[i]))

    colsel <- !is.na(pData(x)[,vars2test$categorical[i]]) & pData(x)[,vars2test$categorical[i]]!=''
    tmpVar <- tmpVar[colsel]
    if (missing(adjustVars)) {
      myFormula <- "~ -1 + tmpVar"
    } else {
      adjustVarsTxt <- paste(paste("pData(x)[colsel,'",adjustVars[!(adjustVars %in% vars2test$categorical[i])],"']",sep='',collapse=' + '))
      myFormula <- paste("~ -1 + tmpVar",adjustVarsTxt,sep=' + ')
    }
    design <- model.matrix(eval(parse(text=myFormula)))
    lm1 <- lmFit(x[,colsel],design=design) #select arrays in the design matrix (e.g. NAs excluded)
    if (approach=='frequentist') {
      mycontrasts <- rbind(rep(-1,nlevels(tmpVar)-1),diag(nlevels(tmpVar)-1),matrix(0,nrow=ncol(design)-nlevels(tmpVar),ncol=nlevels(tmpVar)-1))
      eb <- eBayes(contrasts.fit(lm1,contrasts=mycontrasts))
      tmpSignif <- eb$p.value
    } else if (approach=='bayesian') {
      myFormula <- gsub('-1','+1',paste('exprs',myFormula))
      if (mc.cores==1) {
        signif$categ[,i] <- apply(y[,colsel],1,function(z) postprobBic(z,formula=myFormula))
##         tmpSignif <- matrix(nrow=nrow(x), ncol=length(levels(tmpVar))-1)
##         for (j in 2:length(levels(tmpVar))) {
##           sel <- tmpVar %in% levels(tmpVar)[c(1, j)]
##           tmpSignif[,i] <- apply(exprs(x[,colsel][, sel]),1,function(x) postprobBic(x,formula=myFormula))
##         }
      } else {
        if ('parallel' %in% loadedNamespaces()) {
          signif$categ[,i] <- unlist(parallel::mclapply(exprs.list,function(y) apply(y[,colsel],1,function(z) postprobBic(z,formula=myFormula)),mc.cores=mc.cores))
        } else stop('parallel library has not been loaded!')
      }
    }
    tmpCoef <- coef(lm1)[,1:(length(levels(tmpVar)))]
    colnames(tmpCoef) <- gsub('tmpVar','',colnames(tmpCoef))
    summaryDif$categ[,(ncCoef[i]+fcCoef[i]):(ncCoef[i]+fcCoef[i]+fc[i])] <- tmpCoef
    mynames <- c(mynames,paste(vars2test$categorical[i],colnames(tmpCoef),sep='.'))
    mynames <- c(mynames,paste(vars2test$categorical[i],colnames(tmpCoef)[-1],'fc',sep='.'))
    refCol <- summaryDif$categ[,ncCoef[i]+fcCoef[i]]
    selCol <- summaryDif$categ[,(ncCoef[i]+fcCoef[i]+1):(ncCoef[i]+fcCoef[i]+ncol(tmpCoef)-1)]
    summaryDif$categ[,(ncCoef[i]+fcCoef[i]+nc[i]):(ncCoef[i]+fcCoef[i]+nc[i]+(fc[i]-1))] <- 2^(abs(selCol-refCol))*ifelse(selCol-refCol<0,-1,1)
    signif$categ[, (fcCoef[i]+1):fcCoef[i+1]] <- tmpSignif
  }
  colnames(summaryDif$categ) <- mynames
  colnames(signif$categ) <- gsub('.fc$', '.pval', mynames[grepl('.fc$', mynames)])
}

if (!is.null(vars2test$survival)) {
  summaryDif$surv <- signif$surv <- matrix(NA,nrow=nrow(x),ncol=nrow(vars2test$survival))
  colnames(summaryDif$surv) <- paste(as.character(vars2test$survival[,'event']),'HR',sep='.')
  colnames(signif$surv) <- as.character(vars2test$survival[,'event'])
  for (i in 1:nrow(vars2test$survival)) {
    phenoName <- c(phenoName,vars2test$survival[i,"event"])
    phenoClass <- c(phenoClass,"survival")
    phenoClass.signif <- c(phenoClass.signif,'survival')
    phenoType <- c(phenoType,"summaryDif")
    meanLabel <- c(meanLabel,NA)
    survTime <-  c(survTime,vars2test$survival[i,"time"])

    cat(paste('\nPerforming analysis for survival variable ',vars2test$survival[i],' ',sep=''))
    exprsX <- exprs(x)/rowSD(exprs(x),na.rm=TRUE)
    coxSurvEvent <- pData(x)[,vars2test$survival[i,"event"]]
    coxSurvTime <- pData(x)[,vars2test$survival[i,"time"]]
    if (mc.cores==1) {
      exprs.list <- list(exprsX)
    } else {
      exprs.list <- by(exprsX,as.numeric(cut(1:nrow(x),b=mc.cores)),function(x) x,simplify=FALSE)
    }
    if (missing(adjustVars)) adjustVarsTxt <- '' else adjustVarsTxt <- paste(paste("pData(eset)[,'",adjustVars[!(adjustVars %in% vars2test$survival[i])],"']",sep='',collapse=' + '))
    if (mc.cores==1) {
      result.list <- lapply(exprs.list,function(y) do.call('rbind',apply(y,1,function(z) mycoxph(x=z,eset=x,coxSurvEvent,coxSurvTime,adjustVarsTxt,approach))))
    } else {
      if ('parallel' %in% loadedNamespaces()) {
        result.list <- parallel::mclapply(exprs.list,function(y) do.call('rbind',apply(y,1,function(z) mycoxph(x=z,eset=x,coxSurvEvent,coxSurvTime,adjustVarsTxt,approach))),mc.cores=mc.cores)
      } else stop('parallel library has not been loaded!')
    }
    result.matrix <- do.call('rbind',result.list)
    summaryDif$surv[,i] <- as.numeric(result.matrix[,1])
    signif$surv[,i] <- as.numeric(result.matrix[,2])
  }
}

allsummaryDif <- cbind(summaryDif$cont,summaryDif$ordi,summaryDif$categ,summaryDif$surv)
allsignifs <- cbind(signif$cont, signif$ordi, signif$categ, signif$surv)
allnames <- c(colnames(signif$cont), colnames(signif$ordi), vars2test$categorical, colnames(signif$surv))
rownames(allsummaryDif) <- rownames(allsignifs) <- featureNames(x)
if (p.adjust.method!='none' & approach=='frequentist') allsignifs <- apply(allsignifs,2,function(x) p.adjust(x,p.adjust.method))
cat('\n')

phenoName <- c(phenoName,allnames)
phenoClass <- c(phenoClass,phenoClass.signif)
if (is(allsignifs, 'numeric')) allsignifs <- t(as.matrix(allsignifs))
phenoType <- c(phenoType,rep('signif',ncol(allsignifs)))
meanLabel <- c(meanLabel,rep(NA,ncol(allsignifs)))
survTime <-  c(survTime,rep(NA,ncol(allsignifs)))

pdata <- data.frame(phenoName=as.factor(phenoName),phenoClass=as.factor(phenoClass),phenoType=as.factor(phenoType),meanLabel=as.factor(meanLabel),survTime=as.character(survTime))

pdata$survTime <- as.character(pdata$survTime)
myName <- pdata[pdata$phenoClass=='survival' & is.na(pdata$survTime),'phenoName']
pdata[pdata$phenoName %in% myName & is.na(pdata$survTime),'survTime'] <- pdata[pdata$phenoName %in% myName & !is.na(pdata$survTime),'survTime']
pdata$survTime <- as.factor(pdata$survTime)

rownames(pdata) <- c(colnames(allsummaryDif),colnames(allsignifs))
pdata <- new('AnnotatedDataFrame',pdata)
ans <- new('epheno',exprs=as.matrix(cbind(allsummaryDif,allsignifs)),phenoData=pdata,annotation=annotation(x))
ans@p.adjust.method <- p.adjust.method
ans@approach <- approach

return(ans)
}


#
# METHODS 4 THE EPHENO OBJECT (THERE ARE MORE OUT OF THIS FILE)
#

setMethod("dim",signature(x="epheno"),
  function (x) {
    ans <- c(nrow(exprs(x)),length(unique(pData(x)$phenoName)))
    names(ans) <- c('Features','Phenotypes')
    return(ans)
  }
)

setGeneric("phenoNames",function (x) standardGeneric("phenoNames"))
setMethod("phenoNames",signature(x="epheno"),
  function (x) {
    return(as.character(unique(pData(x)$phenoName)))
  }
)

setGeneric("approach",function (x) standardGeneric("approach"))
setMethod("approach",signature(x="epheno"),
  function (x) {
    err <- try(x@approach,silent=T)
    if (inherits(err, "try-error")) {
      ans <- 'frequentist'
    } else {
      ans <- x@approach
    }
    return(ans)
  }
)

setGeneric("p.adjust.method",function (x) standardGeneric("p.adjust.method"))
setMethod("p.adjust.method",signature(x="epheno"),
  function (x) {
    return(x@p.adjust.method)
  }
)

setGeneric("phenoClass",function (x) standardGeneric("phenoClass"))
setMethod("phenoClass",signature(x="epheno"),
  function (x) {
    tmp <- unique(pData(x)[,c('phenoName','phenoClass')])
    ans <- as.character(tmp[,2])
    names(ans) <- tmp[,1]
    return(ans)
  }
)

setMethod("show",signature(object="epheno"),
  function (object) {
    cat("Object of class 'epheno'\n")
    if (nrow(object)>3) {
      cat(paste("featureNames: ",paste(featureNames(object)[1:3],collapse=", ")," ... (",nrow(object),") feature(s)\n",sep=""))
    } else {
      cat(paste("featureNames: ",paste(head(featureNames(object)),collapse=", "),".  (",nrow(object),") feature(s)\n",sep=""))
    }
    if (ncol(object)>3) {
      cat(paste("phenoNames: ",paste(phenoNames(object)[1:3],collapse=', ')," ... (",ncol(object),") phenotype(s)\n",sep=''))
    } else {
      cat(paste("phenoNames: ",paste(head(phenoNames(object)),collapse=', '),". (",ncol(object),") phenotype(s)\n",sep=''))
    }
    cat("P-value adjustment method: ",p.adjust.method(object),'\n')
    cat("Annotation:",annotation(object),'\n')
    cat("Approach:",approach(object),'\n')
    cat("\nType \"showMethods(classes='epheno')\" for a list of ALL methods\n")
  }
)

setMethod("[",signature(x="epheno"),
  function (x, i, j) {
    if (!missing(i)) {
      if (is(i, "character")) {
        stopifnot(any(i %in% featureNames(x)))
        exprs(x) <- exprs(x)[i,,drop=FALSE]
      } else if (is(i, "numeric") | is(i, "integer")) {
        if (any(c(i<1, i>nrow(x)))) stop('Error: subscript out of bounds')
        exprs(x) <- exprs(x)[i,,drop=FALSE]
      } else if (is(i, "logical")) {
        exprs(x) <- exprs(x)[which(i),,drop=FALSE]
      }
    }
    if (!missing(j)) {
      if (is(j, "character")) {
        sel <- as.character(pData(x)$phenoName) %in% j
      } else if (is(j, "numeric") | is(j, "integer")) {
        if (any(c(j<1, j>length(phenoNames(x))))) stop('Error: subscript out of bounds')
        sel <- as.character(pData(x)$phenoName) == levels(pData(x)$phenoName)[j]
      } else if (is(j, "logical")) {
        if (length(j)!=length(phenoNames(x))) stop('Error: subscript out of bounds')
        sel <- pData(x)$phenoName %in% phenoNames(x)[j]
      }
      pData.n <- new('AnnotatedDataFrame', pData(x)[sel,,drop=FALSE])
      exprs.n <- exprs(x)[,sel,drop=FALSE]
      ans <- new('epheno',exprs=exprs.n,phenoData=pData.n,annotation=annotation(x))
      ans@p.adjust.method <- x@p.adjust.method
      ans@approach <- x@approach
      x <- ans
    }
    return(x)
  }
)

setGeneric("getMeans",function (x) standardGeneric("getMeans"))
setMethod("getMeans",signature(x="epheno"),
  function (x) {
    sel <- pData(x)$phenoType=="mean"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getSignif",function (x) standardGeneric("getSignif"))
setMethod("getSignif",signature(x="epheno"),
  function (x) {
    sel <- pData(x)$phenoType %in% c("signif","pval")
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getPvals",function (x) standardGeneric("getPvals"))
setMethod("getPvals",signature(x="epheno"),
  function (x) {
    stopifnot(approach(x)=='frequentist')
    sel <- pData(x)$phenoType=="pval" | pData(x)$phenoType=="signif"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getPostProbs",function (x) standardGeneric("getPostProbs"))
setMethod("getPostProbs",signature(x="epheno"),
  function (x) {
    stopifnot(approach(x)=='bayesian')
    sel <- pData(x)$phenoType=="signif"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getSummaryDif",function (x) standardGeneric("getSummaryDif"))
setMethod("getSummaryDif",signature(x="epheno"),
  function (x) {
    sel <- pData(x)$phenoType=="summaryDif"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getFc",function (x) standardGeneric("getFc"))
setMethod("getFc",signature(x="epheno"),
  function (x) {
    sel <- pData(x)$phenoType=="summaryDif" & pData(x)$phenoClass!="survival"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("getHr",function (x) standardGeneric("getHr"))
setMethod("getHr",signature(x="epheno"),
  function (x) {
    sel <- pData(x)$phenoType=="summaryDif" & pData(x)$phenoClass=="survival"
    return(exprs(x)[,sel,drop=FALSE])
  }
)

setGeneric("logFcHr",function (x) standardGeneric("logFcHr"))
setMethod("logFcHr",signature(x="epheno"),
  function (x) {
    fc <- getFc(x); fc <- log(abs(fc),2)*sign(fc)
    hr <- getHr(x); hr <- log(abs(hr),exp(1))*sign(hr) 
    ans <- cbind(fc,hr)
    return(ans)
  }
)

setGeneric("export2CSV",function(x, file, row.names=FALSE, ...) standardGeneric("export2CSV"))
setMethod("export2CSV",signature(x="epheno"), function(x, file, row.names=FALSE, ...) {
signifs <- getSignif(x)
stopifnot(approach(x) %in% c('frequentist','bayesian'))
if (approach(x)=='frequentist') {
  txt <- '.Pval'
} else  if (approach(x)=='bayesian') {
  txt <- '.PostProb'
} 
colnames(signifs) <- paste(colnames(signifs),txt,sep='')
if (is.null(fData(x))) {
  ans <- data.frame(featureNames=featureNames(x), getMeans(x), getSummaryDif(x), signifs)
} else {
  ans <- data.frame(fData(x), getMeans(x), getSummaryDif(x), signifs)
}
write.csv(ans, file=file, row.names=row.names, ...)
}
)

setGeneric("pAdjust",function(x, method='BH') standardGeneric("pAdjust"))
setMethod("pAdjust",signature(x="epheno"), function(x, method='BH') {
sel <- which(pData(x)$phenoType=="signif")
if (nrow(x) > 1) {
  pData.n <- new('AnnotatedDataFrame', pData(x))
  exprs.n <- exprs(x)
  exprs.n[, sel] <- apply(exprs.n[, sel], 2, function(xc) p.adjust(xc, method=method))
  ans <- new('epheno',exprs=exprs.n,phenoData=pData.n,annotation=annotation(x))
  ans@p.adjust.method <- method
  ans@approach <- x@approach
} else {
  ans <- x
}
return(ans)
}
)

setGeneric("getVars2test",function(x) standardGeneric("getVars2test"))
setMethod("getVars2test",signature(x="epheno"), function(x) {
ans <- list()
if (any(pData(x)$phenoClass=='continuous')) ans$continuous <- unique(as.character(pData(x)[pData(x)$phenoClass=='continuous','phenoName']))
if (any(pData(x)$phenoClass=='ordinal')) ans$ordinal <- unique(as.character(pData(x)[pData(x)$phenoClass=='ordinal','phenoName']))
if (any(pData(x)$phenoClass=='categorical')) ans$categorical <- unique(as.character(pData(x)[pData(x)$phenoClass=='categorical','phenoName']))
if (any(pData(x)$phenoClass=='survival')) {
  survival <- as.matrix(unique(pData(x)[pData(x)$phenoClass=='survival',c('phenoName','survTime')]))
  colnames(survival) <- colnames(survival) <- c('event','time')
  rownames(survival) <- NULL
  ans$survival <- survival
}
return(ans)
}
)

getChrWithMedVar <- function(x) {
  sel <- table(x$chr) > quantile(table(x$chr),c(0.10)) #remove small chr
  sel <- names(sel[sel])
  x <- x[as.character(x$chr) %in% sel,]
  ans <- by(x,as.character(x$chr),function(x) sum(x$es^2,na.rm=T)/nrow(x))
  mynames <- names(ans)
  ans <- as.numeric(ans)
  names(ans) <- mynames
  ans <- names(sort(ans)[floor(length(ans)/2)])
  ans
}

getEsPositions <- function(epheno,phenoName,organism='human',logEs=T,center=FALSE) {
  #get gene pos from biomaRt
  stopifnot(is(epheno, 'epheno'))
  stopifnot(phenoName %in% phenoNames(epheno))
  if (annotation(epheno) == 'entrezid') {
    stopifnot(organism %in% c('human','mouse','drosophila'))
    if (organism=='human') mymart <- useMart("ensembl", "hsapiens_gene_ensembl")
    if (organism=='mouse') mymart <- useMart("ensembl", "mmusculus_gene_ensembl")
    if (organism=='dmelanogaster') mymart <- useMart("ensembl", "dmelanogaster_gene_ensembl")      
    homol <- getLDS(attributes = c("entrezgene"), mart = mymart, attributesL = c("start_position",'end_position','chromosome_name'), martL = mymart)
    homol <- homol[apply(homol,1,function(x) !any(is.na(x))),]
    homol[,1] <- as.character(homol[,1])
    gene.pos <- data.frame(ids=homol[,1],start=apply(homol[,2:3],1,min),end=apply(homol[,2:3],1,max),chr=homol[,4])
  } else {
    eval(parse(text=paste('library(',annotation(epheno),'.db)',sep='')))
    chr <- eval(parse(text=paste('unlist(lapply(AnnotationDbi::mget(featureNames(epheno),',annotation(epheno),'.db::',annotation(epheno),'CHR),function(x) x[1]))',sep='')))
    st <- eval(parse(text=paste('unlist(lapply(AnnotationDbi::mget(featureNames(epheno),',annotation(epheno),'CHRLOC),function(x) min(abs(x))))',sep='')))
    en <- eval(parse(text=paste('unlist(lapply(AnnotationDbi::mget(featureNames(epheno),',annotation(epheno),'CHRLOCEND),function(x) max(abs(x))))',sep='')))
    gene.pos <- data.frame(ids=featureNames(epheno),start=st,end=en,chr=chr)
  }
  #obtain es and merge with gene positions
  es <- as.numeric(getSummaryDif(epheno[,phenoName]))
  names(es) <- featureNames(epheno)
  mygene.pos <- gene.pos[match(names(es),gene.pos[,'ids']),]
  x <- data.frame(es,chr=mygene.pos$chr,pos=rowMeans(mygene.pos[,c('start','end')]),row.names=names(es))
  x <- x[apply(x,1,function(x) !any(is.na(x))),]
  #some preprocessing
  if (logEs) x$es <- log(abs(x$es))*sign(x$es)
  if (center) x$es <- x$es-mean(x$es)
  x$pos <- x$pos / 1e+6
  x <- x[order(x$chr,x$pos),]
  x$chr <- factor(as.character(x$chr))
  if (any(table(x$chr)<30)) {
    warning('chromosomes with less than 30 genes were removed from the analysis.\n')
    x <- x[x$chr %in% names(table(x$chr)[table(x$chr)>=30]),]
  }
  #
  return(x)
}

preProcess4Ace <- function(x,logEs=T,center=FALSE) {
  if (logEs) x$es <- log(abs(x$es))*sign(x$es)
  if (center) x$es <- x$es-mean(x$es)
  x$pos <- x$pos / 1e+6
  x <- x[order(x$chr,x$pos),]
  x$chr <- factor(as.character(x$chr))
  if (any(table(x$chr)<30)) {
    warning('chromosomes with less than 30 genes were removed from the analysis.\n')
    x <- x[x$chr %in% names(table(x$chr)[table(x$chr)>=30]),]
  }
  return(x)
}

#getPred <- function(es,pos='1',sp,k=round(length(es)/10)) {
getPred <- function(es,pos='1',sp,k=20) {
  if (missing(sp)) {
    mygam <- gam(es ~ s(pos,k=k,bs='cr'))
    sp <- mygam$sp
  } else {
    mygam <- gam(es ~ s(pos,sp=sp,k=k,bs='cr'))
  }
  m.pred <- predict(mygam,as.data.frame(pos))
  return(list(m.pred=m.pred,sp=sp))
}

getDistToClosest <- function(pos,numGenes) {
  ans <- sapply(1:length(pos),function(i) {
    tmp <- ifelse(i-(1:numGenes)<=0,NA,i-(1:numGenes))
    selprev <- ifelse(any(!is.na(tmp)),min(tmp,na.rm=T),0)
    selnext <- max(ifelse(i+(1:numGenes)>length(pos),0,i+(1:numGenes)))
    max(pos[i]-pos[selprev],pos[selnext]-pos[i])
  })
  ans
}

getIndivPvals <- function(obs.pred,sim.pred,p.adjust.method='BY',mc.cores=mc.cores,useAllPerm=F,d,B,useIntegrate=TRUE) {
  if (!useIntegrate) {
    pvals <- pnorm(-abs(obs.pred),mean(sim.pred),sd(sim.pred))*2
  } else {
    mymin <- min(c(obs.pred,sim.pred))*1.5
    mymax <- max(c(obs.pred,sim.pred))*1.5
    if (useAllPerm) {
      myDens <- density(sim.pred,from=mymin,to=mymax,n=2056)
      myFun <- approxfun(myDens$x,myDens$y)
      intAll <- integrate(myFun,mymin,mymax)$value
    }
    myDens <- density(sim.pred,from=mymin,to=mymax,n=2056*2)
    myFun <- approxfun(myDens$x,myDens$y)
    getPval <- function(x,sim.pred,useAllPerm,useIntegrate=TRUE) {
      if(useIntegrate) {
        if (!useAllPerm) {
          mymin <- min(c(sim.pred,x))
          mymax <- max(c(sim.pred,x))
          myDens <- density(sim.pred,from=mymin,to=mymax,n=2056)
          myFun <- approxfun(myDens$x,myDens$y)
          intAll <- integrate(myFun,mymin,mymax)$value
        }
        err <- try(
                   if (x>mean(sim.pred)) {
                     tmp <- integrate(myFun,x,mymax)$value
                   } else {
                     tmp <- integrate(myFun,mymin,x)$value
                   }
                   ,silent=TRUE)
        if (inherits(err, "try-error")) {
          ans <- NA
        } else {
          ans <- tmp / intAll
        }
      } else {
        ans <- pnorm(-abs(x),mean(sim.pred),sd(sim.pred))
      }
      ans
    }
    if (useAllPerm) {
      if (mc.cores==1) {
        pvals <- sapply(obs.pred,function(x) getPval(x,sim.pred,useAllPerm,useIntegrate)) * 2
      } else {
        if ('parallel' %in% loadedNamespaces()) {
          pvals <- unlist(parallel::mclapply(as.list(obs.pred),function(x) getPval(x,sim.pred,useAllPerm,useIntegrate),mc.preschedule=T,mc.cores=mc.cores)) * 2
        } else stop('parallel library has not been loaded!')
      }
    } else {
      mypos <- cbind(1:length(d),order(d))
      d <- d[mypos[,2]]
      sel <- lapply(as.list(1:length(d)),function(i) {
        shiftStart <- sum(((i-9):(i+9))<=0)
        shiftEnd <- sum(((i-9):(i+9))>length(d))
        which(order(abs(d[i]-d[((i-9):(i+9))+shiftStart-shiftEnd])) %in% 1:10) + i + shiftStart - shiftEnd - 10
      })
      mypos <- mypos[order(mypos[,2]),]
      sel <- lapply(sel,function(x) mypos[x,1])
      sel <- lapply(sel,function(sel) as.numeric(sapply((0:(B-1))*length(d),function(x) x + sel)))
      if (mc.cores==1) {      
        pvals <- sapply(1:length(obs.pred),function(i) getPval(obs.pred[i],sim.pred[sel[[i]]],useAllPerm,useIntegrate)) * 2
      } else {
      if ('parallel' %in% loadedNamespaces()) {
        pvals <- unlist(parallel::mclapply(as.list(1:length(obs.pred)),function(i) getPval(obs.pred[i],sim.pred[sel[[i]]],useAllPerm,useIntegrate),mc.preschedule=T,mc.cores=mc.cores)) * 2
      } else stop('parallel library has not been loaded!')
    }
    }
  }
  pvals <- ifelse(pvals>1,1,ifelse(pvals<0,0,pvals))
  pvals <- p.adjust(pvals,p.adjust.method)
  pvals
}

getRegions <- function(x,pvals,minGenes,pvalcutoff=0.01,obs.pred,exprScorecutoff=NA) {
  if (is.na(exprScorecutoff)) myrle <- rle(pvals<pvalcutoff) else myrle <- rle(pvals<pvalcutoff & abs(as.numeric(obs.pred))>exprScorecutoff)
  start <- c(names(pvals[1]),names(myrle$lengths[-length(myrle$lengths)]))
  end <- names(myrle$values) 
  tmp <- cbind(start,end)[myrle$values & myrle$lengths>minGenes,]
  ans <- matrix(x[as.character(tmp),'pos'],ncol=2)
  colnames(ans) <- c('start','end')
  ans
}

plotCopyNumber <- function(es,chr,pos,obs.pred,ssr,es.mean,genome,chrLengths) {
  xlim <- c(0,chrLengths[[chr]])
  chr <- gsub('CHR','',gsub('chr','',gsub('CHR0','',gsub('chr0','',chr))))
  lenNames <- gsub('CHR','',gsub('chr','',gsub('CHR0','',gsub('chr0','',names(chrLengths)))))
  plotCyto <- chr %in% lenNames & genome=='hg18' & chr %in% c(as.character(1:22),'X','Y','M')
  if (plotCyto) {
    def.par <- par(no.readonly = TRUE)    
    par(mar=c(1,3,1.5,1))
    layout(c(1,2),heights=c(2.5,0.5))
  }
  plot(pos,es,pch=20,main=paste("chr",chr),col=densCols(pos,es),xlab='Position in chromosome (Mb)',ylab='Expression Score',xlim=xlim/1e6)
  if (!missing(obs.pred)) lines(pos[order(pos)],obs.pred,col='darkred',lwd=2)
  abline(h=0,lty=2,col='grey')
  if (!missing(es.mean)) abline(h=es.mean,lty=3,col='red')
  abline(v=c(seq(0,xlim[2]/1e6,xlim[2]/1e6/10)),col='green',lty=3)
  if (!missing(ssr)) {
    if (nrow(ssr)>0) apply(ssr,1,function(y) rect(y[1],min(es)*2,y[2],max(es)*2,col=paste(rgb(165/255,42/255,42/255),'15',sep='')))
  }
  ## if (plotCyto) {
  ##   par(mar=c(4,3,1.5,1))
  ##   SNPchip::plotCytoband(chr,cex=0.6,build=genome,xlim=xlim,taper=T)
  ##   abline(v=c(seq(0,xlim[2],xlim[2]/10)),col='green',lty=1)
  ##   par(def.par)
  ## }
}

qcPlot <- function(ids,x,B,sim.pred,minGenes=10) {
  d <- unlist(lapply(ids,function(i) getDistToClosest(x[x$chr==i,'pos'],minGenes)))
  d <- rep(d,B)
  pdf('~/Desktop/qc.pdf')
  plot(d,sim.pred,col=densCols(d,sim.pred),pch=20,xlab='distance to the 10th neighbour',ylab='neighbourhood scores')
  mylm <- lm(sim.pred ~ d)
  summary(mylm)
  abline(mylm,col=2)
  boxplot(sim.pred ~ cut(d,50),xlab='distance to the 10th neighbour',ylab='neighbourhood scores' ,main='Hole genome. 50 boxplots')
  boxplot(sim.pred ~ cut(d,100),xlab='distance to the 10th neighbour',ylab='neighbourhood scores',main='Hole genome. 100 boxplots')
  boxplot(sim.pred ~ cut(d,200),xlab='distance to the 10th neighbour',ylab='neighbourhood scores',main='Hole genome. 200 boxplots')
  tmp <- cut(d,50)
  plot(density(sim.pred[tmp==levels(tmp)[1]]),xlim=c(-0.3,0.1),ylim=c(0,50))
  for (i in seq(2,50,5)) if (sum(tmp==levels(tmp)[i])>5) lines(density(sim.pred[tmp==levels(tmp)[i]]),xlim=c(-0.3,0.1),ylim=c(0,50),col=i)
  dev.off()
}

findCopyNumber <- function(x,minGenes=15,B=100,p.adjust.method='BH',pvalcutoff=0.05,exprScorecutoff=NA,mc.cores=1,useAllPerm=F,genome='hg19',chrLengths,sampleGenome=TRUE,useOneChr=FALSE,useIntegrate=TRUE,plot=TRUE,minGenesPerChr=100) {
  #Input has to be a data frame with gene ids as rownames and 3 columns: es (enrichment score), chr (chromosome) and pos (position in chromosome)
  stopifnot(identical(colnames(x),c('es','chr','pos')))
  #remove chr with less than 100 es
  if (any(table(x$chr)<100)) {
    sel.chr <- names(table(x$chr))[table(x$chr)>minGenesPerChr]
    x <- x[x$chr %in% sel.chr,]
    warning('Chromosomes with less than 100 scores have been removed from the analysis.')
  }
  #order
  x <- x[order(x$chr,x$pos),]
  #get smoothed observed values
  cat('get smoothed observed values...')  
  ids <- as.list(levels(factor(as.character(x$chr))))
  names(ids) <- unlist(ids)
  if (mc.cores==1) {
    tmp <- lapply(ids,function(i) getPred(es=x[x$chr==i,'es'],pos=x[x$chr==i,'pos']))
  } else {
    if ('parallel' %in% loadedNamespaces()) {
      tmp <- parallel::mclapply(ids,function(i) getPred(es=x[x$chr==i,'es'],pos=x[x$chr==i,'pos']),mc.preschedule=F,mc.cores=mc.cores)
    } else stop('parallel library has not been loaded!')      
  }
  obs.pred <- lapply(tmp,function(x) x[['m.pred']])
  mysp <- lapply(tmp,function(x) x[['sp']])
  #get smoothed simulated values
  cat('Ok\nget smoothed simulated values...')
  if (useOneChr) {
    selChr <- getChrWithMedVar(x)
    sim <- sapply(1:(B),function(i) sample(x[x$chr==selChr,'es'])) #sample over all genome
    if (mc.cores==1) {
      sim.pred <- unlist(lapply(as.list(1:(B)),function(j) getPred(es=sim[,j],pos=x[x$chr==selChr,'pos'],sp=mysp[[selChr]])$m.pred))
    } else {
      if ('parallel' %in% loadedNamespaces()) {
        sim.pred <- unlist(parallel::mclapply(as.list(1:(B)),function(j) getPred(es=sim[,j],pos=x[x$chr==selChr,'pos'],sp=mysp[[selChr]])$m.pred,mc.preschedule=T,mc.cores=mc.cores))
      } else stop('parallel library has not been loaded!')      
    }
  } else {
    if (sampleGenome) {
      sim <- sapply(1:(B),function(i) sample(x$es)) #sample over all genome
    } else {
      sim <- sapply(1:(B),function(i) unlist(sapply(as.character(unique(x$chr)),function(chr) sample(x[x$chr==chr,'es'])))) #sample each chromosome
    }
    if (mc.cores==1) {
      sim.pred <- unlist(lapply(as.list(1:(B)),function(j) lapply(ids,function(i) getPred(es=sim[x$chr==i,j],pos=x[x$chr==i,'pos'],sp=mysp[[i]])$m.pred)))
    } else {
      if ('parallel' %in% loadedNamespaces()) {
        sim.pred <- unlist(lapply(as.list(1:(B)),function(j) parallel::mclapply(ids,function(i) getPred(es=sim[x$chr==i,j],pos=x[x$chr==i,'pos'],sp=mysp[[i]])$m.pred,mc.preschedule=F,mc.cores=mc.cores)))
      } else stop('parallel library has not been loaded!')      
    }
  }
  #qc plot
#  qcPlot(ids,x,B,sim.pred,minGenes=10)
  #get pvals
  cat('Ok\nget pvals...')
  if (useOneChr) {
    pvals <- getIndivPvals(obs.pred=unlist(obs.pred),sim.pred=sim.pred,p.adjust.method=p.adjust.method,mc.cores=mc.cores,useAllPerm=T,d=d,B=B,useIntegrate=useIntegrate)
  } else {
    if (useAllPerm | !sampleGenome) {
      d <- 0
    } else {
      if (mc.cores==1) {
        d <- unlist(lapply(ids,function(i) getDistToClosest(x[x$chr==i,'pos'],minGenes)))
      } else {
        if ('parallel' %in% loadedNamespaces()) {
          d <- unlist(parallel::mclapply(ids,function(i) getDistToClosest(x[x$chr==i,'pos'],minGenes),mc.preschedule=F,mc.cores=mc.cores))
        } else stop('parallel library has not been loaded!')
      }
    }
    if (sampleGenome) {
      pvals <- getIndivPvals(obs.pred=unlist(obs.pred),sim.pred=sim.pred,p.adjust.method=p.adjust.method,mc.cores=mc.cores,useAllPerm=useAllPerm,d=d,B=B,useIntegrate=useIntegrate)
    } else {
      pvals <- unlist(sapply(as.character(unique(x$chr)),function(i) {
        sel <- x$chr==i
        ans <- getIndivPvals(obs.pred=obs.pred[[i]],sim.pred=sim.pred[sel],p.adjust.method='none',mc.cores=1,useAllPerm=T,d=d,B=B,useIntegrate=useIntegrate)
        ans
      }))
    }
  }
  names(pvals) <- rownames(x)
  #get enriched regions

##   i <- ids[[1]]
##   getRegions(x[x$chr==i,],pvals[x$chr==i],minGenes=minGenes,pvalcutoff=pvalcutoff,obs.pred=obs.pred[[i]],minScore=1)
  
  cat('Ok\nget enriched regions...')  
  ssr <- lapply(ids,function(i) getRegions(x[x$chr==i,],pvals[x$chr==i],minGenes=minGenes,pvalcutoff=pvalcutoff,obs.pred=obs.pred[[i]],exprScorecutoff=exprScorecutoff))
  #make plot
  if (plot) {
    cat('Ok\nmake plot...')  
    if (missing(chrLengths)) {
      chrLengths <- unlist(as.list(by(x,x$chr,function(x) max(x['pos'],na.rm=T)))) * 1e6
      chrLengths <- chrLengths[!is.na(chrLengths)]
    }
    is.num.ids <- !grepl('[a-zA-Z]',names(ids))
    ids.num <- names(ids)[is.num.ids][order(as.numeric(names(ids)[is.num.ids]))]
    ids.chr <- names(ids)[!is.num.ids][order(names(ids)[!is.num.ids])]
    ids <- ids[c(ids.num,ids.chr)]
    chrLengths <- chrLengths[c(ids.num,ids.chr)]
    lapply(ids,function(i) plotCopyNumber(es=x[x$chr==i,'es'],chr=i,pos=x[x$chr==i,'pos'],obs.pred=obs.pred[[i]],ssr=ssr[[i]],es.mean=mean(x$es),genome=genome,chrLengths=chrLengths))
  }
  #return regions
  cat('Ok\nreturn regions...')  
  ssr <- ssr[lapply(ssr,nrow)>0]
  if (length(ssr)>0) {
    ssr <- do.call(rbind,lapply(1:length(ssr),function(i) data.frame(chr=names(ssr)[i],regionInChr=1:nrow(ssr[[i]]),ssr[[i]])))
    rownames(ssr) <- paste('Region',1:nrow(ssr),sep='')
  }
  cat('Ok\n')
  return(ssr)
}

genesInArea <- function(x,regions) {
  if (length(regions)>0) {
    sel <- apply(regions,1,function(i) which(x$chr==as.character(i['chr']) & x$pos>=as.numeric(i['start']) & x$pos<=as.numeric(i['end'])))
    ans <- lapply(sel,function(i) data.frame(rownames(x[i,]),x[i,'es']))
    if (nrow(regions)==1) names(ans) <- rep('Region1',length(ans))
    ans <- lapply(1:length(ans),function(i) data.frame(rep(names(ans[i]),nrow(ans[[i]])),ans[[i]]))
    ans <- do.call(rbind,ans)
    colnames(ans) <- c('Region','gene','es')
  } else {
    ans <- NULL
  }
  ans
}

eset2genelevel <- function(x) {
  stopifnot(is(x, 'ExpressionSet'))
  annolib <- paste(annotation(x),'.db',sep='')
  if (!(annolib %in% loadedNamespaces())) stop(paste(annolib,'has not been loaded!'))
  #
  ans <- nsFilter(x,require.entrez=FALSE,remove.dupEntrez=TRUE,var.filter=FALSE)$eset
  text <- "as.character(unlist(AnnotationDbi::mget(featureNames(ans),eval(parse(text=paste(annotation(ans),'ENTREZID',sep=''))))))"
  featureNames(ans) <- eval(parse(text=text))
  text <- paste(annotation(ans),'ORGPKG',sep='')
  annotation(ans) <- eval(parse(text=text))
  ans
}

smoothCoxph <- function(time, event, x, xlim, ylim, xlab, ylab, logrisk=TRUE, ...) {
  z <- data.frame(time = time, event = event, x = x)
  z <- z[!is.na(z$event) & !is.na(z$time) & !is.na(z$x),]
  z <- z[order(z$x), ]
  coxph1 <- coxph(Surv(time, event) ~ pspline(x, df = 4), data = z)
  if (logrisk) {
    pred <- predict(coxph1, type = "lp", se.fit = TRUE)
    y <- cbind(pred$fit,pred$fit-1.96*pred$se.fit,pred$fit+1.96*pred$se.fit)
  } else {
    pred <- predict(coxph1, type='risk', se.fit=TRUE)
    y <- cbind(log(pred$fit),log(pred$fit-1.96*pred$se.fit),log(pred$fit+1.96*pred$se.fit))
  }
  if (missing(xlim)) xlim <- range(x)
  if (missing(ylim)) ylim <- c(min(y[,1],median(y[,2]),na.rm=T),max(y[,1],median(y[,3]),na.rm=T)) * 1.5
  if (missing(xlab)) xlab <- 'Gene expression'
  if (missing(ylab)) ylab <- 'log Hazard Ratio'  
  if (any(is.infinite(ylim)))
    plot(z$x[1:length(pred$fit)], y[, 1], type = "l", xlim = xlim,xlab=xlab,ylab=ylab)
  else
  plot(z$x[1:length(pred$fit)], y[, 1], type = "l", ylim = ylim, xlim = xlim,xlab=xlab,ylab=ylab,...)
  lines(z$x[1:length(pred$fit)], y[, 2], lty = 2,...)
  lines(z$x[1:length(pred$fit)], y[, 3], lty = 2,...)
  abline(h=0,lty=2,col='grey')
}

gsea2html <- function(gseaData,epheno,variable,title='',path,file,digits=3,plotEs=FALSE,limit=100) {
  #control errors
  stopifnot(is(gseaData, 'gseaData'))
  stopifnot(is(epheno, 'epheno'))
  stopifnot(!missing(variable))
  stopifnot(variable %in% phenoNames(epheno))
  stopifnot(!missing(file))
  stopifnot(file.exists(path))

  #my label
  mylabel <- gsub('.html','',file)
  
  #preprocess
  id.entrezid <- 'org.' %in% substr(annotation(epheno),0,4)
  epheno <- epheno[,variable]
  sigs <- gseaData[[1]][[1]]$signatures

  gsets <- summary(gseaData)
  gsets <- data.frame(gsets[1:2],plot='',gsets[3:ncol(gsets)])

  genes <- exprs(epheno)
  sel <- pData(epheno)$phenoType=='signif'
  colnames(genes)[sel] <- paste(colnames(genes)[sel],ifelse(approach(epheno)=='frequentist','pvalue','postProb'),sep='.')

  fnames <- featureNames(epheno)
  fnames <- unique(fnames[fnames %in% unlist(sigs)])

  #gene annotation
  if (id.entrezid) {
    entrezid <- featureNames(epheno)
    orglib <- paste(annotation(epheno),'.db',sep='')
    if (!(orglib %in% loadedNamespaces())) stop('parallel library has not been loaded!')
    envir <- eval(parse(text=paste(annotation(epheno),'SYMBOL',sep='')))
    symbol <- unlist(AnnotationDbi::mget(featureNames(epheno),envir,ifnotfound=NA))
    envir <- eval(parse(text=paste(annotation(epheno),'GENENAME',sep='')))
    genename <- unlist(AnnotationDbi::mget(featureNames(epheno),envir,ifnotfound=NA))
  } else {
    entrezid <- unlist(AnnotationDbi::mget(fnames,AnnotationDbi::get(paste(annotation(epheno),"ENTREZID", sep = "")),ifnotfound=NA))
    symbol <- unlist(AnnotationDbi::mget(fnames,AnnotationDbi::get(paste(annotation(epheno),"SYMBOL", sep = "")),ifnotfound=NA))
    genename <- unlist(AnnotationDbi::mget(fnames,AnnotationDbi::get(paste(annotation(epheno),"GENENAME", sep = "")),ifnotfound=NA))
  }
  anno <- data.frame(entrezid=as.character(entrezid),symbol=as.character(symbol),genename=as.character(genename),stringsAsFactors=F)
  
  #filter gsets
  gsets <- gsets[order(gsets$fdr),]
  if (limit<nrow(gsets)) gsets <- gsets[1:limit,]

  #gene set annotation
  #if (gseaData$gsetOrigin=='KEGG') {
    #envir <- eval(parse(text='KEGGPATHID2NAME'))
    #gsetname <- gsub('hsa','',gsub('mmu','',gsets[,'geneSet']))
    #geneSetName <- unlist(AnnotationDbi::mget(gsetname,envir))
    #gsets <- data.frame(gsets[,1:2],geneSetName=geneSetName,gsets[,3:ncol(gsets)])
  #} else if (gseaData$gsetOrigin=='GO') {
  if (gseaData$gsetOrigin=='GO') {
    geneSetName <- Term(as.character(gsets[,'geneSet']))
    gsets <- data.frame(gsets[,1:2],geneSetName=geneSetName,gsets[,3:ncol(gsets)])
  }

  #set links
  linksout <- tiny.pic <- vector('list',ncol(gsets))
  linksout[[2]] <- paste('gsets_',mylabel,'/gset_',1:nrow(gsets),'.html',sep='')
  link2plot <- paste('plots_',mylabel,'/gset_',1:nrow(gsets),'.png',sep='')
  #if (gseaData$gsetOrigin=='KEGG') {
    #linksout[[3]] <- paste('http://www.genome.jp/dbget-bin/www_bget?map',gsub('mmu','',as.character(gsets$geneSet)),sep='')
    #linksout[[4]] <- tiny.pic[[4]] <- link2plot
  #} else if (gseaData$gsetOrigin=='GO') {
  if (gseaData$gsetOrigin=='GO') {
    linksout[[3]] <- paste('http://amigo.geneontology.org/cgi-bin/amigo/term_details?term=',as.character(gsets[,'geneSet']),sep='')
    linksout[[4]] <- tiny.pic[[4]] <- link2plot
  }

  #write html
  plotsdir <- file.path(path,paste('plots',mylabel,sep='_'))
  gsetsdir <- file.path(path,paste('gsets',mylabel,sep='_'))
  if (file.exists(plotsdir)) unlink(plotsdir, recursive = TRUE)
  if (file.exists(gsetsdir)) unlink(gsetsdir, recursive = TRUE)
  dir.create(plotsdir, showWarnings=F)
  dir.create(gsetsdir, showWarnings=F)
  write.html(gsets,file=file.path(path,file),links=linksout,tiny.pic=tiny.pic,title=title,digits=digits)
  for (i in 1:nrow(gsets)) {
    gsetName <- as.character(gsets[i,'geneSet'])
    dat <- data.frame(anno[match(sigs[[gsetName]],anno$entrezid),,drop=FALSE],genes[sigs[[gsetName]],,drop=FALSE])
    links <- vector('list',ncol(dat))
    links[[1]] <- paste('http://www.ncbi.nlm.nih.gov/sites/entrez?db=gene&cmd=Retrieve&dopt=Graphics&list_uids=',anno[sigs[[gsetName]],'entrezid'],sep='')
    links[[2]] <- paste('http://www.genecards.org/index.php?path=/Search/keyword/',anno[sigs[[gsetName]],'symbol'],sep='')
    write.html(dat,links=links,file=paste(path,'/gsets_',mylabel,'/gset_',i,'.html',sep=''),title=title,digits=digits)
    png(file.path(path,paste('plots',mylabel,sep='_'),paste('gset_',i,'.png',sep='')))
    plot(gseaData,es.nes=ifelse(plotEs,'es','nes'),selGsets=as.character(gsetName),selVars=as.character(gsets[i,1]))
    dev.off()
  }
}

barplotCI <- function(x,groups,alpha=.05,exp=FALSE,base='nat',refgroup,plot.ci=TRUE,ci.l=NA,ci.u=NA,order,...) {
  if (!is.factor(groups)) {
    warning('groups have been converted to factor')
    groups <- factor(groups)
  }
  if (length(unique(groups))==1) {
    lm1 <- lm(x ~ 1)
    b <- c(coef(lm1),confint(lm1,level=1-alpha))
    names(b) <- as.character(groups)[1]
    if (exp & base=='nat') { b <- exp(b) } else if (exp & base!='nat') { b <- as.numeric(base)^b }
    barplot2(b[1],names=names(b)[1],plot.ci=TRUE,ci.l=b[2],ci.u=b[3],...)
  } else {
    lm1 <- lm(x ~ -1 + groups)
    b <- cbind(coef(lm1),confint(lm1,level=1-alpha))
    rownames(b) <- sub('groups','',rownames(b))
    if (exp & base=='nat') { b <- exp(b) } else if (exp & base!='nat') { b <- as.numeric(base)^b }
    if (!missing(refgroup)) { b <- b/b[refgroup,1] }
    if (!missing(order)) { b <- b[order,] }
    barplot2(b[,1],names=rownames(b),plot.ci=TRUE,ci.l=b[,2],ci.u=b[,3],...)
  }
}



setGeneric("barplotSignifSignatures",function (x, signatures, referenceSignature, testUpDown=FALSE, simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05, ylab, ylim=ylim, cex.text=1, ...) standardGeneric("barplotSignifSignatures"))

setMethod("barplotSignifSignatures",signature(x="epheno",signatures="list"),
  function (x, signatures, referenceSignature, testUpDown=FALSE, simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05, ylab, ylim=ylim, cex.text=1, ...) {
    stopifnot(approach(x)=='frequentist')
    if (!missing(referenceSignature)) if (!(referenceSignature %in% names(signatures))) stop('referenceSignature not in names(signatures)')
    lensig <- unlist(lapply(signatures,function(x) length(x)))
    if (all(lensig==0)) stop('All signatures are empty. Check the argument signatures.\n')
    if (any(lensig==0)) {
      warning('Removed some signatures with no genes\n')
      signatures <- signatures[lensig>0]
    }
    badsignature <- sapply(signatures,function(y) !(any(y %in% featureNames(x))))
    if (any(badsignature)) stop(paste('Identifiers in signatures',paste(names(signatures)[which(badsignature)],collapse=','),'not in featureNames'))
    if (!missing(referenceSignature)) { if (!(referenceSignature %in% names(signatures))) stop('referenceSignature has to be one of names(signatures) !') }

    #Compute percentages of DE genes
    myFun1 <- function(i) {
      signde <- sign(rowMeans(getSummaryDif(x[,i]))) * (getPvals(x[,i])<alpha)
      ans <- lapply(signatures, function(y) table(factor(signde[featureNames(x) %in% y],levels=-1:1)))
      ans <- do.call(rbind,ans)
      return(ans)
    }
    x2plot <- sapply(phenoNames(x), myFun1, simplify=FALSE)
    if (!missing(referenceSignature)) {
      myorder <- c(which(names(signatures) %in% referenceSignature),which(!(names(signatures) %in% referenceSignature)))
      x2plot <- lapply(x2plot,function(x) x[myorder,])
    }

    #Test for significance
    if (missing(referenceSignature)) {
      myFun2 <- function(x) ifelse(sum(x[c(1,3)])==0,1,binom.test(x=x[1],n=sum(x[c(1,3)]),alternative='two.sided')$p.value)
      pvals <- lapply(x2plot, function(x) apply(x,1,myFun2))
    } else {
      sel <- which(names(signatures) != referenceSignature)
      if (!testUpDown) {
        myFun3 <- function(x) {
          xtab <- sapply(sel,function(y) rbind(x[y,c(1,3)],x[referenceSignature,c(1,3)]),simplify=FALSE)
          names(xtab) <- names(signatures)[sel]
          ans <- unlist(lapply(xtab, function(y) ifelse((any(rowSums(y)==0) | any(colSums(y)==0)),1,chisq.test(y, simulate.p.value=simulate.p.value, B=B)$p.value)))
          return(ans)
        }
        pvals <- lapply(x2plot,myFun3)
      } else {
        myFun4 <- function(x) {
          xtab <- sapply(sel, function(y) matrix(c(x[y,1],sum(x[y,-1]),x[referenceSignature,1],sum(x[referenceSignature,-1])),nrow=2),simplify=FALSE)
          names(xtab) <- names(signatures)[sel]
          pvals.do <- unlist(lapply(xtab,function(x) ifelse((any(rowSums(x)==0) | any(colSums(x)==0)),1,chisq.test(x, simulate.p.value=simulate.p.value, B=B)$p.value)))
          xtab <- sapply(sel, function(y) matrix(c(x[y,3],sum(x[y,-3]),x[referenceSignature,3],sum(x[referenceSignature,-3])),nrow=2),simplify=FALSE)
          names(xtab) <- names(signatures)[sel]
          pvals.up <- unlist(lapply(xtab,function(x) ifelse((any(rowSums(x)==0) | any(colSums(x)==0)),1,chisq.test(x, simulate.p.value=simulate.p.value, B=B)$p.value)))
          ans <- cbind(Down=pvals.do,Up=pvals.up)
          return(ans)
        }
      pvals <- lapply(x2plot,myFun4)
      }
    }

    if (length(signatures)==1) {
      pvals <- as.list(p.adjust(unlist(pvals),method=p.adjust.method))
    } else {
      if (p.adjust.method!='none') pvals <- lapply(pvals,function(x) p.adjust(x,p.adjust.method))
    }
    
    #Plot
    z <- lapply(x2plot,function(x) as.matrix(100*t(x/rowSums(x))[-2,]))
    if (missing(ylim)) ylim <- c(0,1.2*max(sapply(z,'max'),na.rm=T))
    if (missing(ylab)) ylab <- "% differentially expressed genes"
    cex <- ifelse(testUpDown,.8,1)
    if (length(signatures)==1) {
      pval2plot <- ifelse(unlist(pvals) < 1e-04, "P<0.0001", paste("P=", round(unlist(pvals), 4), sep = ""))
      myData <- matrix(unlist(z), ncol = length(z)); colnames(myData) <- names(z); rownames(myData) <- c("Down-regulated", "Up-regulated")
      barplot(myData,beside=TRUE,legend=TRUE,ylim=ylim,ylab=ylab,args.legend=list(cex=0.7,box.lty=0),...)
      text((1:length(pval2plot) * 3) - 1,apply(myData,2,max)+1,pval2plot,cex=cex)
    } else {
      for (i in 1:length(x2plot)) {
        ndec <- ifelse(testUpDown,3,4)
        pval2plot <- ifelse(pvals[[i]]<1/10^ndec,paste('P<',1/10^ndec,sep=''),paste('P=',round(pvals[[i]],ndec),sep=''))
        rownames(z[[i]]) <- c('Down-regulated','Up-regulated')
        barplot(z[[i]],beside=TRUE,legend=TRUE,ylim=ylim,ylab=ylab,args.legend=list(cex=0.7,box.lty=0),...)
        if (missing(referenceSignature)) {
          text((1:length(pval2plot) * 3) - 1,apply(z[[i]],2,max)+1,pval2plot,cex=cex)
        } else {
          if (!testUpDown) {
            text((1:length(pval2plot) * 3) + 2,apply(z[[i]],2,max)[-1]+1,pval2plot,cex=cex)
          } else {
            xpos <- (1:length(pval2plot) * 3)
            xpos <- rep(xpos,2); xpos <- xpos[order(xpos)]; xpos <- xpos - c(1.5,.5); xpos <- xpos[-1:-2]
            text(xpos,apply(z[[i]],2,max)[-1]+1,pval2plot,cex=cex)
          }
        }
      }
    }
  }
)

setMethod("barplotSignifSignatures",signature(x="epheno",signatures="character"),
  function (x, signatures, referenceSignature, testUpDown=FALSE, simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05, ylab, ylim=ylim, cex.text=1, ...) {
    signatures <- list(mySign=signatures)
    barplotSignifSignatures(x, signatures, referenceSignature, testUpDown, simulate.p.value, B, p.adjust.method, alpha, ylab, ylim, cex.text,...)
  }
)

setMethod("barplotSignifSignatures",signature(x="epheno",signatures="GeneSetCollection"),
  function (x, signatures, referenceSignature, testUpDown=FALSE, simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05, ylab, ylim=ylim, cex.text=1, ...) {
    signatures <- geneIds(signatures)
    barplotSignifSignatures(x, signatures, referenceSignature, testUpDown, simulate.p.value, B, p.adjust.method, alpha, ylab, ylim, cex.text,...)
  }
)

setMethod("barplotSignifSignatures",signature(x="epheno",signatures="GeneSet"),
  function (x, signatures, referenceSignature, testUpDown=FALSE, simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05, ylab, ylim=ylim, cex.text=1, ...) {
    signatures <- geneIds(signatures)
    barplotSignifSignatures(x, signatures, referenceSignature, testUpDown, simulate.p.value, B, p.adjust.method, alpha, ylab, ylim, cex.text,...)
  }
)


setGeneric("barplotSignatures",function (x, signatures, referenceSignature, alpha=.05, p.adjust.method='none', ylab, cex.text=1, ...) standardGeneric("barplotSignatures"))

setMethod("barplotSignatures",signature(x="epheno",signatures="list"),
  function (x, signatures, referenceSignature, alpha=.05, p.adjust.method='none', ylab, cex.text=1, ...) {
    if (!missing(referenceSignature)) if (!(referenceSignature %in% names(signatures))) stop('referenceSignature not in names(signatures)')
    lensig <- unlist(lapply(signatures,function(x) length(x)))
    if (all(lensig==0)) stop('All signatures are empty. Check the argument signatures.\n')
    if (any(lensig==0)) {
      warning('Removed some signatures with no genes\n')
      signatures <- signatures[lensig>0]
    }
    badsignature <- sapply(signatures,function(y) !(any(y %in% featureNames(x))))
    if (any(badsignature)) stop(paste('Identifiers in signatures',paste(names(signatures)[which(badsignature)],collapse=','),'not in featureNames'))
    if (missing(ylab)) {
      ylab <- ifelse(length(signatures)==1,'log2 Fold Change / Hazard Ratio',c(rep('log2 Fold Change',ncol(getFc(x))),rep('log Hazard Ratio',ncol(getHr(x)))))
    }
    fc2plot <- lapply(phenoNames(x), function(y) lapply(signatures, function(z) logFcHr(x)[featureNames(x) %in% z,phenoNames(x) %in% y]))
    names(fc2plot) <- phenoNames(x)
    if (length(signatures)==1) {
      groups <- factor(rep(names(fc2plot),each=length(fc2plot[[1]][[1]])))
      o <- names(fc2plot)[unlist(lapply(fc2plot,function(x) length(x)))>0]
      barplotCI(unlist(fc2plot),groups=groups,order=o,alpha=alpha,ylab=ylab,...); abline(h=0,lty=2)
      abline(h=0,lty=2)
    } else {
      for (i in 1:ncol(x)) {
        groups <- factor(rep(names(fc2plot[[i]]),unlist(lapply(fc2plot[[i]],function(x) length(x)))))
        o <- names(fc2plot[[i]])[unlist(lapply(fc2plot[[i]],function(x) length(x)))>0]
        myxpos <- barplotCI(unlist(fc2plot[[i]]),groups=groups,order=o,alpha=alpha,ylab=ylab[i],main=names(fc2plot)[i], ...)
        abline(h=0,lty=2)
        if (!missing(referenceSignature)) {
          sigs2test <- which(names(signatures)!=referenceSignature)
          pvals <- sapply(sigs2test, function(y) wilcox.test(fc2plot[[i]][[y]],fc2plot[[i]][[referenceSignature]])$p.value)
          pvals <- p.adjust(pvals,method=p.adjust.method)
          pvals <- ifelse(pvals<.0001,'P<0.0001',paste('P=',round(pvals,4),sep=''))
          text(myxpos[sigs2test],rep(par("usr")[4],length(sigs2test)),pvals,pos=1,cex=cex.text)
        }
      }
    }
  }
)

setMethod("barplotSignatures",signature(x="epheno",signatures="character"),
  function (x, signatures, referenceSignature, alpha=.05, p.adjust.method='none', ylab, cex.text=1, ...) {
    signatures <- list(mySign=signatures)
    barplotSignatures(x, signatures, referenceSignature, alpha, p.adjust.method, ylab, cex.text, ...)
  }
)

setMethod("barplotSignatures",signature(x="epheno",signatures="GeneSetCollection"),
  function (x, signatures, referenceSignature, alpha=.05, p.adjust.method='none', ylab, cex.text=1, ...) {
    tmpsignatures <- geneIds(signatures); names(tmpsignatures) <- names(signatures); signatures <- tmpsignatures
    barplotSignatures(x, signatures, referenceSignature, alpha, p.adjust.method, ylab, cex.text, ...)
  }
)

setMethod("barplotSignatures",signature(x="epheno",signatures="GeneSet"),
  function (x, signatures, referenceSignature, alpha=.05, p.adjust.method='none', ylab, cex.text=1, ...) {
    signatures <- geneIds(signatures)
    barplotSignatures(x, signatures, referenceSignature, alpha, p.adjust.method, ylab, cex.text, ...)
  }
)


setGeneric("boxplotSignatures",function (x, signatures, referenceSignature, outline=FALSE, cex.text=1, ...) standardGeneric("boxplotSignatures"))

setMethod("boxplotSignatures",signature(x="epheno",signatures="list"),
  function (x, signatures, referenceSignature, outline=FALSE, cex.text=1, ...) {
    if (!missing(referenceSignature)) if (!(referenceSignature %in% names(signatures))) stop('referenceSignature not in names(signatures)')
    lensig <- unlist(lapply(signatures,function(x) length(x)))
    if (all(lensig==0)) stop('All signatures are empty. Check the argument signatures.\n')
    if (any(lensig==0)) {
      warning('Removed some signatures with no genes\n')
      signatures <- signatures[lensig>0]
    }
    badsignature <- sapply(signatures,function(y) !(any(y %in% featureNames(x))))
    if (any(badsignature)) stop(paste('Identifiers in signatures',paste(names(signatures)[which(badsignature)],collapse=','),'not in featureNames'))
    for (i in 1:ncol(x)) {
      logfc <- logFcHr(x)
      fc2plot <- vector("list",length(signatures))
      fc2plot <- lapply(signatures,function(y) logfc[featureNames(x) %in% y])
      boxplot(fc2plot, outline=outline, ...); abline(h=0,lty=2)
      if (!missing(referenceSignature)) {
        sigs2test <- which(names(signatures)!=referenceSignature)
        pvals <- double(length(sigs2test))
        pvals <- unlist(lapply(sigs2test, function(x) wilcox.test(fc2plot[[x]],fc2plot[[referenceSignature]])$p.value))
        pvals <- ifelse(pvals<.0001,'P<0.0001',paste('P=',round(pvals,4),sep=''))
        text(sigs2test,rep(par("usr")[4],length(sigs2test)),pvals,pos=1,cex=cex.text)
      }
    }
  }
)

sortDragHtmlTable <- function(filename) {
#Adds javascript to html to enable sorting.
# INPUT
# - filename: html file to modify.

#get outputdir from filename
lastSlashPos <- gregexpr(.Platform$file.sep,filename)[[1]][length(gregexpr(.Platform$file.sep,filename)[[1]])]
outputdir <- ifelse(lastSlashPos== -1, getwd(), substr(filename,0,lastSlashPos))
  
#import javascript files
st <- system.file("javascript", "sorttable.js", package = "phenoTest")
dt <- system.file("javascript", "dragtable.js", package = "phenoTest")
file.copy(st,outputdir)
file.copy(dt,outputdir)

#rewrite html file
tmpTxt <- readLines(filename)
tmpTxt[1] <- paste('<script src="sorttable.js"></script>\n',tmpTxt[1])
tmpTxt[1] <- paste('<script src="dragtable.js"></script>\n',tmpTxt[1])
tmpTxt <- sub('TABLE','TABLE class="draggable sortable"',tmpTxt)
writeLines(tmpTxt,con=filename)
}

epheno2html <- function(x,epheno,outputdir,prefix='',genelimit=50,categories=3,withPlots=TRUE,mc.cores=1) {
  require(paste(annotation(x),'.db',sep=''),character.only = TRUE)
  stopifnot(annotation(x)==annotation(epheno))
  stopifnot(identical(featureNames(x),featureNames(epheno))  )
  if (!(categories %in% c(2,3,4))) stop('categories has to be 2, 3 or 4!')
  if (genelimit>nrow(x)) genelimit <- nrow(x)
  stopifnot(file.exists(outputdir))
  id.entrezid <- 'org.' %in% substr(annotation(x),0,4)
    
  export2html <- function(varName,x,epheno,varType,categories,outputdir) {
    #keep only columns that contain our variable
    epheno <-  epheno[,varName]

    #make xls file
    if (id.entrezid) {
      probeid <- rep(NA,nrow(x))
      entrezid <- featureNames(x)
    } else {
      probeid <- featureNames(x)
      envir <- eval(parse(text=paste(annotation(x),'ENTREZID',sep='')))
      entrezid <- as.character(AnnotationDbi::mget(featureNames(x),envir,ifnotfound=NA))
    }
    envir <- eval(parse(text=paste(annotation(x),'SYMBOL',sep='')))
    symbol <- as.character(AnnotationDbi::mget(featureNames(x),envir,ifnotfound=NA))
    envir <- eval(parse(text=paste(annotation(x),'GENENAME',sep='')))
    genename <- as.character(AnnotationDbi::mget(featureNames(x),envir,ifnotfound=NA))
    if (varType=='survival') {
      xout <- data.frame(probeid,entrezid,symbol,genename,getSummaryDif(epheno)[featureNames(x),,drop=FALSE],p.value=as.numeric(getSignif(epheno)[featureNames(x),,drop=FALSE]))
    } else {
      xout <- data.frame(probeid,entrezid,symbol,genename,getMeans(epheno)[featureNames(x),,drop=FALSE],getSummaryDif(epheno)[featureNames(x),,drop=FALSE],p.value=as.numeric(getSignif(epheno)[featureNames(x),,drop=FALSE]))
    }
    if (id.entrezid) xout <- xout[,-1]
    if (approach(epheno)=='bayesian') colnames(xout)[grepl('p.value',colnames(xout))] <- 'postProb'
    write.csv(xout,paste(outputdir,'/',ifelse(prefix=='','',paste(prefix,'_',sep='')),varType,'_',varName,'.csv',sep=''),row.names=FALSE)
    if (approach(epheno)=='frequentist') xout <- xout[order(xout$p.value),] else xout <- xout[order(xout$postProb,decreasing=TRUE),]
    
    #keep amount of genes equal to genelimit
    epheno <- epheno[order(getSignif(epheno),decreasing=(approach(epheno)=='bayesian')),]
    epheno <- epheno[1:genelimit,]
    x <- x[featureNames(epheno),]
    if (id.entrezid) sel <- match(featureNames(epheno),entrezid) else sel <- match(featureNames(epheno),probeid)
    probeid <- probeid[sel]
    entrezid <- entrezid[sel]
    symbol <- symbol[sel]
    genename <- genename[sel]
    if (id.entrezid) xout <- xout[xout$entrezid %in% entrezid,] else xout <- xout[xout$probeid %in% probeid,]
    
    #remove samples with nulls on pData
    x <- x[,!is.na(pData(x)[,varName])]
    
    #make plots
    if (withPlots) {
      if (!file.exists(paste(outputdir,'phenoPlots',sep='/'))) dir.create(paste(outputdir,'phenoPlots',sep='/')) 
      geneCategories <- t(apply(exprs(x), 1, function(x) as.numeric(cut2(x,g=categories)))); colnames(geneCategories) <- colnames(exprs(x))
      #make plots
      dirOut <- paste(outputdir,'/phenoPlots/',varName,sep=''); if (!file.exists(dirOut)) dir.create(dirOut)
      if (varType=='survival') {
        varName.time <- as.character(unique(pData(epheno)[varName,'survTime']))
        varName.event <- varName
        s <- Surv(as.numeric(pData(x)[,varName.time]),pData(x)[,varName.event])
        fit <- apply(geneCategories, 1, function(x) survfit(s~x))
        if (categories==2) uniqueCat <- c('Low','High') else if (categories==3) uniqueCat <- c('Low','Medium','High') else if (categories==4) uniqueCat <- c('Low','Medium low','Medium high','High')
        myFun <- function(i) {
          png(paste(dirOut,'/survival_',gsub('/','_',featureNames(x)[i]),'.png',sep=''))
          if (categories==2) {
            plot(fit[[i]],ylab='Survival',xlab=paste('Time to ',varName,sep=''), col=c('green','red'))
            legend("bottomleft",uniqueCat,col=c('green','red'),lty=1)
          } else if (categories==3) {
            plot(fit[[i]],ylab='Survival',xlab=paste('Time to ',varName,sep=''), col=c('green','black','red'))
            legend("bottomleft",uniqueCat,col=c('green','black','red'),lty=1)
          } else if (categories==4) {
            plot(fit[[i]],ylab='Survival',xlab=paste('Time to ',varName,sep=''), col=c('green','blue','yellow','red'))
            legend("bottomleft",uniqueCat,col=c('green','blue','yellow','red'),lty=1)
          }
          title(paste(ifelse(approach(epheno)=='frequentist','p-value','postProb'),ifelse(getSignif(epheno)[i,]<.0001,'<0.0001',paste('=',round(getSignif(epheno)[i,],4),sep='')),sep=''))
          dev.off()
          png(paste(dirOut,'/smooth_',gsub('/','_',featureNames(x)[i]),'.png',sep=''))
          dummy <- try(smoothCoxph(pData(x)[,varName.time],pData(x)[,varName.event],as.numeric(exprs(x[i,]))),silent=T)
          dev.off()
        }
        lapply(as.list(1:nrow(x)),myFun)
      }
      myFun <- function(i) {
        png(paste(dirOut,'/plot_',gsub('/','_',featureNames(x)[i]),'.png',sep=''))
        if (varType=='survival') {
          if (ncol(x)>20) {
            boxplot(as.numeric(exprs(x[i,])) ~ as.factor(pData(x)[,varName.event]),ylab='Expression level',xlab=varName.time)
          } else {
            dotchart(as.numeric(exprs(x[i,])),as.factor(pData(x)[,varName.event]),xlab='Expression level',ylab=varName.time,color=as.numeric(as.factor(pData(x)[,varName.event]))+1,groups=as.factor(pData(x)[,varName.event]))
          }
        } else if (varType=='continuous') {
          mylevels <- levels(pData(epheno)[pData(epheno)$phenoName==varName,'meanLabel'])
          mylevels <- gsub('\\[','',gsub('\\]','',gsub('\\)','',mylevels)))
          mylevels <- strsplit(mylevels,',')
          mylevels <- lapply(mylevels,as.numeric)
          myContPheno <- pData(x)[,varName]
          myCateg <- vector('numeric',length=length(myContPheno))
          for (j in 1:length(mylevels)) {
            sel <- myContPheno >= mylevels[[j]][1] & myContPheno <= mylevels[[j]][2]
            myCateg[sel] <- j
          }
          myCateg <- factor(myCateg,labels=levels(pData(epheno)[pData(epheno)$phenoName==varName,'meanLabel']))
          if (ncol(x)>20) {
            boxplot(as.numeric(exprs(x[i,])) ~ myCateg,ylab='Expression level',xlab=varName)
          } else {
            dotchart(as.numeric(exprs(x[i,])),as.factor(myCateg),xlab='Expression level',ylab=varName,color=as.numeric(as.factor(myCateg))+1,groups=as.factor(myCateg))
          }
        } else {
          if (ncol(x)>20) {
            boxplot(as.numeric(exprs(x[i,])) ~ as.factor(pData(x)[,varName]),ylab='Expression level',xlab=varName)
          } else {
            dotchart(as.numeric(exprs(x[i,])),as.factor(pData(x)[,varName]),xlab='Expression level',ylab=varName,color=as.numeric(as.factor(pData(x)[,varName]))+1,groups=as.factor(pData(x)[,varName]))
          }
        }
        title(paste(ifelse(approach(epheno)=='frequentist','p-value','postProb'),ifelse(getSignif(epheno)[i,]<.0001,'<0.0001',paste('=',round(getSignif(epheno)[i,],4),sep='')),sep=''))
        dev.off()
      }
      lapply(as.list(1:nrow(xout)),myFun)
    }

    #make html file
    xout[is.na(xout)] <- '---'
    if (withPlots) {
      if (varType=='survival') {
        txt <- rep('Open',nrow(xout))
        xout <- data.frame(xout[,c(1:(ifelse(id.entrezid,3,4)))],Plot=txt,Kaplan=txt,SmoothedHR=txt,xout[,c((ifelse(id.entrezid,4,5)):ncol(xout))])
      } else {
        xout <- data.frame(xout[,c(1:(ifelse(id.entrezid,3,4)))],plot=rep('Open',nrow(xout)),xout[,c((ifelse(id.entrezid,4,5)):ncol(xout))])
      }
    }
    links <- tiny.pic <- vector('list',length=ncol(xout))
    if (!id.entrezid) links[[1]] <- paste('https://www.affymetrix.com/LinkServlet?probeset=',xout$probeid,sep='')
    links[[ifelse(id.entrezid,1,2)]] <- paste('http://www.ncbi.nlm.nih.gov/sites/entrez?db=gene&cmd=Retrieve&dopt=Graphics&list_uids=',xout$entrezid,sep='')
    links[[ifelse(id.entrezid,2,3)]] <- paste('http://www.genecards.org/index.php?path=/Search/keyword/',xout$symbol,sep='')
    if (withPlots) {
      tiny.pic[[ifelse(id.entrezid,4,5)]] <- links[[ifelse(id.entrezid,4,5)]] <- paste('phenoPlots/',varName,'/plot_',gsub('/','_',featureNames(x)),'.png',sep='')
      if (varType=='survival') {
        tiny.pic[[ifelse(id.entrezid,5,6)]] <- links[[ifelse(id.entrezid,5,6)]] <- paste('phenoPlots/',varName,'/survival_',gsub('/','_',featureNames(x)),'.png',sep='')
        tiny.pic[[ifelse(id.entrezid,6,7)]] <- links[[ifelse(id.entrezid,6,7)]] <- paste('phenoPlots/',varName,'/smooth_',gsub('/','_',featureNames(x)),'.png',sep='')
      }
    }
    file <- paste(outputdir,'/',ifelse(prefix=='','',paste(prefix,'_',sep='')),varType,'_',varName,'.html',sep='')
    title <- paste(varName,' (',varType,' variable)',sep='')
    write.html(xout,file=file,links=links,tiny.pic=tiny.pic,title=title)
  }
   
  myFun <- function(varName) {
    varType <- as.character(phenoClass(epheno[,varName]))
    export2html(varName=varName,x=x,epheno=epheno,varType=varType,categories=categories,outputdir=outputdir)
  }
  varName <- as.list(phenoNames(epheno))
  if (mc.cores==1) {
    dummy <- lapply(varName, myFun)
  } else {
    if ('parallel' %in% loadedNamespaces()) {
      dummy <- parallel::mclapply(varName, myFun, mc.cores=mc.cores)
    } else stop('parallel library has not been loaded!')
  }

  ##TO DEBUG USE for INSTEAD OF mclapply
  #for (i in 1:ncol(epheno)) {
  #  varName <- phenoNames(epheno)
  #  varType <- as.character(phenoClass(epheno)[phenoClass(epheno)==varName,'phenoClass'])
  #  export2html(varName=varName,x=x,epheno=epheno,varType=varType,categories=categories,outputdir=outputdir)
  #}
}

\documentclass[a4paper,12pt]{article}
%\VignetteIndexEntry{Manual for the phenoTest library}
%\VignettePackage{phenoTest,GSEABase}


\usepackage{amsmath}    % need for subequations
\usepackage{amssymb}    %useful mathematical symbols
\usepackage{bm}         %needed for bold greek letters and math symbols
\usepackage{graphicx}   % need for PS figures
\usepackage{hyperref}   % use for hypertext links, including those to external documents and URLs
\usepackage{natbib}    %number and author-year style referencing

\begin{document}

\title{Test association between phenotype and gene expression}
\author{Evarist Planet \\
\small{Bioinformatics \& Biostatistics Unit} \\
\small{IRB Barcelona}}
\date{}  %comment to include current date
\maketitle

\tableofcontents

\section{Introduction}
\label{sec:intro}

Imagine a situation where we have gene expression and phenotype
variables and we want to test the association of each gene with
phenotype. We would probably be interested in testing association of 
groups of genes (or gene sets) with phenotype. This library provides
the tools to do both things in a way that is efficient, structured,
fast and scalable. We also provide tools to do \emph{GSEA} (Gene set
enrichment analysis) of all phenotype variables at once.

The functions and methods presented on this \texttt{vignette} provide
tools to easily test association between gene expression levels of
individual genes or gene sets of genes and the selected phenotypes of
a given gene expression dataset. These can be particularly useful for
datasets arising from RNAseq or microarray gene expression studies. 

We will load the \texttt{ExpressionSet} of a cohort (\emph{GSE2034})
we downloaded from \href{http://www.ncbi.nlm.nih.gov/geo/}{GEO}. 

\begin{scriptsize}
<<loadLibAndData>>=
options(width=100)
library(phenoTest)
data(eset)
eset
@ 
\end{scriptsize}

For illustration purposes we selected the first 1000 genes and the
first 100 samples, created a continuous variable (named
\emph{Tumor.size}) and added a new category to the categorical
variable \emph{lymph.node.status} to illustrate the functionality of
the package.

\begin{scriptsize}
<<preprocess>>=
Tumor.size <- rnorm(ncol(eset),50,2)
pData(eset) <- cbind(pData(eset),Tumor.size)
pData(eset)[1:20,'lymph.node.status'] <- 'positive'
@
\end{scriptsize}

\section{Individual gene(s) association with phenotype(s)}
\label{sec:Preprocess}

\subsection{Creating an \texttt{epheno}}

The \texttt{epheno} object will contain the univariate association between a list of phenotype 
variables and the gene expression from the given \texttt{ExpressionSet}. 
We will use the \texttt{ExpressionPhenoTest} function to create the \texttt{epheno} object.
We will have to tell this function which phenotype variables we want to test and the type of
these variables (if they are \emph{ordinal}, \emph{continuous}, \emph{categorical} or \emph{survival}
variables). 
For this purpose we will create a variable called (for instance) \texttt{vars2test}. 
This variable has to be of class \texttt{list} with components \emph{continuous}, \emph{categorical}, 
\emph{ordinal} and \emph{survival} indicating which phenotype variables should be tested. \emph{continuous},
\emph{categorical} and \emph{ordinal} must be character vectors, \emph{survival} a matrix with columns
named \emph{time} and \emph{event}. 
The names must match names in \texttt{names(pData(eset))} (being
\texttt{eset} the \texttt{ExpressionSet} of the cohort we are
interested in). 

\begin{scriptsize}
<<setVars2test>>=
head(pData(eset))
survival <- matrix(c("Relapse","Months2Relapse"),ncol=2,byrow=TRUE)
colnames(survival) <- c('event','time')
vars2test <- list(survival=survival,categorical='lymph.node.status',continuous='Tumor.size')
vars2test
@ 
\end{scriptsize}
  
Now we have everything we need to create the \texttt{epheno} object:

\begin{scriptsize}
<<runExpressionPhenoTest>>=
epheno <- ExpressionPhenoTest(eset,vars2test,p.adjust.method='none')
epheno
@ 
\end{scriptsize}
  
P values can also be adjusted afterwards:

\begin{scriptsize}
<<pValueAdjust>>=
p.adjust.method(epheno)
epheno <- pAdjust(epheno,method='BH')
p.adjust.method(epheno)
@ 
\end{scriptsize}
  
The \texttt{epheno} object extends the \texttt{ExpressionSet} object and therefore methods that are
available for \texttt{ExpressionSet}s are also available for \texttt{epheno}s.

The effect of both \emph{continuous}, \emph{categorical} and \emph{ordinal} phenotype variables on gene
expression levels are tested via \texttt{lmFit} from package \texttt{limma} (\cite{smyth:2005}). For
\emph{ordinal} variables a single coefficient is used to test its effect on gene expression (trend
test), which is then used to obtain a P-value. Gene expression effects on \emph{survival} are tested
via Cox proportional hazards model (\cite{cox:1972}), as implemented in function \texttt{coxph} from
package \texttt{survival}.

If we want we can compute posterior probabilities instead of pvalues
we can set the argument \texttt{approach}='bayesian'. 
The default value is 'frequentist'.

\texttt{ExpressionPhenoTest} implements parallel computing via the function \texttt{mclapply} from
the package \texttt{multicore}. Currently \texttt{multicore} only operates on Unix systems. If you
are a windows user you should set \texttt{mc.cores}=1 (the default).

\subsection{Useful methods for the \texttt{epheno} object}

Some of the methods for the epheno objects are shown here.

The object can be subseted by phenotype names:

\begin{scriptsize}
<<ephenoSubsetByName>>=
phenoNames(epheno)
epheno[,'Tumor.size']
epheno[,2]
@ 
\end{scriptsize}
  
or by class (class can be ordinal, continuous, categorical or survival):

\begin{scriptsize}
<<ephenoSubsetByClass>>=
phenoClass(epheno)
epheno[,phenoClass(epheno)=='survival']
@ 
\end{scriptsize}
  
\texttt{epheno} objects contain information summarizing the association between genes and
phenotypes. \texttt{getMeans} can be used to obtain the average expression for each group in
categorical and ordinal variables, as well as for categorized version of the continuous variables.

\begin{scriptsize}
<<ephenoGetMeans>>=
head(getMeans(epheno))
@ 
\end{scriptsize}

Here we see that tumor size has been categorized into 3 groups. The
number of categories can be changed with the argument
\texttt{continuousCategories} in the call to
\texttt{ExpressionPhenoTest}. 

\texttt{epheno} objects also contain fold changes and hazard ratios (for survival variables). These
can be accessed with \texttt{getSummaryDif}, \texttt{getFc} and \texttt{getHr}.

\begin{scriptsize}
<<ephenoGetSummaries>>=
head(getSummaryDif(epheno))
head(getFc(epheno))
head(getHr(epheno))
@ 
\end{scriptsize}

\texttt{ExpressionPhenoTest} also computes P-values. \texttt{eBayes} from \texttt{limma} package is
used for continuous, categorical and ordinal phenotypes. A Cox proportional hazards likelihood-ratio
test is used for survival phenotypes. P-values can be accessed with \texttt{getSignif}. Notice that a
single P-value is reported for each phenotype variable. For categorical variables these corresponds
to the overall null hypothesis that there are no differences between groups.

\begin{scriptsize}
<<ephenoGetSignif>>=
head(getSignif(epheno))
@ 
\end{scriptsize}

We can also ask for the variables we sent to the \texttt{ExpressionPhenoTest} function:

\begin{scriptsize}
<<ephenoGetVars2test>>=
getVars2test(epheno)
@ 
\end{scriptsize}

\subsection{Export an \texttt{epheno}}

Functions \texttt{export2csv} and \texttt{epheno2html} can be used to export to a comma separated value
(csv) or an html file. The html file will have useful links to online databases that will provide information
about each known gene. For more information about how to use these functions and examples read their help manuals.

\section{Gene set(s) association with phenotype(s)}

Gene sets can be stored in a list object. Each element of the list will contain one gene set.
The names of the list will be the names of the gene sets. Here we select genes at random to build our
gene sets:

\begin{scriptsize}
<<getSignatures>>=
set.seed(777)
sign1 <- sample(featureNames(eset))[1:20]
sign2 <- sample(featureNames(eset))[1:50]
mySignature <- list(sign1,sign2)
names(mySignature) <- c('My first signature','Another signature')
mySignature
@ 
\end{scriptsize}

Gene sets can also be stored in gene set collection objects. From here
on all functions have methods for gene sets stored as \texttt{list}s,
\texttt{GeneSet}s or \texttt{GeneSetCollection}s. You can use the one
you feel more confortable with. We will work with
\texttt{GeneSetCollection}:

\begin{scriptsize}
<<makeGeneSets>>=
library(GSEABase)
myGeneSetA <- GeneSet(geneIds=sign1, setName='My first signature')
myGeneSetB <- GeneSet(geneIds=sign2, setName='Another signature')
mySignature <- GeneSetCollection(myGeneSetA,myGeneSetB)
mySignature
@ 
\end{scriptsize}

\subsection{Plots that use \texttt{epheno} as input}

\texttt{barplotSignifSignatures} will plot the percentage of
up regulated and down regulated genes that are statistically
significant in each signature. In our random selection of genes we did
not find any statistically significant genes. Therefore, and just to
show the plot we set the alpha value 0.99. The plot can be seen in
Figure \ref{fig:barplotSignifSignatures}.  

\begin{scriptsize}
\setkeys{Gin}{width=0.6\textwidth}
<<label=barplotSignifSignatures,include=FALSE>>=
barplotSignifSignatures(epheno[,'lymph.node.status'],mySignature,alpha=0.99)
@                                                                                                                                               
\begin{figure}
\begin{center}
<<label=barplotSignifSignatures,fig=TRUE,echo=FALSE>>=
<<barplotSignifSignatures>>
@                                                                                                                                               
\end{center}
\caption{\texttt{barplotSignifSignatures}: Number of diferentially
  expressed genes in each gene set that are statistically
  significant. P-values test for differences in each signature between
  the number of up and down regulated genes.}
\label{fig:barplotSignifSignatures}
\end{figure}
\end{scriptsize}

By default \texttt{barplotSignifSignatures} performs a binomial test
(\texttt{binom.test} from package \texttt{stats}) for each signature
to test if the proportions of up regulated and down regulated genes
are different.
For example, Figure \ref{fig:barplotSignifSignatures} indicates that in the
first signature the proportion of up regulated genes is higher than
the proportion of down regulated genes. The second signature shows no
significant statistical differences.

Sometimes we want to compare the proportions of up and down regulated
genes in our signature with the proportions of up and down regulated
of all genes in the genome. In this case we may provide a reference
signature via the argument \texttt{referenceSignature}. When providing the
\texttt{referenceSignature} argument a chi-square test comparing the
proportion of up and down regulated genes in each signature with the 
proportion in the reference set will be computed.

When a reference gene set is provided and parameter
\texttt{testUpDown} is \texttt{TRUE} (by default it is \texttt{FALSE})
the proportion of up regulated genes is compared with those of the
reference gene set. The same is done for down regulated genes.

\texttt{barplotSignatures} plots the average log2 fold change or
hazard ratio of each phenotype for each gene set. Figure
\ref{fig:barplotSignatures} shows an example of it.

\begin{scriptsize}
\setkeys{Gin}{width=0.6\textwidth} 
<<label=barplotSignatures,include=FALSE>>=
barplotSignatures(epheno[,'Tumor.size'],mySignature, ylim=c(0,1))
@ 
\begin{figure}
\begin{center}
<<label=barplotSignatures,fig=TRUE,echo=FALSE>>=
<<barplotSignatures>>
@                                                                                                                                                     
\end{center}
\caption{\texttt{barplotSignatures}: Averge fold change or hazard ratio.}
\label{fig:barplotSignatures}
\end{figure}
\end{scriptsize}

We can also cluster our samples in two clusters based on the expression levels of one gene set of genes and
then test the effect of cluster on phenotypes. For \emph{ordinal} and
\emph{continuous} variables a Kruskal-Wallis Rank Sum test is used, for \emph{categorical} variables
a chi-square test is used and for \emph{survival} variables a Cox proportional hazards likelihood-ratio
test is used. The \texttt{heatmapPhenoTest} function can be used to this end. Its results can be seen
in Figure \ref{fig:heatmap} and \ref{fig:kaplan}. 

\begin{scriptsize}
\setkeys{Gin}{width=0.6\textwidth} 
<<label=heatmapPhenoTestHeat,include=FALSE>>=
pvals <- heatmapPhenoTest(eset,mySignature[[1]],vars2test=vars2test[1],heat.kaplan='heat')
@ 

\begin{scriptsize}
<<heatmapPhenoTestHeatPvals>>=
pvals
@ 
\end{scriptsize}

\begin{figure}
\begin{center}
<<label=heatmapPhenoTestHeat,fig=TRUE,echo=FALSE>>=
<<heatmapPhenoTestHeat>>
@                                                                                                                                                     
\end{center}
\caption{Heatmap produced with \texttt{heatmapPhenoTest} function. All
  variables in \texttt{vars2test} that are of class \emph{logical}
  will be plotted under the heatmap.} 
\label{fig:heatmap}
\end{figure}
\end{scriptsize}

\begin{scriptsize}
\setkeys{Gin}{width=0.6\textwidth} 
<<label=heatmapPhenoTestKaplan,include=FALSE>>=
pvals <- heatmapPhenoTest(eset,mySignature[[1]],vars2test=vars2test[1],heat.kaplan='kaplan')
@ 

\begin{scriptsize}
<<heatmapPhenoTestKaplanPvals>>=
pvals
@ 
\end{scriptsize}

\begin{figure}
\begin{center}
<<label=heatmapPhenoTest2,fig=TRUE,echo=FALSE>>=
<<heatmapPhenoTestKaplan>>
@                                                                                                                                                     
\end{center}
\caption{Kaplan-Meier produced with \texttt{heatmapPhenoTest} function.}
\label{fig:kaplan}
\end{figure}
\end{scriptsize}

\subsection{GSEA (Gene Set Enrichment Analysis)}

A popular way to test association between gene sets' gene expression and phenotype is GSEA (\cite{subramanian:2005}).
The main idea is to test the association between the gene set \emph{as a whole} and a phenotype.

Although GSEA and several extensions are already available in other \emph{Biconductor} packages,
here we implement a slightly different extension. Most GSEA-like approaches assess statistical
significance by permuting the values of the phenotype of interest. From a statisticall point of view
this tests the null hypothesis that no genes are associated with phenotype. However in many
applications one is actually interested in testing if the proportion of genes associated with
phenotype in the gene set is greater than that outside of the gene set. As a simple example, imagine
a cancer study where 25\% of the genes are differentially expressed. In this setup a randomly chosen
gene set will have around 25\% of differentially expressed genes, and classical GSEA-like approaches
will tend to flag the gene set as statistically significant. In contrast, our implementation will
tend to select only gene sets with more than 25\% of differentially expressed genes.

We will use the \texttt{gsea} method to compute \emph{enrichment
scores} (see \cite{subramanian:2005} for details about the enrichment
scores) and \emph{simulated enrichment scores} (by permuting the
selection of genes).
The \emph{simulated enrichment scores} are used
to compute P-values and FDR. 
We can summarize the results obtained using the \texttt{summary} method. 
The following chunk of code is an illustrative example of it:  

\begin{scriptsize}
<<gsea>>=
my.gsea <- gsea(x=epheno,gsets=mySignature,B=1000,p.adjust='BH')
my.gsea
summary.gseaData(my.gsea)
@ 
\end{scriptsize}

We receive one message for each phenotype we are testing.

We can produce plots as follows:

\begin{scriptsize}
<<gseaPlot>>=
plot.gseaData(my.gsea)
@ 
\end{scriptsize}

This will produce two plots (one for \emph{enrichment score} and another for \emph{normalised enrichment
score}) for every phenotype and gene set (in our case 12 plots). Following code shows an example on
plotting only \emph{enrichment score} for variable \emph{Relapse} on the first gene set of genes. Plot
can be seen in Figure \ref{fig:plotGsea}.

\begin{scriptsize}
<<label=plotGseaPrepare,include=FALSE>>=
my.gsea <- gsea(x=epheno[,'Relapse'],gsets=mySignature[1],B=100,p.adjust='BH')
summary.gseaData(my.gsea)
@ 
<<label=plotGseaEs,include=FALSE>>=
plot.gseaData(my.gsea,es.nes='es',selGsets='My first signature')  
@ 
\begin{figure}
\begin{center}
<<label=plotGsea,fig=TRUE,echo=FALSE>>=
<<plotGseaEs>>
@                                                                                                                                                     
\end{center}
\caption{GSEA plot.}
\label{fig:plotGsea}
\end{figure}
\end{scriptsize}

\texttt{gsea} can be used not only with \texttt{epheno} objects but also with objects of class \texttt{numeric}
or \texttt{matrix}. For more information read the \texttt{gsea} function help.

Following similar ideas to \cite{virtaneva:2001} we also implemented a
Wilcoxon test. This can be used instead of the permutation test which
can be slow if we use a lot of permutations and we can not use the
\texttt{multicore} package. The plot we will obtain will also be
different. Instead of plotting the \emph{enrichment scores} we will
plot the density function and the mean log2 fold change or hazard
ratio of the genes that belong to our gene set. This will allow us to
compare how similar/different from 0 the mean of our gene set is. The
plot using Wilcoxon test can be seen in Figure
\ref{fig:plotGseaWilcox}.  

\begin{scriptsize}
<<plotGseaWilcoxPrepare>>=
my.gsea <- gsea(x=epheno[,'Relapse'],gsets=mySignature,B=100,test='wilcox',p.adjust='BH')
summary.gseaData(my.gsea)
@
<<label=plotGseaWilcox,include=FALSE>>=
plot.gseaData(my.gsea,selGsets='My first signature')
@ 
\begin{figure}
\begin{center}
<<label=plotGseaWilcox,fig=TRUE,echo=FALSE>>=
<<plotGseaWilcox>>
@                                                                                                                                                     
\end{center}
\caption{GSEA plot using Wilcoxon test.}
\label{fig:plotGseaWilcox}
\end{figure}
\end{scriptsize}

Notice that using a Wilcoxon test is conceptually very similar to the average gene set fold change
presented in figure \ref{fig:barplotSignatures}.

A current limitation of \texttt{gseaSignatures} is that it does not consider the existance of
dependence between genes in the gene set. This will be addressed in future versions. Nevertheless we
believe \texttt{gseaSignatures} is usefull in that it targets the correct null hypothesis that gene
set is as enriched as a randomly selected gene set, opposed to testing that there are no enriched
genes in the set as is done in GSEA.

\bibliographystyle{plainnat}
\bibliography{references}

\end{document}

\name{gseaSignatures-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaSignatures-class}
\title{Class "gseaSignatures" ES and EsSim container.}
\description{This object contains de ES (enrichment scores) and
  simulated ES that will be used in the GSEA (Gene Set Enrichment
  Analysis) process.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaSignatures", ...)}. 
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"} .}
    \item{\code{es}:}{Object of class \code{"numeric"} Contains the
      observed enrichment scores. The ones that were compted from the
      data without permuting anything.  }
    \item{\code{es.sim}:}{Object of class \code{"numeric"} Contains the
      enrichment score that were obtained after permutations. }
    \item{\code{signature}:}{Object of class \code{"numeric"} The subset
      of genes we are interested in. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
No methods defined with class "gseaSignatures" in the signature.
}
\author{Evarist Planet}
\examples{
showClass("gseaSignatures")
}
\keyword{classes}

\name{eset}
\alias{eset}
\docType{data}
\title{
Example data.
}
\description{
Example data of class ExpressionSet. 
}
\usage{data(eset)}
\format{
  The format is:
Formal class 'ExpressionSet' [package "Biobase"] with 7 slots
  ..@ assayData        :<environment: 0x1050d9390> 
  ..@ phenoData        :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	7 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr [1:7] NA NA NA NA ...
  .. .. ..@ data             :'data.frame':	286 obs. of  7 variables:
  .. .. .. ..$ PID              : int [1:286] 3 5 6 7 8 9 11 14 15 17 ...
  .. .. .. ..$ GEOaccession     : Factor w/ 286 levels "GSM36777","GSM36778",..: 17 20 21 22 24 25 58 59 60 61 ...
  .. .. .. ..$ lymph.node.status: chr [1:286] "negative" "negative" "negative" "negative" ...
  .. .. .. ..$ Months2Relapse   : int [1:286] 101 118 9 106 37 125 109 14 99 137 ...
  .. .. .. ..$ Relapse          : int [1:286] 0 0 1 0 1 0 0 1 0 0 ...
  .. .. .. ..$ ER.Status        : num [1:286] 0 1 0 0 0 1 1 0 1 1 ...
  .. .. .. ..$ BrainRelapse     : int [1:286] 0 0 0 0 0 0 0 0 0 0 ...
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ featureData      :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	16 obs. of  3 variables:
  .. .. .. ..$ Column          : chr [1:16] "ID" "GB_ACC" "SPOT_ID" "Species Scientific Name" ...
  .. .. .. ..$ Description     : Factor w/ 15 levels "","A gene symbol, when one is available (from UniGene).",..: 3 5 15 13 12 1 11 1 10 14 ...
  .. .. .. ..$ labelDescription: chr [1:16] NA NA NA NA ...
  .. .. ..@ data             :'data.frame':	1000 obs. of  16 variables:
  .. .. .. ..$ ID                              : Factor w/ 22284 levels "1007_s_at","1053_at",..: 1 2 3 4 5 6 7 8 9 10 ...
  .. .. .. ..$ GB_ACC                          : Factor w/ 21129 levels "AF052179","AF061832",..: 93 30 95 97 25 24 96 99 28 20 ...
  .. .. .. ..$ SPOT_ID                         : chr [1:1000] NA NA NA NA ...
  .. .. .. ..$ Species.Scientific.Name         : Factor w/ 2 levels "Homo sapiens",..: 1 1 1 1 1 1 1 1 1 1 ...
  .. .. .. ..$ Annotation.Date                 : Factor w/ 2 levels "Jul 11, 2007",..: 1 1 1 1 1 1 1 1 1 1 ...
  .. .. .. ..$ Sequence.Type                   : Factor w/ 4 levels "Consensus sequence",..: 2 2 2 2 2 2 2 2 2 2 ...
  .. .. .. ..$ Sequence.Source                 : Factor w/ 3 levels "Affymetrix Proprietary Database",..: 1 2 1 2 1 2 1 1 2 1 ...
  .. .. .. ..$ Target.Description              : Factor w/ 21363 levels "Consensus includes gb:AI656011 /FEA=EST /DB_XREF=gi:4739990 /DB_XREF=est:tt42e08.x1 /CLONE=IMAGE:2243462 /UG=Hs.116875 KIAA0156"| __truncated__,..: 16 13 18 20 8 7 19 22 11 4 ...
  .. .. .. ..$ Representative.Public.ID        : Factor w/ 21197 levels "AF052179","AF061832",..: 93 30 95 97 25 24 96 99 28 20 ...
  .. .. .. ..$ Gene.Title                      : Factor w/ 14208 levels "ADP-ribosylation factor 1",..: 35 66 46 60 44 97 96 64 26 33 ...
  .. .. .. ..$ Gene.Symbol                     : Factor w/ 13293 levels "ABCF1","ARF1",..: 20 59 40 53 33 96 94 58 15 18 ...
  .. .. .. ..$ ENTREZ_GENE_ID                  : chr [1:1000] "780" "5982" "3310" "7849" ...
  .. .. .. ..$ RefSeq.Transcript.ID            : Factor w/ 13074 levels "NM_000409","NM_000661 /// NM_001024921",..: 37 45 41 52 1 50 49 82 47 4 ...
  .. .. .. ..$ Gene.Ontology.Biological.Process: Factor w/ 7245 levels "","0000074 // regulation of progression through cell cycle // traceable author statement /// 0006139 // nucleobase, nucleoside, nu"| __truncated__,..: 61 22 78 32 79 60 14 63 72 20 ...
  .. .. .. ..$ Gene.Ontology.Cellular.Component: Factor w/ 4148 levels "","0000502 // proteasome complex (sensu Eukaryota) // traceable author statement /// 0005634 // nucleus // inferred from electroni"| __truncated__,..: 72 45 1 44 1 1 42 71 6 68 ...
  .. .. .. ..$ Gene.Ontology.Molecular.Function: Factor w/ 7314 levels "","0000049 // tRNA binding // non-traceable author statement /// 0000166 // nucleotide binding // inferred from electronic annotat"| __truncated__,..: 23 26 27 40 81 18 39 71 74 69 ...
  .. .. ..@ dimLabels        : chr [1:2] "featureNames" "featureColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ experimentData   :Formal class 'MIAME' [package "Biobase"] with 13 slots
  .. .. ..@ name             : chr ""
  .. .. ..@ lab              : chr ""
  .. .. ..@ contact          : chr ""
  .. .. ..@ title            : chr ""
  .. .. ..@ abstract         : chr ""
  .. .. ..@ url              : chr ""
  .. .. ..@ pubMedIds        : chr ""
  .. .. ..@ samples          : list()
  .. .. ..@ hybridizations   : list()
  .. .. ..@ normControls     : list()
  .. .. ..@ preprocessing    : list()
  .. .. ..@ other            : list()
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 0 0
  ..@ annotation       : chr "hgu133a"
  ..@ protocolData     :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	0 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr(0) 
  .. .. ..@ data             :'data.frame':	286 obs. of  0 variables
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. ..@ .Data:List of 4
  .. .. .. ..$ : int [1:3] 2 12 0
  .. .. .. ..$ : int [1:3] 2 10 0
  .. .. .. ..$ : int [1:3] 1 3 0
  .. .. .. ..$ : int [1:3] 1 0 0
}
\references{
Has been obtained from GEO (GSE2034). Only first 1000 probesets where stored
(the rest has been removed).
}
\examples{
data(eset)
## maybe str(eset) ; plot(eset) ...
}
\keyword{datasets}

\name{write.html}
\alias{write.html}
\title{
Write a data.frame to an html file.
}
\description{
Creates an html file with links and plots from a table.
}
\usage{
write.html(x, links, tiny.pic, tiny.pic.size = 100, title = "", file, digits = 3)
}
\arguments{
  \item{x}{
    Object of class \code{data.frame}.
  }
  \item{links}{
    Object of class \code{list} with one item per column in x.
    If we want the ith column of x to have links to a site or local file
    we will have to write those links into the ith element of links.
  }
  \item{tiny.pic}{
    Object of class \code{list} with one item per column in x.
    If we want the ith column of x to show plots instead of text we will
    have to write the path to the plots into the ith element of links.
  }
  \item{tiny.pic.size}{
    size of the pictures if any.
  }
  \item{title}{
    Title that will be shown on top of the html file.
  }
  \item{file}{
    path and name of the file that will be created.
  }
  \item{digits}{
    number of digits that will be shown in numeric columns of x.
  }
}
\author{
Evarist Planet
}
\seealso{
  write.csv, write.table, htmlpage
}
\examples{
##
##Code has been commented to avoid the creation of files
##
#(x <- data.frame(gene.symbol=c('AARS','ABCF1','ABLIM1'),value=c(2.054,30.024,5.0221),plot=rep('Open',3)))
#tiny.pic <- links <- vector('list',length=ncol(x))
#links[[1]] <- paste('http://www.genecards.org/index.php?path=/Search/keyword/',x[,1])
#for (i in 1:nrow(x)) {
#  png(paste('~/Desktop/',x[i,1],'.png',sep=''))
#  plot(1:3,log(1:3))
#  dev.off()
#}
#tiny.pic[[3]] <- links[[3]] <- paste(x[,1],'.png',sep='')
#write.html(x,links=links,tiny.pic=tiny.pic,file='~/Desktop/x.html',title='My html test')
}
\keyword{ ~kwd1 }

\name{epheno-class}
\Rdversion{1.1}
\docType{class}
\alias{epheno-class}
\alias{[,epheno,ANY,ANY-method}
\alias{dim,epheno-method}
\alias{getFc,epheno-method}
\alias{getHr,epheno-method}
\alias{getMeans,epheno-method}
\alias{getPvals,epheno-method}
\alias{getPostProbs,epheno-method}
\alias{getSignif,epheno-method}
\alias{getSummaryDif,epheno-method}
\alias{logFcHr,epheno-method}
\alias{p.adjust.method,epheno-method}
\alias{phenoClass,epheno-method}
\alias{phenoNames,epheno-method}
\alias{approach,epheno-method}
\alias{show,epheno-method}

\title{Class "epheno"}
\description{
Object obtained with the ExpressionPhenoTest function. Contains FC, HR
and pvals from testing expression values of each gene against phenotypic
variables.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("epheno",
  assayData, phenoData, featureData, exprs, ...)}. 
}
\section{Slots}{
  \describe{
    \item{\code{p.adjust.method}:}{Object of class \code{"character"}
      containing the multiple testing adjustment method used (if one was
      used). }
    \item{\code{approach}:}{Object of class \code{"character"}
      containing 'frequentist' or 'bayesian' depending on the user's
      selecion.} 
    \item{\code{assayData}:}{Object of class \code{"AssayData"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
    \item{\code{phenoData}:}{Object of class \code{"AnnotatedDataFrame"}
      that contains information about the variables stored in the
      experimentData slot such as their class (continuous, categorical,
      etc) or type (mean, summaryDif, pval, etc).}
    \item{\code{featureData}:}{Object of class \code{"AnnotatedDataFrame"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
    \item{\code{experimentData}:}{Object of class \code{"MIAME"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
    \item{\code{annotation}:}{Object of class \code{"character"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
    \item{\code{protocolData}:}{Object of class \code{"AnnotatedDataFrame"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
    \item{\code{.__classVersion__}:}{Object of class \code{"Versions"} that is
      inherited from the \code{ExpressionSet} object used to create the
      \code{epheno} object. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{ExpressionSet}"}, directly.
Class \code{"\linkS4class{eSet}"}, by class "ExpressionSet", distance 2.
Class \code{"\linkS4class{VersionedBiobase}"}, by class "ExpressionSet", distance 3.
Class \code{"\linkS4class{Versioned}"}, by class "ExpressionSet", distance 4.
}
\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "epheno", i = "ANY", j = "ANY")}:
      inherited from the \code{ExpressionSet} class. }
    \item{dim}{\code{signature(x = "epheno")}: inherited from the
      \code{ExpressionSet} class.}
    \item{export2CSV}{\code{signature(x = "epheno")}: ... }
    \item{getFc}{\code{signature(x = "epheno")}: getter for the fold
      changes.}
    \item{getHr}{\code{signature(x = "epheno")}: getter for the hazard
      ratios.}
    \item{getMeans}{\code{signature(x = "epheno")}: getter for the means.}
    \item{getSignif}{\code{signature(x = "epheno")}: getter for the
      pvalues or posterior probabilities.}
    \item{getPvals}{\code{signature(x = "epheno")}: getter for the
      pvalues.}
    \item{getPostProbs}{\code{signature(x = "epheno")}: getter for the
      posterior probabilities.}
    \item{getSummaryDif}{\code{signature(x = "epheno")}: getter that
      returns hazard ratios, fold changes and pvalues. }
    \item{gseaSignatures}{\code{signature(x = "epheno", signatures =
	"list")}: Used to compute GSEA. Please read the gseaSignatures
      manual.}
    \item{logFcHr}{\code{signature(x = "epheno")}: getter for the log of
      fold changes and hazard ratios.}
    \item{p.adjust.method}{\code{signature(x = "epheno")}: getter for
      the p value adjustment method that has been used. }
    \item{phenoClass}{\code{signature(x = "epheno")}: Returns the class
      off all variables. }
    \item{phenoNames}{\code{signature(x = "epheno")}: Returns the names
      of the tested phenotypes.}
    \item{show}{\code{signature(object = "epheno")}: Shows a brief
      overview of the object.}
	 }
}
\author{
Evarist Planet
}
\examples{
showClass("epheno")
}
\keyword{classes}

\name{getEsPositions}
\alias{getEsPositions}
\title{
Obtain chromosome positions for each gene.
}
\description{
Given an object of class epheno obtain the gene positions on the genome.
}
\usage{
getEsPositions(epheno, phenoName, organism = "human", logEs = T, center = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{epheno}{
An object of class \code{epheno} usually obtained with \code{ExpressionPhenoTest}
}
  \item{phenoName}{
The phenotype that we want to use. Has to be in \code{phenoNames(epheno)}
}
  \item{organism}{
Has to be 'human' or 'mouse'. The default is 'human'.
}
  \item{logEs}{
If the values have to be log scaled.
}
  \item{center}{
If the values have to be genome centered. If TRUE the genome average
will be substracted to every value.
}
}
\details{
The output will usually be passed to findCopyNumber.
}
\value{
An object of class \code{data.frame} will be returned containing 3
variables: es (enichment score for fold change or hazard ratio), chr
(chromosome), pos (position in Mb).
epheno's featureNames will be used as row names. 
}
\author{
Evarist Planet
}
\examples{
data(epheno)
phenoNames(epheno)
mypos <- getEsPositions(epheno,'Relapse')
head(mypos)
}
\keyword{ ~kwd1 }

\name{getters for the epheno object}
\alias{getFc}
\alias{getHr}
\alias{getMeans}
\alias{getSignif}
\alias{getPostProbs}
\alias{getPvals}
\alias{getSummaryDif}
\alias{logFcHr}
\alias{p.adjust.method}
\alias{phenoClass}
\alias{phenoNames}
\alias{approach}
\alias{[,epheno,ANY,ANY,ANY-method}
\docType{data}
\title{
Getters for the epheno object:
}
\description{
\code{getFc} gets the fold changes. \code{getHr} gets the hazard
ratios. \code{getMeans}  gets the means. \code{getPvals} gets the p
values. \code{getPostProbs} get the posterior
probabilities. \code{getSignif} gets the pvalues or the posterior
probabilities depending on the approach (frequentist or bayesian) that
was used when the epheno object was created. \code{getSummaryDif} gets
fold changes and hazard ratios. \code{lofFcHr} gets the fold changes and
hazard ratios after log scaling. \code{p.adjust.method} gets the p value
adjustment method that was used when creating the
object. \code{phenoClass} returns a data.frame telling the class
(ordinal, continuous, categorical or survival) of each
phenotype. \code{phenoNames} gets the phenotype names. \code{approach}
gets the approach that was used (either frequentist or bayesian).
}
\usage{
getFc(x)
getHr(x)
getMeans(x)
getSignif(x)
getPvals(x)
getPostProbs(x)
getSummaryDif(x)
logFcHr(x)
p.adjust.method(x)
phenoClass(x)
phenoNames(x)
approach(x)
}
\arguments{
\item{x}{ \code{epheno} object}
}
\keyword{datasets}
\author{
Evarist Planet
}

\name{getVars2test-methods}
\docType{methods}
\alias{getVars2test-methods}
\alias{getVars2test,epheno-method}
\title{Methods for Function getVars2test in Package `phenoTest'}
\description{
Methods for function \code{getVars2test} in Package `phenoTest'. For
more information read the funcion's manual.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "epheno")}}{
Method for an object of class \code{epheno}.
}
}}
\keyword{methods}

\name{plot.gseaData}
\alias{plot.gseaData}
\docType{data}
\title{
GSEA-like Plot.
}
\description{
Builds a GSEA plot using a \code{gseaData} object.
\code{gseaData} object can be obtained with the \code{gsea} function.
}
\usage{
plot.gseaData(x,selGsets,selVars,...)
}
\arguments{
\item{x}{ this has to be of class \code{gseaData}}
\item{selGsets}{ object of class \code{character} containing the names
  of the gene sets that we want to plot.}
\item{selVars}{ object of class \code{character} containing the names of
  the variables that we want to plot.}
\item{...}{ Arguments to be passed to \code{plot}.}
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment Analysis}. 
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}
}
\examples{
#for examples see the help file of gseaSigntaures: ?gseaSignatures
}
\author{
Evarist Planet
}
\keyword{datasets}

\name{show-methods}
\docType{methods}
\alias{show-methods}
\alias{show,AnnotatedDataFrame-method}
\alias{show,ANY-method}
\alias{show,classRepresentation-method}
\alias{show,container-method}
\alias{show,eSet-method}
\alias{show,genericFunction-method}
\alias{show,gseaSignaturesSign-method}
\alias{show,gseaSignaturesVar-method}
\alias{show,gseaSignificanceSign-method}
\alias{show,gseaSignificanceVar-method}
\alias{show,LargeDataObject-method}
\alias{show,MethodDefinition-method}
\alias{show,MethodSelectionReport-method}
\alias{show,MethodWithNext-method}
\alias{show,MIAME-method}
\alias{show,namedList-method}
\alias{show,ObjectsWithPackage-method}
\alias{show,oldClass-method}
\alias{show,ScalarCharacter-method}
\alias{show,ScalarObject-method}
\alias{show,signature-method}
\alias{show,TestResults-method}
\alias{show,traceable-method}
\alias{show,Versioned-method}
\alias{show,Versions-method}
\alias{show,VersionsNull-method}
\title{Methods for Function show in Package `methods'.}
\description{
Methods for function \code{show} in Package `methods'.
}
\section{Methods}{
\describe{

\item{\code{signature(object = "AnnotatedDataFrame")}}{
Will show an object of class \code{AnnotatedDataFrame}.
}

\item{\code{signature(object = "ANY")}}{
Will show an object of class \code{ANY}.
}

\item{\code{signature(object = "classRepresentation")}}{
Will show an object of class \code{classRepresentation}.
}

\item{\code{signature(object = "container")}}{
Will show an object of class \code{container}.
}

\item{\code{signature(object = "epheno")}}{
Will show an object of class \code{epheno}.
}

\item{\code{signature(object = "eSet")}}{
Will show an object of class \code{eSet}.
}

\item{\code{signature(object = "genericFunction")}}{
Will show an object of class \code{genericFunction}.
}

\item{\code{signature(object = "gseaSignaturesSign")}}{
Will show an object of class \code{gseaSignaturesSign}.
}

\item{\code{signature(object = "gseaSignaturesVar")}}{
Will show an object of class \code{gseaSignaturesVar}.
}

\item{\code{signature(object = "gseaSignificanceSign")}}{
Will show an object of class \code{gseaSignificanceSign}.
}

\item{\code{signature(object = "gseaSignificanceVar")}}{
Will show an object of class \code{gseaSignificanceVar}.
}

\item{\code{signature(object = "LargeDataObject")}}{
Will show an object of class \code{LargeDataObject}.
}

\item{\code{signature(object = "MethodDefinition")}}{
Will show an object of class \code{MethodDefinition}.
}

\item{\code{signature(object = "MethodSelectionReport")}}{
Will show an object of class \code{MethodSelectionReport}.
}

\item{\code{signature(object = "MethodWithNext")}}{
Will show an object of class \code{MethodWithNext}.
}

\item{\code{signature(object = "MIAME")}}{
Will show an object of class \code{MIAME}.
}

\item{\code{signature(object = "namedList")}}{
Will show an object of class \code{namedList}.
}

\item{\code{signature(object = "ObjectsWithPackage")}}{
Will show an object of class \code{ObjectsWithPackage}.
}

\item{\code{signature(object = "oldClass")}}{
Will show an object of class \code{oldClass}.
}

\item{\code{signature(object = "ScalarCharacter")}}{
Will show an object of class \code{ScalarCharacter}.
}

\item{\code{signature(object = "ScalarObject")}}{
Will show an object of class \code{ScalarObject}.
}

\item{\code{signature(object = "signature")}}{
Will show an object of class \code{signature}.
}

\item{\code{signature(object = "TestResults")}}{
Will show an object of class \code{TestResults}.
}

\item{\code{signature(object = "traceable")}}{
Will show an object of class \code{traceable}.
}

\item{\code{signature(object = "Versioned")}}{
Will show an object of class \code{Versioned}.
}

\item{\code{signature(object = "Versions")}}{
Will show an object of class \code{Versions}.
}

\item{\code{signature(object = "VersionsNull")}}{
Will show an object of class \code{VersionsNull}.
}
}}
\keyword{methods}

\name{eset.genelevel}
\alias{eset.genelevel}
\docType{data}
\title{
Example data.
}
\description{
Example data of class ExpressionSet with one probeset per gene. 
}
\usage{data(eset.genelevel)}
\format{
  The format is:
Formal class 'ExpressionSet' [package "Biobase"] with 7 slots
  ..@ assayData        :<environment: 0x1050d9390> 
  ..@ phenoData        :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	7 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr [1:7] NA NA NA NA ...
  .. .. ..@ data             :'data.frame':	286 obs. of  7 variables:
  .. .. .. ..$ PID              : int [1:286] 3 5 6 7 8 9 11 14 15 17 ...
  .. .. .. ..$ GEOaccession     : Factor w/ 286 levels "GSM36777","GSM36778",..: 17 20 21 22 24 25 58 59 60 61 ...
  .. .. .. ..$ lymph.node.status: chr [1:286] "negative" "negative" "negative" "negative" ...
  .. .. .. ..$ Months2Relapse   : int [1:286] 101 118 9 106 37 125 109 14 99 137 ...
  .. .. .. ..$ Relapse          : int [1:286] 0 0 1 0 1 0 0 1 0 0 ...
  .. .. .. ..$ ER.Status        : num [1:286] 0 1 0 0 0 1 1 0 1 1 ...
  .. .. .. ..$ BrainRelapse     : int [1:286] 0 0 0 0 0 0 0 0 0 0 ...
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ featureData      :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	16 obs. of  3 variables:
  .. .. .. ..$ Column          : chr [1:16] "ID" "GB_ACC" "SPOT_ID" "Species Scientific Name" ...
  .. .. .. ..$ Description     : Factor w/ 15 levels "","A gene symbol, when one is available (from UniGene).",..: 3 5 15 13 12 1 11 1 10 14 ...
  .. .. .. ..$ labelDescription: chr [1:16] NA NA NA NA ...
  .. .. ..@ data             :'data.frame':	1000 obs. of  16 variables:
  .. .. .. ..$ ID                              : Factor w/ 22284 levels "1007_s_at","1053_at",..: 1 2 3 4 5 6 7 8 9 10 ...
  .. .. .. ..$ GB_ACC                          : Factor w/ 21129 levels "AF052179","AF061832",..: 93 30 95 97 25 24 96 99 28 20 ...
  .. .. .. ..$ SPOT_ID                         : chr [1:1000] NA NA NA NA ...
  .. .. .. ..$ Species.Scientific.Name         : Factor w/ 2 levels "Homo sapiens",..: 1 1 1 1 1 1 1 1 1 1 ...
  .. .. .. ..$ Annotation.Date                 : Factor w/ 2 levels "Jul 11, 2007",..: 1 1 1 1 1 1 1 1 1 1 ...
  .. .. .. ..$ Sequence.Type                   : Factor w/ 4 levels "Consensus sequence",..: 2 2 2 2 2 2 2 2 2 2 ...
  .. .. .. ..$ Sequence.Source                 : Factor w/ 3 levels "Affymetrix Proprietary Database",..: 1 2 1 2 1 2 1 1 2 1 ...
  .. .. .. ..$ Target.Description              : Factor w/ 21363 levels "Consensus includes gb:AI656011 /FEA=EST /DB_XREF=gi:4739990 /DB_XREF=est:tt42e08.x1 /CLONE=IMAGE:2243462 /UG=Hs.116875 KIAA0156"| __truncated__,..: 16 13 18 20 8 7 19 22 11 4 ...
  .. .. .. ..$ Representative.Public.ID        : Factor w/ 21197 levels "AF052179","AF061832",..: 93 30 95 97 25 24 96 99 28 20 ...
  .. .. .. ..$ Gene.Title                      : Factor w/ 14208 levels "ADP-ribosylation factor 1",..: 35 66 46 60 44 97 96 64 26 33 ...
  .. .. .. ..$ Gene.Symbol                     : Factor w/ 13293 levels "ABCF1","ARF1",..: 20 59 40 53 33 96 94 58 15 18 ...
  .. .. .. ..$ ENTREZ_GENE_ID                  : chr [1:1000] "780" "5982" "3310" "7849" ...
  .. .. .. ..$ RefSeq.Transcript.ID            : Factor w/ 13074 levels "NM_000409","NM_000661 /// NM_001024921",..: 37 45 41 52 1 50 49 82 47 4 ...
  .. .. .. ..$ Gene.Ontology.Biological.Process: Factor w/ 7245 levels "","0000074 // regulation of progression through cell cycle // traceable author statement /// 0006139 // nucleobase, nucleoside, nu"| __truncated__,..: 61 22 78 32 79 60 14 63 72 20 ...
  .. .. .. ..$ Gene.Ontology.Cellular.Component: Factor w/ 4148 levels "","0000502 // proteasome complex (sensu Eukaryota) // traceable author statement /// 0005634 // nucleus // inferred from electroni"| __truncated__,..: 72 45 1 44 1 1 42 71 6 68 ...
  .. .. .. ..$ Gene.Ontology.Molecular.Function: Factor w/ 7314 levels "","0000049 // tRNA binding // non-traceable author statement /// 0000166 // nucleotide binding // inferred from electronic annotat"| __truncated__,..: 23 26 27 40 81 18 39 71 74 69 ...
  .. .. ..@ dimLabels        : chr [1:2] "featureNames" "featureColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ experimentData   :Formal class 'MIAME' [package "Biobase"] with 13 slots
  .. .. ..@ name             : chr ""
  .. .. ..@ lab              : chr ""
  .. .. ..@ contact          : chr ""
  .. .. ..@ title            : chr ""
  .. .. ..@ abstract         : chr ""
  .. .. ..@ url              : chr ""
  .. .. ..@ pubMedIds        : chr ""
  .. .. ..@ samples          : list()
  .. .. ..@ hybridizations   : list()
  .. .. ..@ normControls     : list()
  .. .. ..@ preprocessing    : list()
  .. .. ..@ other            : list()
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 0 0
  ..@ annotation       : chr "hgu133a"
  ..@ protocolData     :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	0 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr(0) 
  .. .. ..@ data             :'data.frame':	286 obs. of  0 variables
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. ..@ .Data:List of 4
  .. .. .. ..$ : int [1:3] 2 12 0
  .. .. .. ..$ : int [1:3] 2 10 0
  .. .. .. ..$ : int [1:3] 1 3 0
  .. .. .. ..$ : int [1:3] 1 0 0
}
\references{
Has been obtained from GEO (GSE2034). Only first 1000 probesets where stored
(the rest has been removed).
After that the expressionSet was filtered to keep only one probeset per
gene. We used the nsFilter function from package genefilter to
accomplish this task.
}
\examples{
data(eset.genelevel)
## maybe str(eset.genelevel) ; plot(eset.genelevel) ...
}
\keyword{datasets}

\name{gsea.selGsets}
\alias{gsea.selGsets}
\alias{gsea.selVars}
\title{
Subset an object of class gseaData.
}
\description{
\code{gsea.selGsets} to select a subgroup of gene sets from an object of
class gseaData.
\code{gsea.selVars} to select a subgroup of variables from an object of
class gseaData.
}
\usage{
gsea.selGsets(x, selGsets)
gsea.selVars(x, selVars)

}
\arguments{
  \item{x}{ an object of class \code{gseaData}.}
  \item{selGsets}{ names of the gene sets that we want to keep.}
  \item{selVars}{ names of the variables that we want to keep.}
}
\value{
Returns an object of class \code{gseaData}.
}
\author{
Evarist Planet
}
\keyword{ ~kwd1 }

\name{phenoTest-package}
\alias{phenoTest-package}
\alias{phenoTest}
\docType{package}
\title{
Test correlation between gene expression and phenotype.
}
\description{
Test correlation between gene expression and phenotype.
}
\details{
\tabular{ll}{
Package: \tab phenoTest\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2010-04-28\cr
License: \tab What license is it under?\cr
LazyLoad: \tab yes\cr
}
}
\author{
Evarist Planet
Maintainer: Evarist Planet <evarist.planet@irbbarcelona.org>
}

\name{pca}
\alias{pca}
\title{
Principal components plot.
}
\description{
Creates a Principal Components plot where we can show paired samples,
and confidence intervals for the mean of every group of interest.
We can also choose the component or components we want to plot.
}
\usage{
pca(x, group, group2, pair, names, ellipse = FALSE, main = "", components = c(1, 2))
}
\arguments{
  \item{x}{
    An object of class ExpressionSet.
}
  \item{group}{
    Variable in pData(x) that contains the groups of interest. Samples
    of the same group will be plotted with the same color.
}
  \item{group2}{
    Variable in pData(x) that contains secondary groups of
    interest. Sample of the same secondary group of interest will be
    plotted with the same symbol.
}
  \item{pair}{
    Variable in pData(x) that contains the information about the pairs
    of data. Those pairs will be joined by a line.
}
  \item{names}{
    Variable in pData(x) that contains the information about the names
    of the samples.
}
  \item{ellipse}{
    If we want to plot ellipses with the 95 percent confidence intervals
    for every group.
}
  \item{main}{
    A title for the plot.
}
  \item{components}{
    Which components we want to plot. By default the first principal
    component will be plotted on the x axis and the second principal
    component will be plotted on the y axis.
    More than two components may be specified. If so multiple plots will
    be produced.
}
}
\author{
  Evarist Planet
}
\seealso{
  prcomp.
}
\examples{
data(eset)
pca(x=eset, group='Relapse', names='GEOaccession')
#pca(x=eset, group='Relapse', names='GEOaccession', components=1:3)
}
\keyword{datasets}
    

\name{ClusterPhenoTest}
\alias{ClusterPhenoTest}
\title{
Test association of clusters with phenotype.
}
\description{
Test the associations between clusters that each sample belongs to
(based on gene expression) and each phenotype.
}
\usage{
ClusterPhenoTest(x,cluster,vars2test,B=10^4,p.adjust.method='none')
}
\arguments{
  \item{x}{ExpressionSet with phenotype information stored in \code{pData(x)}.}
  \item{cluster}{variable of class \code{character} or \code{factor}
    telling at which cluster each sample belongs to.}
  \item{vars2test}{ list with components 'continuous', 'categorical',
  'ordinal' and 'survival' indicating which phenotype variables should
  be tested. 'continuous', 'categorical' and 'ordinal' must be character
  vectors, 'survival' a matrix with columns named 'time' and
  'event'. The names must match names in \code{names(pData(x))}.}
  \item{B}{An integer specifying the number of replicates used in the
  chi-square Monte Carlo test (passed on to \code{chisq.test}).}
  \item{p.adjust.method}{Method for P-value adjustment, passed on to
  \code{p.adjust}.}
}
\details{
Test association between the provided clusters and each phenotype.
  
For variables in vars2test\$continuous and vars2test\$ordinal a
Kruskal-Wallis Rank Sum test is used; for vars2test\$categorical a
chi-square test (with exact p-value if \code{simulate.p.value} is set to
TRUE); for var2test\$survival a Cox proportional hazards
likelihood-ratio test. 
}
\examples{
#load data
data(eset)  
eset

#construct vars2test
survival <- matrix(c("Relapse","Months2Relapse"),ncol=2,byrow=TRUE)
colnames(survival) <- c('event','time')
#add positive to have more than one category
pData(eset)[1:20,'lymph.node.status'] <- 'positive' 
vars2test <- list(survival=survival,categorical='lymph.node.status')
vars2test

#first half of the samples will be one cluster and the rest the other cluster
cluster <- c(rep('Cluster1',floor(ncol(eset)/2)),rep('Cluster2',ncol(eset)-floor(ncol(eset)/2)))

#test association
ClusterPhenoTest(eset,cluster,vars2test=vars2test)
}
\author{
David Rossell
}

\name{gseaData-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaData-class}
\alias{show,gseaData-method}
\alias{getEs,gseaData-method}
\alias{getEsSim,gseaData-method}
\alias{getFcHr,gseaData-method}

\title{Class "gseaData"}
\description{
This class is an ES (enrichment score) and ES.sim (simulated enrichment
score) container that will be used in the GSEA (Gene Set Enrichment
Analysis) process.
There is one container for every gene signature.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaData", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"} .}
    \item{\code{gseaSignaturesSign}:}{Object of class
      \code{"gseaSignaturesSign"} or \code{"gseaSignaturesVar"}.}
    \item{\code{gseaSignificanceSign}:}{Object of class
      \code{"gseaSignificanceSign"} or \code{"gseaSignificanceVar"}.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
  \describe{
    \item{getEs}{\code{signature(x = "gseaData")}: Returns the
      enrichment scores.}
    \item{getEsSim}{\code{signature(x = "gseaData")}: Returns
      the simulated enrichemt scores (the ones obtained after
      permutations).} 
    \item{getFcHr}{\code{signature(x = "gseaData")}: Returns
      the fold change and/or the hazard ratio that were used to compute
      the enrichment scores. }
	 }
}
\author{Evarist Planet}
\examples{
showClass("gseaSignaturesSign")
}
\keyword{classes}

\name{ExpressionPhenoTest}
\alias{ExpressionPhenoTest}
\docType{data}
\title{Tests univariate association between a list of phenotype
  variables and gene expression.} 
\description{
Tests univariate association between a list of phenotype variables and
gene expression. 
}
\usage{
ExpressionPhenoTest(x, vars2test, adjustVars,
p.adjust.method='BH',continuousCategories=3,mc.cores,approach='frequentist') 
}
\arguments{
\item{x}{ ExpressionSet containing expression levels in \code{exprs(x)}
  and phenotype information in \code{pData(x)}.} 
\item{vars2test}{ list with components 'continuous', 'categorical',
  'ordinal' and 'survival' indicating which phenotype variables should 
  be tested. 'continuous', 'categorical' and 'ordinal' must be character
  vectors, 'survival' a matrix with columns named 'time' and
  'event'. The names must match names in names(pData(x)).} 
\item{adjustVars}{ variables that will be used as adjustment variables
  when fitting linear models and/or cox models. This variables have to
  exist in \code{colnames(pData(x))}.} 
\item{p.adjust.method}{ method for p-value adjustment, passed on to
  \code{p.adjust}. Valid values are c("holm", "hochberg", "hommel",
  "bonferroni", "BH", "BY", "fdr", "none").} 
\item{continuousCategories}{ number of categories used for continuous
  variables.} 
\item{mc.cores}{ the number of cores to use, i.e. how many processes
  will be spawned (at most).}
\item{approach}{ this can be either 'frequentist' or 'bayesian'. With
  frequentist pvalues will be computed. With 'bayesian' posterior
  probabilities will be computed.}
}
\details{
If approach is 'frequentist':
-The effect of both continuous, categorical and ordinal phenotype
variables on gene expression levels are tested via lmFit.
-For ordinal variables a single coefficient is used to test its effect
on gene expression (trend test), which is then used to obtain a P-value
(means for each category are reported in the output).
-Gene expression effects on survival are tested via Cox proportional
hazards model, as implemented in function 'coxph'.

If approach is bayesian posterior probabilities are computed comparing
the BIC of a model with the variable of interest as explanatory variable
against the BIC of the same model without the variable of interest as
explanatory variable.
}
\value{
The output is an \code{epheno} object, which basically extends an
\code{ExpressionSet} object. The means, fold changes, standarized hazard
ratios and pvalues are stored in the \code{experimentData} slot which is
accessible with the \code{exprs} method. Information about the kind of
information of each variable can be found in the \code{phenoData} slot
which is accessible with the \code{pData} method.

There are several methods that can be used to access the information
stored in an \code{epheno} object. For more information please type one
of the following: \code{getFc(x), getHr(x), getMeans(x), getSignif,
getPvals(x), getPostProbs, getSummaryDif(x), logFcHr(x),
p.adjust.method(x), phenoClass(x), phenoNames(x)}.
}
\references{
Kass R.E. and Wasserman L. A Reference Bayesian Test for Nested
Hypotheses and its Relationship to the Schwarz Criterion. Journal of the
American Statistical Association, 90, pp. 928-934.
}
\keyword{datasets}
\examples{
#load eset
data(eset)
eset

#prepare vars2test
survival <- matrix(c("Relapse","Months2Relapse"),ncol=2,byrow=TRUE)
colnames(survival) <- c('event','time')
vars2test <- list(survival=survival)

#run ExpressionPhenoTest
epheno <- ExpressionPhenoTest(eset,vars2test,p.adjust.method='none')
epheno
}
\author{
David Rossell
}

\name{eset2genelevel}
\alias{eset2genelevel}
\title{
Filter ExpressionSet to keep one probeset per gene.
}
\description{
Only one probeset per gene will be kept and entrezid will be used as
gene identifier.
\code{nsFilter} from package \code{genefilter} is used to select the
probeset.
The selected probeset is the one with higher interquartilic range.
}
\usage{
eset2genelevel(x)
}
\arguments{
  \item{x}{an object of class \code{ExpressionSet}.}
  }
\author{
Evarist Planet
}
\seealso{
genefilter::nsFilter
}
\examples{
#data(eset)
#library(hgu133a.db)
#x <- eset2genelevel(eset)
#x
#head(featureNames(x))
}
\keyword{ ~kwd1 }

\name{findCopyNumber}
\alias{findCopyNumber}
\title{
  Find copy number regions using expression data in a similar way ACE does.
}
\description{
  Given enrichment scores between two groups of samples and the
  chromosomical positions of those enrichment scores this function finds
  areas where the enrichemnt is bigger/lower than expected if the
  positions where assigned at random.
  Plots of the regions and positions of the enriched regions are provided.
}
\usage{
findCopyNumber(x, minGenes = 15, B = 100, p.adjust.method = "BH",
pvalcutoff = 0.05, exprScorecutoff = NA, mc.cores = 1, useAllPerm = F,
genome = "hg19", chrLengths, sampleGenome = TRUE, useOneChr = FALSE,
useIntegrate = TRUE,plot=TRUE,minGenesPerChr=100)
}
\arguments{
  \item{x}{
    An object of class \code{data.frame} with gene or probe identifiers as
    row names and the following columns: es (the enrichment score), chr (the
    chromosome where the gene or probe belong to) and pos (position in the
    chromosome in megabases).
    It can be obtained (from an epheno object) with the function
    getEsPositions.
  }
  \item{minGenes}{
    Minimum number of genes in a row that have to be enriched to mark the
    region as enriched.
    Has to be bigger than 2.
  }
  \item{B}{
    Number of permuations that will be computed to calculate pvalues.
    If \code{useAllPerm} is FALSE this value has to be bigger than 100. 
    If \code{useAllPerm} is TRUE the computations are much more
    expensive, therefore it is not recommended to use a B bigger than 100.
  }
  \item{p.adjust.method}{
    P value adjustment method to be used. p.adjust.methods provides a list
    of available methods.
  }
  \item{pvalcutoff}{
    All genes with an adjusted p value lower than this parameter will be
    considered enriched.
  }
  \item{exprScorecutoff}{
    Genes with a smoothed score that is not bigger (lower if the given
    number is negative) than the specified value will not be considered
    significant.
  }
  \item{mc.cores}{
    Number of cores to be used in the computation. If \code{mc.cores} is
    bigger than 1 the \code{multicore} library has to be loaded.
  }
  \item{useAllPerm}{
    If FALSE for each gene only permutations of genes that are in an area
    with similar density (similar number of genes close to them) are used to
    compute pvalues. 
    If TRUE all permutations are used for each gene.

    We recommend to use the option FALSE after having observed that
    the enrichment can depend on the number of genes that are in the area.

    We recommend to use the option TRUE if the positions of the enrichment
    score are equidistant. Take into account that this option is much slower
    and needs less permutations, therefore a smaller \code{B} is preferred.

    See details for more info.
  }
  \item{genome}{
    Genome that will be used to draw cytobands.
  }
  \item{chrLengths}{
    An object of class \code{numeric} containing chromosome names as names.
    This names have to be the same as the ones used in \code{x$chr}
    If missing the last position is used.
  }
  \item{sampleGenome}{
    If positions are sampled over the hole genome (across chromosomes) or
    within each chromosome. This is TRUE by default. 
  }
  \item{useOneChr}{
    Use only one chromosome to build the distribution under the null
    hypothesis that genes/probes are not enriched. By default this is
    FALSE.
    The chromosome that is used is chosen as follows: after removing small
    chromosomes we select the one closest to the median quadratic distance
    to 0.
    Setting this parameter to TRUE decreases processing time.
  }
  \item{useIntegrate}{
    If we want to use \code{integrate} or \code{pnorm} to compute
    pvalues. The first does not assume any distribution for the
    distribution under the null hypothesis, the second assumes it is
    normally distributed.
  }
  \item{plot}{
    If FALSE the function will make no plots.
  }
  \item{minGenesPerChr}{
    Chromosomes with less than \code{minGenesPerChr} will be removed
    from the analysis.
  }
}
\details{
  Enrichemnt scores can be either log fold changes, log hazard ratios, log
  variabiliy ratios or any other score.
  
  Within each chromosome a smoothed score for each gene is obtained via
  generalized additive models, the smoothing parameter for each chromosome
  being chosen via cross-validation.
  The obtained smoothing parameter of each chromosome will be used in
  permutations.

  We assessed statistical significance by permuting the positions thrue
  the hole genome.
  If \code{useAllPerm} is FALSE for each gene the permutations of genes
  that are in an area with similar density (distance to tenth gene) are
  used to compute pvalues. We observed that genes with similar densities
  tend to have similar smoothed scores.
  If we set 1000 permutations (\code{B}=1000) scores are permuted thrue 
  the hole genome 10 times (1000/100). For each smoothed scored the
  permutations of the 100 smoothed scores with most similar density
  (distance to tenth gene) are used. Therefore each smoothed score will be
  compared to 1000 smoothed scores obtained from permutations.

  If scores are at the same distance in the genome from each other (for
  instance when we have a score every fixed certain bases) the option
  \code{useAllPerm}=TRUE is recommended. In this case every smoothed score
  is compared to all smoothed scores obtained via permutations.
  In this case having 20,000 genes and setting the paramter \code{B=10}
  would mean that the scores are permuted 10 times times thrue the hole
  genome, obtaining 200,000 permuted smoothed scores. Each observed smoothed
  score will be tested against the distribution of the 200,000 permuted
  smoothed scores.

  Only regions with as many genes as told in \code{minGenes} being
  statistically significant (pvalue lower than parameter
  \code{pvalcutoff}) after adjusting pvalues with the method specified in
  \code{p.adjust.method} will be selected as enriched.
  If \code{exprScorecutoff} is different form NA, a gene to be
  statistically significant will need (aditionally to the pvalue cutoff)
  to have a smoothed score bigger (lower if \code{exprScorecutoff} is
  negative) than the specified value.
}
\value{
  Plots all chromomes and marks the enriched regions.
  Also returns a \code{data.frame} containing the positions of the
  enriched regions. This output can be passed by to the \code{genesInArea}
  function to obtain the names of the genes that are in each region.
}
\author{
  Evarist Planet
}
\seealso{
  getEsPositions, genesInArea
}
\examples{
data(epheno)
phenoNames(epheno)
mypos <- getEsPositions(epheno,'Relapse')
mypos$chr <- '1' #we set all probes to chr one for illustration purposes
                 #(we want a minimum number of probes per chromosome) 
head(mypos)
set.seed(1)
regions <- findCopyNumber(mypos,B=10,plot=FALSE) 
head(regions)
}
\keyword{ ~kwd1 }

\name{get gseaSignatures' elements}
\alias{getEs}
\alias{getEsSim}
\alias{getNes}
\alias{getFcHr}
\docType{data}
\title{
Substract element's of a gseaSignaturesSign or gseaSignaturesVar object (obtained using the
gseaSignatures function). 
}
\description{
getEs returns ES (enrichment scores) getEsSim returns simulated ES
(needed to compute pvals), getNes returns NES (normalized enrichment
scores) and getFcHr returns the fold changes or hazard used to compute
the ES, simulated ES and NES.
}
\usage{
getEs(x)
getEsSim(x)
getNes(x)
getFcHr(x)
}
\arguments{
\item{x}{ an \code{gseaSignaturesSign} or \code{gseaSignaturesVar} object. Those
  objects are obtained using the \code{gseaSignatures} function.}
}
\keyword{datasets}
\author{
Evarist Planet
}

\name{gsea2html}
\alias{gsea2html}
\title{
Export an object of class gseaData to an html file.
}
\description{
Exports \code{gseaData} objects to html files with plots and links to online databases.
}
\usage{
gsea2html(gseaData, epheno, variable, title = "", path, file, digits = 3, plotEs = FALSE, limit=100)
}
\arguments{
  \item{gseaData}{ an object of class \code{gseaData}.}
  \item{epheno}{ the object of class \code{epheno} that was used to
    create \code{gseaData}.}
  \item{variable}{ varible that we are interested in.}
  \item{title}{ title that will be shown on top of the table.}
  \item{path}{ directory where we want to store the html files.}
  \item{file}{ filename.}
  \item{digits}{ Number of decimal digits that will be shown on the
    table.} 
  \item{plotEs}{ if this is TRUE enrichment score plots will be plotted
    instead of normalized enrichment score plots.}
  \item{limit}{ maximum number of gene sets that will be exported.}
}
\details{
This function produces a browseable version of the table that we can
obtain with \code{summary(gseaData)}.
We will obtain one plot per NES (or ES) and we will be able to see which
genes belong to each gene set and the values they have in the
\code{epheno} object.
}
\author{
Evarist Planet
}
\examples{
#WITH PROBESET AS IDENTIFIER
data(eset)
data(epheno)

set.seed(777)
sign1 <- sample(featureNames(eset))[1:20]
sign2 <- sample(featureNames(eset))[1:50]
mySignature <- list(sign1,sign2)
names(mySignature) <- c('My first signature','Another signature')
mySignature

mygsea <- gsea(x=epheno[,1],gsets=mySignature,B=100,p.adjust='BH')
summary(mygsea)

#following line has been commented to prevent the creation of files
#gsea2html(gseaData=mygsea,epheno=epheno,variable=phenoNames(epheno)[1],title='My test',path='~/Desktop',file='myGSEA.html')

#WITH ENTREZID AS IDENTIFIER
data(eset.genelevel)
eset.genelevel

set.seed(777)
sign1 <- sample(featureNames(eset.genelevel))[1:20]
sign2 <- sample(featureNames(eset.genelevel))[1:50]
mySignature.genelevel <- list(sign1,sign2)
names(mySignature.genelevel) <- c('My first signature','Another signature')
mySignature.genelevel

epheno.genelevel <- ExpressionPhenoTest(eset.genelevel,vars2test=list(categorical='lymph.node.status'))
mygsea.genelevel <- gsea(x=epheno.genelevel,gsets=mySignature.genelevel,B=100,p.adjust='BH')
summary(mygsea.genelevel)

#following line has been commented to prevent the creation of files
#gsea2html(gseaData=mygsea.genelevel,epheno=epheno.genelevel,variable=phenoNames(epheno.genelevel),title='My test (at genelevel)',path='~/Desktop',file='myGSEA_genelevel.html')
}
\keyword{ ~kwd1 }

\name{pAdjust}
\alias{pAdjust}
\title{
Adjust p values of an epheno object.
}
\description{
Adjusts the p values of an epheno object. The \code{p.adjust} function will be
used. For more information read the \code{p.adjust} function's help.
}
\usage{
pAdjust(x, method = "BH")
}
\arguments{
  \item{x}{an \code{epheno} object. }
  \item{method}{the correction method that will be used. Se the
    \code{p.adjust} help for more info about the methods.}
}
\examples{
#load epheno object
data(epheno)
epheno

#Adjust pvalue
p.adjust.method(epheno)
epheno <- pAdjust(epheno,method='BH')
p.adjust.method(epheno)
}
\author{
Evarist Planet
}

\name{gseaSignaturesSign-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaSignaturesSign-class}
\alias{getEs,gseaSignaturesSign-method}
\alias{getEsSim,gseaSignaturesSign-method}
\alias{getFcHr,gseaSignaturesSign-method}
\alias{gseaSignificance,gseaSignaturesSign-method}

\title{Class "gseaSignaturesSign"}
\description{
This class is an ES (enrichment score) and ES.sim (simulated enrichment
score) container that will be used in the GSEA (Gene Set Enrichment
Analysis) process.
There is one container for every gene signature.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaSignaturesSign", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"} .}
    \item{\code{gseaSignatures}:}{Object of class
      \code{"gseaSignatures"} This is the object that will contain the
      ES and ES.sim. }
    \item{\code{es.sim.gam}:}{Object of class \code{"matrix"} enrichment
      scores computed with the gam method. }
    \item{\code{fc.hr}:}{Object of class \code{"character"} fold change
      or hazard ratio used to compute the enrichment scores. }
    \item{\code{s}:}{Object of class \code{"logical"} The subset of
      genes we are interested in.} 
    \item{\code{test}:}{Object of class \code{"character"} The
      statistical test that will be used. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
  \describe{
    \item{getEs}{\code{signature(x = "gseaSignaturesSign")}: Returns the
      enrichment scores.}
    \item{getEsSim}{\code{signature(x = "gseaSignaturesSign")}: Returns
      the simulated enrichemt scores (the ones obtained after
      permutations).} 
    \item{getFcHr}{\code{signature(x = "gseaSignaturesSign")}: Returns
      the fold change and/or the hazard ratio that were used to compute
      the enrichment scores. }
    \item{gseaSignificance}{\code{signature(x = "gseaSignaturesSign")}:
      This is the next step in the process of performing GSEA. This
      funtion will test if the gene sets are enriched. }
	 }
}
\author{Evarist Planet}
\examples{
showClass("gseaSignaturesSign")
}
\keyword{classes}

\name{gseaSignatures}
\alias{gseaSignatures}
\docType{data}
\title{
Compute ES (enrichment scores) and es.sim (simulated enrichment scores)
for given phenotipic variable(s) and signature(s).
}
\description{
This function has been deprecated. You could better use \code{gsea}
instead.
  
This function computes the first step in the process of obtaining a
GSEA-like plot.
It computes the enrichment scores and simulated enrichment scores for
each variable and signature. The output will usually be used as input
for the \code{gseaSignificance} function.
An important parameter of the function is \code{logScale}. Its default
value is TRUE which means that by default the provided scores (i.e. fold
changes, hazard ratios) will be log scaled. Remember to change this
parameter to FALSE if your scores are already log scaled.
The \code{getEs}, \code{getEsSim}, \code{getFc}, \code{getHr} and
\code{getFcHr} methods can be used to acces each subobject. For more
information please visit the man pages of each method.
}
\usage{
gseaSignatures(x,gsets,logScale=TRUE,absVals=FALSE,averageRepeats=FALSE,
  B=1000,mc.cores=1,test='perm', minGenes=10,maxGenes=500,center=FALSE)
}
\arguments{
\item{x}{ \code{ePhenoTest}, \code{numeric} or \code{matrix} object
  containing hazard ratios or fold changes.}
\item{gsets}{ character or list object containing the names of the
  genes that belong to each signature.}
\item{logScale}{ if values should be log scaled.}
\item{absVals}{ if TRUE fold changes and hazard ratios that are negative
  will be turned into positive before starting the process. This is
  useful when genes can go in both directions.}
\item{averageRepeats}{ if x is of class numeric and has repeated names
  (several measures for some indivdual names) we can average the
  measures of the same names.}
\item{B}{ number of simulations to perform.}
\item{mc.cores}{ number of processors to use.}
\item{test}{ the test that will be used. 'perm' stands for the permutation
  based method, 'wilcox' stands for the wilcoxon test (this is the fastest
  one) and 'ttperm' stands for permutation t test.}
\item{minGenes}{ gene sets with less than minGenes genes will be removed
  from the analysis.}
\item{maxGenes}{ gene sets with more than maxGenes genes will be removed
  from the analysis.}
\item{center}{ if we want to center scores (fold changes or hazard
  ratios). The following is will be done: x = x-mean(x).}
}
\details{
The following preprocessing was done on the provided scores (i.e. fold changes, hazard
ratios) to avoid errors during the enrichment score computation:
-When having two scores with the same name its average was used.
-Zeros were removed.
-Scores without names (which can not be in any signature) removed. 
-Non complete cases (i.e. NAs, NaNs) were removed.
ES score was calculated for each signature and variable (see
references). If parameter \code{test} is 'perm' the signature was
permutted and the ES score was recalculated (this happened B times for
each variable, 1000 by default).
If \code{test} is 'wilcox' a wilcoxon test in which we test the fact
that the average value of the genes that do belong to our signtaure is
different from the average value of the genes that do not belong to our
signature will be performed.
If \code{test} is 'ttperm' a permutation t-test will be used.
Take into account that the final plot will be different when 'wilcox' is used.
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment Analysis}. 
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}
}
\keyword{datasets}
\examples{
#load epheno object
data(epheno)
epheno

#we construct two signatures
sign1 <- sample(featureNames(epheno))[1:20]
sign2 <- sample(featureNames(epheno))[50:75]
mySignature <- list(sign1,sign2)
names(mySignature) <- c('My first signature','My preferred signature')

#run gsea functions
#my.gseaSignatures <- gseaSignatures(x=epheno,signatures=mySignature,B=100,mc.cores=1)
#my.gseaSignificance <- gseaSignificance(my.gseaSignatures)
#my.summary <- summary(my.gseaSignificance)
#my.summary 
#plot(my.gseaSignatures,my.gseaSignificance)
}
\author{
Evarist Planet
}

\name{smoothCoxph}
\alias{smoothCoxph}
\title{
Plots the Cox proportional hazard smoothed by gene expression level.
}
\description{
Builds a plot showing how hazard behaves over different levels of
expression of a given gene. Confidence intervals are also provided. 
}
\usage{
smoothCoxph(time, event, x, xlim, ylim, xlab, ylab, logrisk=TRUE, ...)
}
\arguments{
  \item{time}{variable where time to survival is stored.}
  \item{event}{variable where survival event is stored.}
  \item{x}{\code{numeric} containing the expression levels of a given gene.}
  \item{xlim}{\code{xlim} for the plot.}
  \item{ylim}{\code{ylim} for the plot.}
  \item{xlab}{\code{xlab} for the plot.}
  \item{ylab}{\code{ylab} for the plot.}  
  \item{logrisk}{\code{logrisk} if we want to compute risk or logrisk
    estimates. By default this is TRUE, which has a better behaviour
    under small sample sizes.}  
  \item{\dots}{other arguments that will be passed to plot.}
}
\author{
David Rossell.
}
\examples{
#load eset
data(eset)

#make plot
smoothCoxph(pData(eset)$Months2Relapse,pData(eset)$Relapse,exprs(eset)[25,])
}
\keyword{datasets}

\name{heatmapPhenoTest-methods}
\docType{methods}
\alias{heatmapPhenoTest-methods}
\alias{heatmapPhenoTest,ExpressionSet,character-method}
\alias{heatmapPhenoTest,ExpressionSet,list-method}
\alias{heatmapPhenoTest,ExpressionSet,missing-method}
\alias{heatmapPhenoTest,ExpressionSet,GeneSet-method}
\alias{heatmapPhenoTest,ExpressionSet,GeneSetCollection-method}
\title{ Methods for Function heatmapPhenoTest in Package `phenoTest'}
\description{
Methods for function \code{heatmapPhenoTest} in Package `phenoTest'. For
more information read the function's manual.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "ExpressionSet", signatures = "character")}}{
Method for an \code{ExpressionSet} object and one signature stored in an
object of class \code{character}.
}

\item{\code{signature(x = "ExpressionSet", signatures = "list")}}{
Method for an \code{ExpressionSet} object and several signatures stored
in an object of class \code{list}.
}

\item{\code{signature(x = "ExpressionSet", signatures = "missing")}}{
Method for an \code{ExpressionSet} object and no signatures.
}

\item{\code{signature(x = "ExpressionSet", signatures = "GeneSet")}}{
Method for an \code{ExpressionSet} object and one signature stored in an
object of class \code{GeneSet}.
}

\item{\code{signature(x = "ExpressionSet", signatures = "GeneSetCollection")}}{
Method for an \code{ExpressionSet} object and several signatures stored
in an object of class \code{GeneSetCollection}.
}
}}
\keyword{methods}

\name{gseaSignificanceSign-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaSignificanceSign-class}

\title{Class "gseaSignificanceSign"}
\description{This object contains the results of the test of enrichment
that was performed on each gene set.
There is one container for every gene signature.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaSignificanceSign", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"} .}
    \item{\code{gseaSignificance}:}{Object of class \code{"matrix"}
      Contains the statistics. Use the \code{summary} method to access
      this information.}
    \item{\code{p.adjust.method}:}{Object of class \code{"character"}
      The p-value adjustment method that was used.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
No methods defined with class "gseaSignificanceSign" in the signature.
}
\author{Evarist Planet}
\examples{
showClass("gseaSignificanceSign")
}
\keyword{classes}

\name{gseaSignatures-methods}
\docType{methods}
\alias{gseaSignatures-methods}
\alias{gseaSignatures,ANY,character-method}
\alias{gseaSignatures,ANY,GeneSet-method}
\alias{gseaSignatures,epheno,list-method}
\alias{gseaSignatures,matrix,list-method}
\alias{gseaSignatures,numeric,list-method}
\alias{gseaSignatures,matrix,GeneSetCollection-method}
\alias{gseaSignatures,epheno,GeneSetCollection-method}
\alias{gseaSignatures,numeric,GeneSetCollection-method}
\title{Methods for Function gseaSignatures in Package `phenoTest'}
\description{
Methods for function \code{gseaSignatures} in Package `phenoTest'. For
more information read the function's manual.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "ANY", signatures = "character")}}{
Method for signature of class \code{character}.
}

\item{\code{signature(x = "ANY", signatures = "GeneSet")}}{
Method for signature of class \code{character}.
}

\item{\code{signature(x = "epheno", signatures = "list")}}{
Method for an \code{epheno} object and several signatures stored in an
object of class \code{list}.
}

\item{\code{signature(x = "matrix", signatures = "GeneSetCollection")}}{
Method for an \code{matrix} object and several signatures stored in an
object of class \code{GeneSetCollection}.
}

\item{\code{signature(x = "epheno", signatures = "GeneSetCollection")}}{
Method for an \code{epheno} object and several signatures stored in an
object of class \code{GeneSetCollection}.
}

\item{\code{signature(x = "numeric", signatures = "GeneSetCollection")}}{
Method for an \code{numeric} object and several signatures stored in an
object of class \code{GeneSetCollection}.
}

\item{\code{signature(x = "matrix", signatures = "list")}}{
Method for an \code{matrix} object and several signatures stored in an
object of class \code{list}.
}

\item{\code{signature(x = "numeric", signatures = "list")}}{
Method for an \code{numeric} object and several signatures stored in an
object of class \code{list}.
}
}}
\keyword{methods}

\name{barplotSignatures-methods}
\docType{methods}
\alias{barplotSignatures-methods}
\alias{barplotSignatures,epheno,character-method}
\alias{barplotSignatures,epheno,GeneSetCollection-method}
\alias{barplotSignatures,epheno,GeneSet-method}
\alias{barplotSignatures,epheno,list-method}
\title{Methods for Function barplotSignatures in Package `phenoTest'}
\description{
Methods for function \code{barplotSignatures} in Package
`phenoTest'. For more information read the function's manual.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "epheno", signatures = "character")}}{
Method for an \code{epheno} object and one signature stored in an object
of class \code{character}.
}

\item{\code{signature(x = "epheno", signatures = "GeneSetCollection")}}{  
Method for an \code{epheno} object and several signatures stored in an
object of class \code{GeneSetCollection}.
}

\item{\code{signature(x = "epheno", signatures = "GeneSet")}}{        
Method for an \code{epheno} object and one signature stored in an object
of class \code{GeneSet}.
}

\item{\code{signature(x = "epheno", signatures = "list")}}{
Method for an \code{epheno} object and several signatures stored in an
object of class \code{list}.
}

}}
\keyword{methods}

\name{gsea}
\alias{gsea}
\docType{data}
\title{
GSEA (Gene Set Enrichment Analysis).}
\description{
Computes the enrichment scores and simulated enrichment scores for
each variable and signature.
An important parameter of the function is \code{logScale}. Its default
value is TRUE which means that by default the provided scores (i.e. fold
changes, hazard ratios) will be log scaled. Remember to change this
parameter to FALSE if your scores are already log scaled.
The \code{getEs}, \code{getEsSim}, \code{getFc}, \code{getHr} and
\code{getFcHr} methods can be used to acces each subobject. For more
information please visit the man pages of each method.

It also computes the NES (normalized enrichment score), p values and fdr
(false discovery rate) for all variables and signatures.
For an overview of the output use the \code{summary} method.

In case of providing gene sets which have more than 10 distinct lengths
an approximation of the calculation of the enrichment score simulations
(ESM) will be computed.
The value of the ESM only depends on the length of the gene
set. Therefore we compute the ESM over a grid of possible gene set
lengths which are representative of the lengths of the provided gene
sets. Then we fit a generalized additive model model with cubic splines
to predict the NES value based on the length of every gene set. This
provides a much faster approach that can be very useful when we need to
run the software over a huge number of gene sets.
}
\usage{
gsea(x,gsets,logScale=TRUE, absVals=FALSE, averageRepeats=FALSE, B=1000,
     mc.cores=1, test="perm",p.adjust.method="none",
     pval.comp.method="original",pval.smooth.tail=TRUE,minGenes=10,
     maxGenes=500,center=FALSE) 
}
\arguments{
\item{x}{ \code{ePhenoTest}, \code{numeric} or \code{matrix} object
  containing scores (hazard ratios or fold changes).}
\item{gsets}{ character or list object containing the names of the
  genes that belong to each signature.}
\item{logScale}{ if values should be log scaled.}
\item{absVals}{ if TRUE fold changes and hazard ratios that are negative
  will be turned into positive before starting the process. This is
  useful when genes can go in both directions.}
\item{averageRepeats}{ if x is of class numeric and has repeated names
  (several measures for some indivdual names) we can average the
  measures of the same names.}
\item{B}{ number of simulations to perform.}
\item{mc.cores}{ number of processors to use.}
\item{test}{ the test that will be used. 'perm' stands for the permutation
  based method, 'wilcox' stands for the wilcoxon test (this is the fastest
  one) and 'ttperm' stands for permutation t test.}
\item{p.adjust.method}{ p adjustment method to be used. Common options
  are 'BH', 'BY', 'bonferroni' or 'none'. All available options and
  their explanations can be found on the \code{p.adjust} function manual.}
\item{pval.comp.method}{ the p value computation method. Has to be one
  of 'signed' or 'original'. The default one is 'original'. See details for
  more information.}
\item{pval.smooth.tail}{ if we want to estimate the tail of the
  ditribution where the pvalues will be generated.}
\item{minGenes}{ gene sets with less than minGenes genes will be removed
  from the analysis.}
\item{maxGenes}{ gene sets with more than maxGenes genes will be removed
  from the analysis.}
\item{center}{ if we want to center scores (fold changes or hazard
  ratios). The following is will be done: x = x-mean(x).}
}
\details{
The following preprocessing was done on the provided scores (i.e. fold
changes, hazard ratios) to avoid errors during the enrichment score
computation:
-When having two scores with the same name its average was used.
-Zeros were removed.
-Scores without names (which can not be in any signature) removed. 
-Non complete cases (i.e. NAs, NaNs) were removed.
ES score was calculated for each signature and variable (see
references). If parameter \code{test} is 'perm' the signature was 
permutted and the ES score was recalculated (this happened B times for
each variable, 1000 by default).
If \code{test} is 'wilcox' a wilcoxon test in which we test the fact
that the average value of the genes that do belong to our signtaure is
different from the average value of the genes that do not belong to our
signature will be performed.
If \code{test} is 'ttperm' a permutation t-test will be used.
Take into account that the final plot will be different when 'wilcox' is used.

The simulated enrichment scores and the calculated one
are used to find the p value.
 
P value calculation depends on the parameter
\code{pval.comp.method}. The default value is 'original'. In 'original' 
we are simply computing the proportion of anbolute simulated ES which
are larger than the observed absolute ES. In 'signed' we are computing
the proportion of simulated ES which are larger than the observed ES (in
case of having positive enrichment score) and the proportion of
simulated ES which are smaller than the observed ES (in case of having
negative enrichment score). 
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment
  Analysis}.
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}

C.A. Tsai and J.J. Chen. \emph{Kernel estimation for adjusted p-values
  in multiple testing. Computational Statistics & Data Analysis}
\url{http://econpapers.repec.org/article/eeecsdana/v_3a51_3ay_3a2007_3ai_3a8_3ap_3a3885-3897.htm}
}
\keyword{datasets}
\seealso{
gsea.go
}
\examples{
#load epheno object
data(epheno)
epheno

#we construct two signatures
sign1 <- sample(featureNames(epheno))[1:20]
sign2 <- sample(featureNames(epheno))[50:75]
mySignature <- list(sign1,sign2)
names(mySignature) <- c('My first signature','My preferred signature')

#run gsea functions
gseaData <- gsea(x=epheno,gsets=mySignature,B=100,mc.cores=1)
my.summary <- summary(gseaData)
my.summary 
#plot(gseaData)
}
\author{
Evarist Planet
}

\name{gseaSignificanceVar-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaSignificanceVar-class}

\title{Class "gseaSignificanceVar"}
\description{This object contains the results of the test of enrichment
that was performed on each gene set and phenotype.
There is one container for every phenotype. Every one of this containers
(of class \code{gseaSignificanceSign}) is a container itsef and has the
results of the tests for all signatures.
\code{GseaSignificanceVar} contains one element per phenotype (phenotypic
variable). Every one of this elements is of class
\code{gseaSignificanceSign} and contains one element per signature.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaSignificanceVar", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"} .}
    \item{\code{gseaSignificance}:}{Object of class
      \code{"gseaSignificanceSign"} This object contains the results of
      the tests. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
No methods defined with class "gseaSignificanceVar" in the signature.
}
\author{Evarist Planet}
\examples{
showClass("gseaSignificanceVar")
}
\keyword{classes}

\name{gseaSignificance}
\alias{gseaSignificance}
\docType{data}
\title{
ES' (enrichment scores) sifgnificance.
}
\description{
This function has been deprecated. You could better use \code{gsea}
instead.
  
This function performs the second step in the process of obtaining a
GSEA-like plot.
It computes the NES (normalized enrichment score), p values and fdr
(false discovery rate) for all variables and signatures. A
\code{gseaSignaturesSign} or \code{gseaSignaturesVar} object will 
be needed as input (these objects can be obtained with the
\code{gseaSignatures} function).
For an overview of the output use the \code{summary} method.
The next step after using the \code{gseaSignificance} function would be using
the \code{plot} method.
}
\usage{
gseaSignificance(x,p.adjust.method='none',pval.comp.method='original',pval.smooth.tail=TRUE)
}
\arguments{
\item{x}{ \code{gseaSignaturesSign} or \code{gseaSignaturesVar} object obtained
  with the \code{gseaSignatures} method. This object contains the enrichment scores
  ,the simulated enrichment scores and the fold changes or hazard ratios.}
\item{p.adjust.method}{ p adjustment method to be used. Common options
  are 'BH', 'BY', 'bonferroni' or 'none'. All available options and
  their explanations can be found on the \code{p.adjust} function manual.}
\item{pval.comp.method}{ the p value computation method. Has to be one
  of 'signed' or 'original'. The default one is 'original'. See details for
  more information.}
\item{pval.smooth.tail}{ if we want to estimate the tail of the
  ditribution where the pvalues will be generated.}
}
\details{
The simulated enrichment scores and the calculated one
are used to find the p value.
  
P value calculation depends on the parameter
\code{pval.comp.method}. The default value is 'original'. In 'original'
we are simply computing the proportion of anbolute simulated ES which
are larger than the observed absolute ES. In 'signed' we are computing
the proportion of simulated ES which are larger than the observed ES (in
case of having positive enrichment score) and the proportion of
simulated ES which are smaller than the observed ES (in case of having
negative enrichment score). 
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment
  Analysis}.
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}

C.A. Tsai and J.J. Chen. \emph{Kernel estimation for adjusted p-values
  in multiple testing. Computational Statistics & Data Analysis}
\url{http://econpapers.repec.org/article/eeecsdana/v_3a51_3ay_3a2007_3ai_3a8_3ap_3a3885-3897.htm}
}
\keyword{datasets}
\examples{
#for examples see the help file of gseaSigntaures: ?gseaSignatures
}
\author{
Evarist Planet
}

\name{heatmapPhenoTest}
\alias{heatmapPhenoTest}
\title{
Produce heatmap from phenotype data.
}
\description{
Show the associations between clusters that each sample belongs to and
each phenotype in a heatmap and/or a Kaplan-Meier plot.
}
\usage{
heatmapPhenoTest(x, signatures, vars2test, probes2genes = FALSE,
filterVar, filteralpha = 0.05, distCol = "pearson", nClust = 2, distRow
= "cor", p.adjust.method = "none", simulate.p.value = FALSE, B = 10^5,
linkage = "average", equalize = FALSE, center = TRUE, col, survCol,
heat.kaplan="both", ...)
}
\arguments{
  \item{x}{ExpressionSet with phenotype information stored in \code{pData(x)}.}
  \item{signatures}{Either character vector or list of character vectors
  with gene sets to be used to draw heatmaps (gene names should match
  those in \code{featureNames(x)}). A separate heatmap will be produced
  for each element in the list.}
  \item{vars2test}{ list with components 'continuous', 'categorical',
  'ordinal' and 'survival' indicating which phenotype variables should
  be tested. 'continuous', 'categorical' and 'ordinal' must be character
  vectors, 'survival' a matrix with columns named 'time' and
  'event'. The names must match names in \code{names(pData(x))}.}
  \item{probes2genes}{If set to \code{TRUE} a single probe is selected for
  each gene. \code{nsFilter} is used to select the probe with highest
  inter-quartile range.}
  \item{filterVar}{If specified, only genes with significant differences
  in the variable \code{filterVar} will be displayed in the
  heatmap. Note that this option will not affec the sample clustering,
  as this is obtained using both significant and non-significant genes.}
  \item{filteralpha}{Significance level for the filtering based on \code{filterVar}.}
  \item{distCol}{Distance metric used to cluster columns
  (e.g. patients/samples). Can take any value accepted by
  \code{dist}. Pearson and Spearman correlations are also 
  allowed. Write 'spearman' or 'pearson' to use them.}
  \item{nClust}{Number of desired clusters.}
  \item{distRow}{Distance metric used to cluster rows (e.g. genes). Can
  take any value accepted by \code{distancematrix}.}
  \item{p.adjust.method}{Method for P-value adjustment, passed on to
  \code{p.adjust}.}
  \item{simulate.p.value}{If set to FALSE the chi-square test p-values are
  computed using asymptotics, otherwise a simulation is used (see
  \code{chisq.test} for details).}
  \item{B}{An integer specifying the number of replicates used in the
  chi-square Monte Carlo test (passed on to \code{chisq.test}).}
  \item{linkage}{Linkage used for clustering. Must be either 'complete',
  'average' or 'minimum'.}
  \item{equalize}{Should color codes be equalized between genes, i.e. all
  genes present the same range of colors. Passed on to
  \code{heatmap_plus}.}
  \item{center}{centering is done by subtracting the column means
  (omitting NAs).}
  \item{col}{Color scheme to be used for heatmap. Defaults to a green/red
  scheme designed to look nice for microarray data.}
  \item{survCol}{Colors for the Kaplan-Meier survival curves.}
  \item{heat.kaplan}{can be "heat" if we want to plot a heatmap,
    "kaplan" if we want to plot a kaplan-meier or "both" if we want both
    of them.}
  \item{\dots}{Other arguments for the survival plot, e.g. lty etc.}
}
\details{
Makes two clusters of samples based on the expression levels of the
genes from the given signature and plots a heatmap and/or a Kaplan-Meier
showing the association  between belonging to one cluster or the other
and each phenotype.
  
For variables in vars2test\$continuous and vars2test\$ordinal a Kruskal-Wallis Rank Sum test is
used; for vars2test\$categorical a chi-square test (with exact p-value
if \code{simulate.p.value} is set to TRUE); for var2test\$survival a Cox proportional hazards likelihood-ratio test.
}
\examples{
#load data
data(eset)  
eset

#construct vars2test
survival <- matrix(c("Relapse","Months2Relapse"),ncol=2,byrow=TRUE)
colnames(survival) <- c('event','time')
vars2test <- list(survival=survival)
vars2test

#construct a signature
sign <- sample(featureNames(eset))[1:20]

#make plot
heatmapPhenoTest(eset,sign,vars2test=vars2test,heat.kaplan='heat')
heatmapPhenoTest(eset,sign,vars2test=vars2test,heat.kaplan='kaplan')
}
\author{
David Rossell
}

\name{gseaSignaturesVar-class}
\Rdversion{1.1}
\docType{class}
\alias{gseaSignaturesVar-class}
\alias{getEs,gseaSignaturesVar-method}
\alias{getEsSim,gseaSignaturesVar-method}
\alias{getFcHr,gseaSignaturesVar-method}
\alias{gseaSignificance,gseaSignaturesVar-method}

\title{Class "gseaSignaturesVar"}
\description{This class is an ES (enrichment score) and ES.sim (simulated enrichment
score) container that will be used in the GSEA (Gene Set Enrichment
Analysis) process.
There is one container for every phenotype. Every one of this containers
(of class \code{gseaSignaturesSign}) is a container itsef and has the
enrichment scores of all signatures.
\code{GseaSignaturesVar} contains one element per phenotype (phenotypic
variable). Every one of this elements is of class
\code{gseaSignaturesSign} and contains one element per signature.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("gseaSignaturesVar", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}:}{Object of class \code{"list"}. }
    \item{\code{gseaSignatures}:}{Object of class
      \code{"gseaSignaturesSign"} This object contains the enrichment
      scores and other elements that will be used in the GSEA process. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
Class \code{"\linkS4class{AssayData}"}, by class "list", distance 2.
}
\section{Methods}{
  \describe{
    \item{getEs}{\code{signature(x = "gseaSignaturesVar")}: Returns the
      enrichment scores.}
    \item{getEsSim}{\code{signature(x = "gseaSignaturesVar")}: Returns
      the simulated enrichemt scores (the ones obtained after
      permutations).} 
    \item{getFcHr}{\code{signature(x = "gseaSignaturesVar")}: Returns
      the fold change and/or the hazard ratio that were used to compute
      the enrichment scores.}
    \item{gseaSignificance}{\code{signature(x = "gseaSignaturesVar")}:
      This is the next step in the process of performing GSEA. This
      funtion will test if the gene sets are enriched. } 
	 }
}
\author{Evarist Planet}
\examples{
showClass("gseaSignaturesVar")
}
\keyword{classes}

\name{pAdjust-methods}
\docType{methods}
\alias{pAdjust-methods}
\alias{pAdjust,epheno-method}
\title{ Methods for Function pAdjust in Package `phenoTest'}
\description{
Methods for function \code{pAdjust} in Package `phenoTest'. This
function adjusts the p-values of an \code{epheno} object. For more
informe read the function's manual.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "epheno")}}{
Adjusts the pvalues of an epheno object.
}
}}
\author{
Evarist Planet
}
\keyword{methods}

\name{epheno}
\alias{epheno}
\docType{data}
\title{
epheno object.
}
\description{
Object obatined with ExpressionPhenoTest function.
}
\usage{data(epheno)}
\format{
  The format is:
Formal class 'epheno' [package "phenoTest"] with 8 slots
  ..@ p.adjust.method  : chr "none"
  ..@ assayData        :<environment: 0x1050d5a78> 
  ..@ phenoData        :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	5 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr [1:5] NA NA NA NA ...
  .. .. ..@ data             :'data.frame':	12 obs. of  5 variables:
  .. .. .. ..$ phenoName : Factor w/ 3 levels "lymph.node.status",..: 3 3 3 3 3 1 1 1 2 3 ...
  .. .. .. ..$ phenoClass: Factor w/ 3 levels "categorical",..: 2 2 2 2 2 1 1 1 3 2 ...
  .. .. .. ..$ phenoType : Factor w/ 3 levels "mean","pval",..: 1 1 1 3 3 1 1 3 3 2 ...
  .. .. .. ..$ meanLabel : Factor w/ 5 levels "[45.2,49.2)",..: 1 2 3 NA NA 4 5 NA NA NA ...
  .. .. .. ..$ survTime  : Factor w/ 1 level "Months2Relapse": NA NA NA NA NA NA NA NA 1 NA ...
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ featureData      :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	0 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr(0) 
  .. .. ..@ data             :'data.frame':	1000 obs. of  0 variables
  .. .. ..@ dimLabels        : chr [1:2] "featureNames" "featureColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ experimentData   :Formal class 'MIAME' [package "Biobase"] with 13 slots
  .. .. ..@ name             : chr ""
  .. .. ..@ lab              : chr ""
  .. .. ..@ contact          : chr ""
  .. .. ..@ title            : chr ""
  .. .. ..@ abstract         : chr ""
  .. .. ..@ url              : chr ""
  .. .. ..@ pubMedIds        : chr ""
  .. .. ..@ samples          : list()
  .. .. ..@ hybridizations   : list()
  .. .. ..@ normControls     : list()
  .. .. ..@ preprocessing    : list()
  .. .. ..@ other            : list()
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 0 0
  ..@ annotation       : chr "hgu133a"
  ..@ protocolData     :Formal class 'AnnotatedDataFrame' [package "Biobase"] with 4 slots
  .. .. ..@ varMetadata      :'data.frame':	0 obs. of  1 variable:
  .. .. .. ..$ labelDescription: chr(0) 
  .. .. ..@ data             :'data.frame':	12 obs. of  0 variables
  .. .. ..@ dimLabels        : chr [1:2] "sampleNames" "sampleColumns"
  .. .. ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. .. .. ..@ .Data:List of 1
  .. .. .. .. .. ..$ : int [1:3] 1 1 0
  ..@ .__classVersion__:Formal class 'Versions' [package "Biobase"] with 1 slots
  .. .. ..@ .Data:List of 4
  .. .. .. ..$ : int [1:3] 2 12 0
  .. .. .. ..$ : int [1:3] 2 10 0
  .. .. .. ..$ : int [1:3] 1 3 0
  .. .. .. ..$ : int [1:3] 1 0 0
}
\examples{
data(epheno)
## maybe str(epheno) ; plot(epheno) ...
}
\keyword{datasets}

\name{genesInArea}
\alias{genesInArea}
\title{
Find genes that are in given areas.
}
\description{
Combine the output of getEsPositions and findCopyNumber to see which
genes are in the enriched areas.

Given areas of enrichment (obtained with \code{findCopyNumber}) and a
set of genes or probes and their positions in the genome (obtained with
\code{getEsPositions}) the function tells which genes fall in each
area.
}
\usage{
genesInArea(x, regions)
}
\arguments{
  \item{x}{
An object of class \code{data.frame} with gene or probe identifiers as
row names and the following columns: es (the enrichment score), chr (the
chromosome where the gene or probe belong to) and pos (position in the
chromosome in megabases).
It can be obtained with the function \code{getEsPositions}.
}
  \item{regions}{
This is usually the output of \code{findCopyNumber} function.
}
}
\author{
Evarist Planet
}
\seealso{
getEsPositions, findCopyNumber
}
\examples{
data(epheno)
phenoNames(epheno)
mypos <- getEsPositions(epheno,'Relapse')
head(mypos)
#regions <- findCopyNumber(mypos)
#head(regions)
#genes <- genesInArea(mypos,regions)
#head(genes)
}
\keyword{ ~kwd1 }

\name{export2CSV}
\alias{export2CSV}
\docType{data}
\title{
Export object to comma-separated text file.
}
\description{
Saves object as comma-separated text file (CSV), using \code{write.csv}.
}
\usage{
export2CSV(x, file, row.names=FALSE, ...)
}
\arguments{
\item{x}{ object to be exported. Currently methods for objects of class epheno (produced with
  ExpressionPhenoTest function) are implemented.}
\item{file}{ Name of the file where the results are to be saved}
\item{row.names}{Passed on to \code{write.csv}}
\item{...}{Other arguments to be passed on to \code{write.csv}}
}
\keyword{datasets}

\name{epheno2html}
\alias{epheno2html}
\title{
Create html files and plots from an epheno object.
}
\description{
Creates html files and plots using an epheno object, which stores the
association between a list of variables and gene expression.
}
\usage{
epheno2html(x, epheno, outputdir, prefix = "", genelimit = 50, categories = 3, withPlots = TRUE, mc.cores = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{An object of class \code{ExpressionSet} (used to generate
    the epheno object) containing expression levels in exprs(x),
    phenotype information in pData(x) and annotation in annotation(x).}
  \item{epheno}{an object produced by ExpressionPhenoTest. this object
    will contain univariate association between a list of phenotype
    variables and gene expression as weel as p-values.}
  \item{outputdir}{where to place files.}
  \item{prefix}{will be used to add a text to the beginning of the files
    that will be created.}
  \item{genelimit}{maximum number of genes on the list.}
  \item{categories}{Number of categories used for continuous
    variables. It has to be the same as the one used for
    ExpressionPhenoTest.}
  \item{withPlots}{when FALSE no plots will be produced. Makes the
    process faster.}
  \item{mc.cores}{number of cores that will be used to run the process.}
}
\author{
Evarist Planet
}
\examples{
#Example on building homology tables for human.
#mart <- useMart("ensembl", "hsapiens_gene_ensembl")
#homol.symbol <- getLDS(attributes = c("entrezgene"), 
#    mart = mart, attributesL = c("external_gene_id"), 
#    martL = mart, filters = "entrezgene", values = entrezid)
#mart <- useMart("ensembl", "hsapiens_gene_ensembl")
#homol.genename <- getLDS(attributes = c("entrezgene"), 
#    mart = mart, attributesL = c("description"), martL = mart, 
#    filters = "entrezgene", values = entrezid)
}

\name{getVars2test}
\alias{getVars2test}
\docType{data}
\title{
Get phenotypic variables that were tested.
}
\description{
Returns an object containing the names of the variables that were tested
when the epheno object was created.
Will return an object of class list. Variables of the same type
(categorical, survival, etc) will be in the same slot of the list. The
slot names are the types of the variables.
}
\keyword{datasets}
\examples{
data(epheno)
getVars2test(epheno)
}
\author{
Evarist Planet
}

\name{plot.gseaSignatures}
\alias{plot.gseaSignaturesSign}
\alias{plot.gseaSignaturesVar}
\docType{data}
\title{
GSEA-like Plot.
}
\description{
Builds a GSEA plot using a gseaSignature object (one of
\code{gseaSignaturesSign} or \code{gseaSignaturesVar} obtained with the
\code{gseaSignatures} function) and a gseaSignificance object (one of
\code{gseaSignificanceSign} or \code{gseaSignificanceVar} obtained with
the \code{gseaSignificance} function).
}
\usage{
plot.gseaSignaturesSign(x,gseaSignificance,es.ylim,nes.ylim,es.nes="both",...)
}
\arguments{
\item{x}{ object of class \code{gseaSignaturesSign} or
  \code{gseaSignaturesVar}.}
\item{gseaSignificance}{ object of class \code{gseaSignificanceSign} or \code{gseaSignificanceVar}.}
\item{es.ylim}{ ylim values for the ES plot.}
\item{nes.ylim}{ ylim values for the NES plot.}
\item{es.nes}{ can be "es" if we want to plot enrichment score, "nes" if
  we want to plot normalised enrichment scores or "both"if we want to
  plot them both.}
\item{...}{ Arguments to be passed to \code{plot}.}
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment Analysis}. 
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}
}
\examples{
#for examples see the help file of gseaSigntaures: ?gseaSignatures
}
\seealso{
plot.gseaSignaturesSign, plot.gseaSignaturesVar
}
\author{
Evarist Planet
}
\keyword{datasets}

\name{export2CSV-methods}
\docType{methods}
\alias{export2CSV-methods}
\alias{export2CSV,epheno-method}
\title{ Methods for Function export2CSV in Package 'phenoTest'}
\description{
 Methods for function \code{export2CSV} in Package 'phenoTest'
}
\section{Methods}{
\describe{
\item{\code{signature(x = "epheno")}}{ Exports summary differences
  (fold changes, hazard ratios), p-values and gene annotation (when
  available) to a CSV (comma separated value) file }
}}
\keyword{methods}

\name{summary.gseaSignificance}
\alias{summary.gseaSignificanceSign}
\alias{summary.gseaSignificanceVar}
\docType{data}
\title{
Obtain a data.frame with the pvalues and fdr for all signatures and
variables of a gseaSignificanceSign or gseaSignificanceVar object.
}
\description{
Builds a data.frame object that can easily be written to a csv file
containing the ES, NES, pval.ES, pval.NES and FDR.
}
\usage{
summary.gseaSignificanceSign(object,...)
}
\arguments{
\item{object}{ object of class \code{gseaSignificanceSign} or \code{gseaSignificanceVar}.}
\item{...}{ Arguments to be passed to \code{summary}.}
}
\references{
Aravind Subramanian, (October 25, 2005) \emph{Gene Set Enrichment Analysis}. 
\url{www.pnas.org/cgi/doi/10.1073/pnas.0506580102}
}
\examples{
#for examples see the help file of gseaSigntaures: ?gseaSignatures
}
\author{
Evarist Planet
}
\keyword{datasets}

\name{barplotSignatures}
\alias{barplotSignatures}
\alias{barplotSignifSignatures}
\title{Summary plots for gene signature vs phenotype association}
\description{
Summarizes the univariate relationships between genes in one or more
signatures and several phenotype variables, as summarized in
\code{epheno} objects (which can be created with the
\code{ExpressionPhenoTest} function).

By default \code{barplotSignifSignatures} performs a binomial test
(\code{binom.test} from package \code{stats}) for each signature
to see if the number of up up regulated and down regulated genes is
different enough to be statistically different.
When a reference gene set is provided we test if the proportions of up
and down regulated genes of each gene set is different from the
proportions in the reference gene set. This has been done with a
chi-square test.
When a reference gene set is provided and parameter
\code{testUpDown} is \code{TRUE} (by default its \code{FALSE})
the number of genes corresponding to up and down regulated are
compared with those of the reference gene set separately.
}
\usage{
barplotSignatures(x, signatures, referenceSignature, alpha=.05,
p.adjust.method='none', ylab, cex.text=1, ...)
barplotSignifSignatures(x, signatures, referenceSignature, testUpDown=FALSE,
simulate.p.value = FALSE, B = 10^4, p.adjust.method='none', alpha=.05,
ylab, ylim=ylim, cex.text=1, ...)
}
\arguments{
\item{x}{ \code{epheno} object, as returned by
  \code{ExpressionPhenoTest}.} 
\item{signatures}{List with each element corresponding to a
  signature. The gene names in each signature must match those in
  \code{epheno}.}
\item{referenceSignature}{If specified, the average fold change in each
  signature is compared to the average fold change in the signature
  \code{referenceSignature}.}
\item{testUpDown}{If set to \code{TRUE}, bars corresponding to
  up and down-regulated genes are compared with those of
  \code{referenceSignature} separately. This argument is ignored if
  \code{referenceSignature} is not specified.}
\item{cex.text}{Character expansion for the text indicating the
  P-values. Ignored if referenceSignature is missing.}
\item{alpha}{Confidence levels for barplot error bars.}
\item{p.adjust.method}{P-value adjustment method, passed on to
  \code{p.adjust}.}
\item{simulate.p.value}{A logical indicating whether chi-square p-values
  should be computed by Monte Carlo simulation (passed on to \code{chisq.test}).}
\item{B}{Integer specifying the number of replicates in the Monte Carlo
  simulation (passed on to \code{chisq.test}).}
\item{ylab}{y-axis labels}
\item{ylim}{y-axis limits}
\item{...}{Other arguments to be passed on to boxplot.}
}
\value{
  When a single signature is provided as input, a single plot assessing the
  association of that signature with all phenotype variables is created.
  If several signatures are provided, one separate plot is created for
  each phenotype variable.
}
\examples{
#create epheno
data(epheno)

#construct two signatures
sign1 <- sample(featureNames(epheno))[1:20]
sign2 <- sample(featureNames(epheno))[1:15]
mySignature <- list(sign1,sign2)
names(mySignature) <- c('My first signature','My preferred signature')

#plot
barplotSignifSignatures(epheno[,'Relapse'],mySignature,alpha=0.05)
}
\author{
Evarist Planet
}
\keyword{graph}
